@@ -1,4 +1,4 @@
-# $Id: getperlflags.pl,v 1.2 2007/05/13 21:30:12 ekalin Exp $
+# $Id: getperlflags.pl 1014 2009-02-01 12:46:52Z ekalin $
 # The ExtUtils::Embed module returns, under Windows, compiler options
 # using the syntax used by Microsoft Compilers.
 # Since I compile KildClient for Windows with mingw32, a port of gcc and
@@ -12,12 +12,6 @@ use strict;
 sub kcccopts {
   my $cflags = ccopts();
 
-  if ($^O eq 'MSWin32') {
-    $cflags =~ s/-[^DI][a-zA-Z0-9_=]*//g;
-    $cflags =~ s/-I"([^"]+)"/-I$1/g;
-    $cflags =~ s#\\#/#g;
-  }
-
   print $cflags;
 }
 
@@ -26,19 +20,10 @@ sub kcldopts {
   my $ldflags = ldopts();
 
   if ($^O eq 'MSWin32') {
-    $ldflags =~ /-libpath:"([^"]+)"/;
-    my $libincdir = $1;
-
-    #$ldflags =~ / ([A-Za-z0-9_\\:.]+)\b/;
-    #my $libfile = $1;
-
-    my $winldflags = "-L$libincdir $libincdir/perl58.lib";
-    $winldflags =~ s#\\#/#g;
-
-    print $winldflags;
-  } else {
-    print $ldflags;
+    $ldflags =~ s#\\#/#g;
   }
+
+  print $ldflags;
 }
 
 
