@@ -1,6 +1,6 @@
-## $Id: kildclient.pl 1027 2009-02-12 18:27:11Z ekalin $
+## $Id: kildclient.pl 1152 2011-03-20 15:31:10Z ekalin $
 
-### Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+### Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -130,6 +130,38 @@ sub _ { return gettext($_[0]); }
   }
 
 
+  sub path {
+    my $self = shift;
+    my $path = $_[0];
+    if (!$path) {
+      return;
+    }
+
+    my ($num, $cmd, @cmdlist);
+
+    # Splits the string in groups formed by an optional number
+    # followed by the command and adds that to the list.
+    do {
+      $path =~ /(\d*)([[:alpha:]]|\{[^}]+\})/;
+      $num = $1;
+      $cmd = $2;
+
+      $path = substr($path, $+[0]);
+
+      $num = 1 if ($num eq '');
+      $cmd =~ s/\{(.+)\}/$1/;
+
+      my $i;
+      for ($i = 0; $i < $num; ++$i) {
+        push(@cmdlist, $cmd);
+      }
+    } until ($path eq '');
+
+    # Sends the commands
+    $self->sendlines(\@cmdlist);
+  }
+
+
   sub prev {
     my $self = shift;
     my $num = $_[0];
@@ -1105,33 +1137,6 @@ sub stripcolorize {
 }
 
 
-sub path {
-  my $path = $_[0];
-
-  if (!$path) {
-    return;
-  }
-
-  my ($num, $cmd);
-
-  do {
-    $path =~ /(\d*)([[:alpha:]]|\{[^}]+\})/;
-    $num = $1;
-    $cmd = $2;
-
-    $path = substr($path, $+[0]);
-
-    $num = 1 if ($num eq '');
-    $cmd =~ s/\{(.+)\}/$1/;
-
-    my $i;
-    for ($i = 0; $i < $num; ++$i) {
-      $::world->send("$cmd");
-    }
-  } until ($path eq '');
-}
-
-
 sub __sethelpfile {
   $__helpfile = $_[0];
 }
@@ -1177,3 +1182,8 @@ sub minimize {
   $::window->minimize();
 }
 
+sub path {
+  $::world->echonl(_('Warning: the path() function is deprecated and will be removed.'));
+  $::world->echonl(_('Use $world->path() instead.'));
+  $::world->path($_[0]);
+}
