@@ -1,7 +1,7 @@
-/* $Id: aliases.c 1020 2009-02-05 17:36:25Z ekalin $ */
+/* $Id: aliases.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -64,7 +63,8 @@ substitute_aliases(World *world, const char *cmd)
                                                     cmd,
                                                     alias->pattern_re,
                                                     alias->substitution,
-                                                    alias->perl_eval))) {
+                                                    alias->perl_eval,
+                                                    FALSE))) {
       if (debug_matches) {
         fprintf(stderr,
                 "**Alias %d '%s' matched for world %s\n"
@@ -98,6 +98,11 @@ remove_alias(World *world, GSList *aliasitem)
 {
   Alias *alias = (Alias *) aliasitem->data;
 
+  /* Necessary when free_alias dereferences alias->pattern_re */
+  if (world->perl_interpreter) {
+    PERL_SET_CONTEXT(world->perl_interpreter);
+  }
+
   we_alias_delete_alias(world, alias);
 
   world->aliases = g_slist_remove_link(world->aliases, aliasitem);
@@ -137,6 +142,7 @@ free_alias(Alias *alias, gpointer data)
   g_free(alias->name);
   g_free(alias->pattern);
   g_free(alias->substitution);
+  SvREFCNT_dec(alias->pattern_re);
 
   g_free(alias);
 }
@@ -145,7 +151,7 @@ free_alias(Alias *alias, gpointer data)
 void
 list_aliases(World *world, Plugin *plugin)
 {
-  int     i;
+  guint   i;
   guint   total_width;
   guint   field_width;
   GSList *aliasptr;
