@@ -1,7 +1,7 @@
-/* $Id: ansitextview.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: ansitextview.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kcircularqueue.h"
 
@@ -285,7 +284,7 @@ ansitextview_insert_ansi_string(WorldGUI    *gui,
    */
   static guchar text_buffer[MAX_BUFFER];
   gint          text_buffer_pos;
-  gint          original_pos;
+  guint         original_pos;
 
   text_buffer_pos = 0;
   original_pos    = 0;
@@ -391,8 +390,8 @@ ansitextview_append_echoed_string(WorldGUI    *gui,
   ANSIFSMState  saved_fsmstate;
 
   GtkTextIter end_iter;
-  gint        printed;
-  gint        to_print;
+  guint       printed;
+  guint       to_print;
 
 
   memcpy(saved_buffer, gui->ta.ansiseq_buffer, gui->ta.ansiseq_pos);
@@ -466,7 +465,8 @@ ansitextview_text_added(WorldGUI *gui)
 
   /* Scroll to end if appropriate */
   adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
-  if (adjustment->value != adjustment->upper - adjustment->page_size
+  if (gtk_adjustment_get_value(adjustment)
+          != gtk_adjustment_get_upper(adjustment) - gtk_adjustment_get_page_size(adjustment)
       && !(gui->world && gui->world->scrollOutput)) {
     return;
   }
@@ -614,7 +614,7 @@ ansitextview_verify_urls_and_add(WorldGUI     *gui,
   static gboolean   re_compiled = FALSE;
   static guchar     ctext[MAX_BUFFER + 1];
   static regmatch_t match;
-  int               search_start;
+  guint             search_start;
 
   if (gui->ta.state.linedraw) {
     ansitextview_insert_vt100linedraw_string(gui, pos, text, len);
@@ -672,7 +672,7 @@ ansitextview_insert_vt100linedraw_string(WorldGUI     *gui,
                                          const guchar *text,
                                          gsize         len)
 {
-  gint i;
+  guint i;
 
   worldgui_determine_supported_chars(gui);
 
@@ -1020,6 +1020,12 @@ ansitextview_update_color_tags(WorldGUI *gui, World *world)
   gtk_widget_modify_base(GTK_WIDGET(gui->txtView),
                          GTK_STATE_NORMAL,
                          world->defback);
+  gtk_widget_modify_base(GTK_WIDGET(gui->txtViewScroll),
+                         GTK_STATE_NORMAL,
+                         world->defback);
+  gtk_widget_modify_text(GTK_WIDGET(gui->txtViewScroll),
+                         GTK_STATE_NORMAL,
+                         world->deffore);
   gtk_widget_modify_text(GTK_WIDGET(gui->txtView),
                          GTK_STATE_NORMAL,
                          world->deffore);
@@ -1172,9 +1178,9 @@ get_tag_for_xterm256_color(WorldGUI *gui,
 
   if (x256idx <= 15) {   /* Highlighted ANSI colors */
     if (code_used == XTERM256_SET_FORE) {
-      return gui->ta.ansi_fore_tags[x256idx + ANSI_BOLD_BASE_IDX];
+      return gui->ta.ansi_fore_tags[x256idx - ANSI_N_COLORS + ANSI_BOLD_BASE_IDX];
     } else if (code_used == XTERM256_SET_BACK) {
-      return gui->ta.ansi_back_tags[x256idx + ANSI_BOLD_BASE_IDX];
+      return gui->ta.ansi_back_tags[x256idx - ANSI_N_COLORS + ANSI_BOLD_BASE_IDX];
     }
   }
 
