@@ -1,7 +1,7 @@
-/* $Id: cmdhistory.c 1029 2009-02-15 13:37:36Z ekalin $ */
+/* $Id: cmdhistory.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gmodule.h>
 
 #include "kildclient.h"
@@ -52,13 +51,13 @@ typedef struct CmdHistorySendData_s CmdHistorySendData;
  * Function prototypes *
  ***********************/
 static GtkWidget *create_cmd_history_dialog(GtkWindow *parent, World *world);
-static void       cmdhistory_recall_cb(GtkWidget *widget, gpointer data);
-static void       cmdhistory_start_send_cb(GtkWidget *widget, gpointer data);
 static gboolean   cmdhistory_send_timer_cb(gpointer data);
-static void       cmdhistory_find_cb(GtkWidget *widget, gpointer data);
 static void       save_tentative_command(World *world);
-/* Glade signals */
+/* XML UI signals */
 G_MODULE_EXPORT void menu_command_history_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void cmdhistory_recall_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void cmdhistory_start_send_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void cmdhistory_find_cb(GtkWidget *widget, gpointer data);
 
 
 void
@@ -185,28 +184,38 @@ static
 GtkWidget *
 create_cmd_history_dialog(GtkWindow *parent, World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *dlg;
+  gchar            *objects[] = { "dlgCmdHistory",
+                                  "adjustment1", "adjustment2",
+                                  NULL };
+  GError           *error = NULL;
+  GObject          *dlg;
   GtkTreeView      *viewCommands;
   GtkTreeSelection *selection;
   GtkCellRenderer  *renderer;
-  GtkWidget        *btnRecall;
-  GtkWidget        *btnRecallClose;
-  GtkWidget        *btnSend;
-  GtkWidget        *btnSendClose;
-  GtkWidget        *btnClose;
-  GtkWidget        *btnFind;
-  GtkWidget        *btnFindNext;
-  GtkWidget        *spnDelay;
-  GtkWidget        *spnNCommands;
+  GObject          *btnSendClose;
+  GObject          *btnRecallClose;
+  GObject          *btnFind;
+  GtkSpinButton    *spnDelay;
+  GtkSpinButton    *spnNCommands;
 
   /* Create the dialog */
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgCmdHistory", NULL);
-  dlg      = glade_xml_get_widget(gladexml, "dlgCmdHistory");
+  if (!world->ui_builder) {
+    world->ui_builder = gtk_builder_new();
+  }
+  if (!gtk_builder_add_objects_from_file(world->ui_builder,
+                                         get_kildclient_installed_file("dlgCmdHistory.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+  dlg = gtk_builder_get_object(world->ui_builder, "dlgCmdHistory");
 
   /* Configure tree view */
-  viewCommands = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCommands"));
+  viewCommands = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                      "viewCommands"));
   gtk_tree_view_set_model(viewCommands, world->cmd_list);
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_data_func(viewCommands,
@@ -219,54 +228,33 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
   selection = gtk_tree_view_get_selection(viewCommands);
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
-  /* Signals */
-  glade_xml_signal_autoconnect(gladexml);
-  btnClose = glade_xml_get_widget(gladexml, "btnCHClose");
-  g_signal_connect_swapped(G_OBJECT(btnClose), "clicked",
-                           G_CALLBACK(gtk_widget_hide), dlg);
-
-  btnSend = glade_xml_get_widget(gladexml, "btnCHSend");
-  g_signal_connect(G_OBJECT(btnSend), "clicked",
-                   G_CALLBACK(cmdhistory_start_send_cb), world);
-  btnSendClose = glade_xml_get_widget(gladexml, "btnSendClose");
-  g_object_set_data(G_OBJECT(btnSendClose), "close", GINT_TO_POINTER(1));
-  g_signal_connect(G_OBJECT(btnSendClose), "clicked",
-                   G_CALLBACK(cmdhistory_start_send_cb), world);
-
-  btnRecall = glade_xml_get_widget(gladexml, "btnRecall");
-  g_signal_connect(G_OBJECT(btnRecall), "clicked",
-                   G_CALLBACK(cmdhistory_recall_cb), world);
-  btnRecallClose = glade_xml_get_widget(gladexml, "btnRecallClose");
-  g_object_set_data(G_OBJECT(btnRecallClose), "close", GINT_TO_POINTER(1));
-  g_signal_connect(G_OBJECT(btnRecallClose), "clicked",
-                   G_CALLBACK(cmdhistory_recall_cb), world);
-
-  btnFind = glade_xml_get_widget(gladexml, "btnFind");
-  g_object_set_data(G_OBJECT(btnFind), "startsearch", GINT_TO_POINTER(1));
-  g_signal_connect(G_OBJECT(btnFind), "clicked",
-                   G_CALLBACK(cmdhistory_find_cb), world);
-  btnFindNext = glade_xml_get_widget(gladexml, "btnFindNext");
-  g_signal_connect(G_OBJECT(btnFindNext), "clicked",
-                   G_CALLBACK(cmdhistory_find_cb), world);
+  /* This information is used in the callbacks */
+  btnSendClose = gtk_builder_get_object(world->ui_builder, "btnSendClose");
+  g_object_set_data(btnSendClose, "close", GINT_TO_POINTER(1));
+  btnRecallClose = gtk_builder_get_object(world->ui_builder, "btnRecallClose");
+  g_object_set_data(btnRecallClose, "close", GINT_TO_POINTER(1));
+  btnFind = gtk_builder_get_object(world->ui_builder, "btnFind");
+  g_object_set_data(btnFind, "startsearch", GINT_TO_POINTER(1));
 
   /* Default values */
-  spnDelay = glade_xml_get_widget(gladexml, "spnCHDelay");
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnDelay),
-                            globalPrefs.multi_cmd_delay);
-  spnNCommands = glade_xml_get_widget(gladexml, "spnNCommands");
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnNCommands),
-                            globalPrefs.multi_cmd_group_size);
+  spnDelay = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                    "spnCHDelay"));
+  gtk_spin_button_set_value(spnDelay, globalPrefs.multi_cmd_delay);
+  spnNCommands = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                        "spnNCommands"));
+  gtk_spin_button_set_value(spnNCommands, globalPrefs.multi_cmd_group_size);
 
-  return dlg;
+  /* Signals */
+  gtk_builder_connect_signals(world->ui_builder, world);
+
+  return GTK_WIDGET(dlg);
 }
 
 
-static
 void
 cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
 {
   World            *world = (World *) data;
-  GladeXML         *gladexml;
   GtkTreeView      *viewCommands;
   GtkTreeModel     *model;
   GtkTreeSelection *selection;
@@ -274,8 +262,8 @@ cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
   gint              n_selected;
   gchar            *command;
 
-  gladexml = glade_get_widget_tree(widget);
-  viewCommands = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCommands"));
+  viewCommands = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                      "viewCommands"));
   selection = gtk_tree_view_get_selection(viewCommands);
 
   n_selected = gtk_tree_selection_count_selected_rows(selection);
@@ -322,17 +310,15 @@ cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
 }
 
 
-static
 void
 cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
 {
   World            *world = (World *) data;
-  GladeXML         *gladexml;
   GtkTreeView      *viewCommands;
   GtkTreeModel     *model;
   GtkTreeSelection *selection;
-  GtkWidget        *spnDelay;
-  GtkWidget        *spnNCommands;
+  GtkSpinButton    *spnDelay;
+  GtkSpinButton    *spnNCommands;
   GList            *selected_rows;
   GList            *list_iter;
   gdouble           delay;
@@ -341,20 +327,21 @@ cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
     gtk_widget_hide(world->dlgCmdHistory);
   }
 
-  gladexml = glade_get_widget_tree(widget);
-  viewCommands = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCommands"));
+  viewCommands = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                      "viewCommands"));
   selection = gtk_tree_view_get_selection(viewCommands);
 
   selected_rows = gtk_tree_selection_get_selected_rows(selection, &model);
   list_iter = selected_rows;
-
   if (!list_iter) {
     return;
   }
 
-  spnDelay = glade_xml_get_widget(gladexml, "spnCHDelay");
-  delay = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spnDelay));
-  spnNCommands = glade_xml_get_widget(gladexml, "spnNCommands");
+  spnDelay = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                    "spnCHDelay"));
+  delay = gtk_spin_button_get_value(spnDelay);
+  spnNCommands = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                        "spnNCommands"));
 
   if (delay) {
     gchar              *command;
@@ -362,7 +349,6 @@ cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
     GSList             *cmd_list = NULL;
     CmdHistorySendData *data;
 
-
     /* Create list of commands */
     while (list_iter) {
       gtk_tree_model_get_iter(model, &iter, (GtkTreePath *) list_iter->data);
@@ -376,7 +362,7 @@ cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
     data->list      = cmd_list;
     data->list_iter = cmd_list;
     data->lines_at_a_time
-      = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spnNCommands));
+      = gtk_spin_button_get_value_as_int(spnNCommands);
 
     /* Send first command immediately */
     if (cmdhistory_send_timer_cb(data)) {
@@ -436,43 +422,57 @@ cmdhistory_send_timer_cb(gpointer data)
 }
 
 
-static
 void
 cmdhistory_find_cb(GtkWidget *widget, gpointer data)
 {
   World            *world = (World *) data;
-  GladeXML         *gladexml;
   GtkTreeView      *viewCommands;
   GtkTreeModel     *model;
   GtkTreeSelection *selection;
-  GladeXML         *gladexmlfind;
-  GtkWidget        *dlgFind;
-  GtkWidget        *txtSearchText;
-  GtkWidget        *radUpwards;
   gboolean          found;
   gboolean          error;
   GtkTreePath      *search_path = NULL;
 
-  gladexml = glade_get_widget_tree(widget);
-  viewCommands = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCommands"));
+  viewCommands = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                      "viewCommands"));
   model = gtk_tree_view_get_model(viewCommands);
   selection = gtk_tree_view_get_selection(viewCommands);
 
   /* Start search if asked */
   if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "startsearch"))) {
-    gladexmlfind = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                                 "dlgFind", NULL);
-    dlgFind       = glade_xml_get_widget(gladexmlfind, "dlgFind");
-    txtSearchText = glade_xml_get_widget(gladexmlfind, "txtSearchText");
-    radUpwards    = glade_xml_get_widget(gladexmlfind, "radUpwards");
+    GtkEntry        *txtSearchText;
+    GtkToggleButton *radUpwards;
 
-    gtk_dialog_set_default_response(GTK_DIALOG(dlgFind), GTK_RESPONSE_OK);
+    if (!world->dlgCmdHistoryFind) {
+      gchar  *objects[] = { "dlgFind", NULL };
+      GError *gerror    = NULL;
+
+      if (!gtk_builder_add_objects_from_file(world->ui_builder,
+                                             get_kildclient_installed_file("dlgCmdHistory.ui"),
+                                             objects,
+                                             &gerror)) {
+        g_warning(_("Error loading UI from XML file: %s"), gerror->message);
+        g_error_free(gerror);
+        return;
+      }
+
+      world->dlgCmdHistoryFind
+        = GTK_WIDGET(gtk_builder_get_object(world->ui_builder, "dlgFind"));
+    }
+
+    txtSearchText = GTK_ENTRY(gtk_builder_get_object(world->ui_builder,
+                                                   "txtSearchText"));
+    radUpwards    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                             "radUpwards"));
+
+    gtk_dialog_set_default_response(GTK_DIALOG(world->dlgCmdHistoryFind),
+                                    GTK_RESPONSE_OK);
 
     while (1) {
-      if (gtk_dialog_run(GTK_DIALOG(dlgFind)) == GTK_RESPONSE_OK) {
+      if (gtk_dialog_run(GTK_DIALOG(world->dlgCmdHistoryFind)) == GTK_RESPONSE_OK) {
         const gchar *search_text;
 
-        search_text = gtk_entry_get_text(GTK_ENTRY(txtSearchText));
+        search_text = gtk_entry_get_text(txtSearchText);
         if (strcmp(search_text, "") == 0) {
           GtkWidget *msgdlg;
 
@@ -487,17 +487,20 @@ cmdhistory_find_cb(GtkWidget *widget, gpointer data)
           continue;
         }
 
-        world->cmdfind_upwards = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radUpwards));
+        world->cmdfind_upwards = gtk_toggle_button_get_active(radUpwards);
         g_free(world->cmdfind_string);
         world->cmdfind_string = g_strdup(search_text);
         break;
       } else {
-        gtk_widget_destroy(dlgFind);
+        /* Clear dialog */
+        gtk_entry_set_text(txtSearchText, "");
+        gtk_widget_hide(world->dlgCmdHistoryFind);
         return;
       }
     }
-    gtk_widget_destroy(dlgFind);
-    g_object_unref(gladexmlfind);
+    /* Clear dialog */
+    gtk_entry_set_text(txtSearchText, "");
+    gtk_widget_hide(world->dlgCmdHistoryFind);
 
     if (world->cmdfind_row) {
       gtk_tree_row_reference_free(world->cmdfind_row);
