@@ -1,7 +1,7 @@
-/* $Id: hooks.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: hooks.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -154,7 +153,7 @@ list_hooks(World *world, const gchar *hookname, Plugin *plugin)
   Hook   *hook;
   guint   total_width;
   guint   field_width;
-  int     i;
+  guint   i;
 
   hooklist = get_hook_list_for_reading(world, hookname, NULL);
   if (!hooklist) {
@@ -227,10 +226,13 @@ list_hook(World *world, const gchar *hookname, int pos)
 
 
 gboolean
-delete_hook(World *world, const gchar *hookname, int pos)
+delete_hook(World *world, const gchar *hookname, int pos, GSList *hook)
 {
+  /* Can receive either the list item poiting to the hook itself
+   * (in this case, pos is ignored),
+   * or, if hook is NULL, the position of the hook to delete.
+   */
   GSList **hooklist;
-  GSList  *hook;
   gint    *hook_pos;
 
   hooklist = get_hook_list_for_writing(world, hookname, &hook_pos);
@@ -238,7 +240,9 @@ delete_hook(World *world, const gchar *hookname, int pos)
     return FALSE;
   }
 
-  hook = g_slist_nth(*hooklist, pos);
+  if (hook == NULL) {
+    hook = g_slist_nth(*hooklist, pos);
+  }
   if (!hook) {
     return FALSE;
   }
