@@ -1,7 +1,7 @@
-/* $Id: kildclient.h 1027 2009-02-12 18:27:11Z ekalin $ */
+/* $Id: kildclient.h 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@
 /**********************
  * Limits and defines *
  **********************/
-#define MAX_BUFFER   1024
+#define MAX_BUFFER   4096
 #define MAX_TIME_STR 150
 #define VTE_ENC      "UTF-8"
 #define DEFAULT_TERMINAL_FONT "Monospace 10"
@@ -283,9 +283,14 @@ struct WorldGUI_s
 {
   struct world_s    *world;
   GtkWidget         *vbox;
+  GtkPaned          *split_pane;
+  gint               last_split_position;
   GtkScrolledWindow *scrolled_win;
   GtkTextView       *txtView;
+  GtkScrolledWindow *scrwinScroll;
+  GtkTextView       *txtViewScroll;
   GtkTextBuffer     *txtBuffer;
+  gdouble            last_scroll_info; /* For scrolled_win_size_allocate_cb() */
 
   GtkBox            *hboxTab;
   GtkLabel          *lblNotebook;
@@ -317,15 +322,8 @@ struct WorldGUI_s
 
   guint              prune_timeout_id;
 
-  /* Tooltips */
+  /* Line times for tooltips */
   KCircularQueue    *line_times;
-  guint              tooltip_timeout_id;
-  gint               tooltip_line_y;
-  gint               tooltip_line_height;
-  gint               tooltip_line_number;
-  gint               tooltip_x;
-  gint               tooltip_y;
-  GtkWidget         *tooltip_window;
 
   /* Determines the appearance of the text */
   TextAppearance     ta;
@@ -347,6 +345,7 @@ struct world_s
   gchar           *port;
   proxy_settings   proxy;
   proxy_settings   proxy_used;
+  gboolean         keep_alive;
 
   /* Auto-login data */
   ConnStyle            connection_style;
@@ -359,6 +358,9 @@ struct world_s
   char             *statusfont;
   gboolean          usebold;
   gboolean          wrap;
+  gint              wrap_indent;
+  gboolean          use_wrap_column;
+  gint              wrap_column;
   GdkColor         *deffore;
   GdkColor         *defbold;
   GdkColor         *defback;
@@ -366,12 +368,14 @@ struct world_s
   GdkColor         *ansicolors;
   NameDisplayStyle  name_display_style;
   char             *name_display_pattern;
+  gboolean          use_tooltips;
 
   /* Scripting support */
   gchar           *scriptfile;
   GSList          *triggers;
   gint             trigger_pos;
   gboolean         disable_triggers;
+  SV              *trigger_action_re; /* Pattern to replace $n in action */
   GSList          *aliases;
   gint             alias_pos;
   gboolean         disable_aliases;
@@ -407,6 +411,7 @@ struct world_s
   gint             autocompl_minprefix;
   gchar            command_separator[10];
   gboolean         updating_controls;     /* To prevent infinite loop */
+  gboolean         ignore_up_down_keys;
 
   /* Command history support */
   GtkTreeModel    *cmd_list;
@@ -462,13 +467,13 @@ struct world_s
   guchar           dbuffer[MAX_BUFFER];      /* Buffer for decompressed data */
   guchar          *pbuffer;                  /* Buffer to process */
   guint            pbuf_alloc_size;          /* Current size of pbuffer */
-  int              pbuf_size;                /* Buffer size */
+  guint            pbuf_size;                /* Buffer size */
   /* For MCCP handling */
   z_stream        *zstream;                  /* Used for decompression */
   gint             mccp_ver;                 /* Version of MCCP supported */
 
   /* Time counts */
-  int              printed_until;
+  guint            printed_until;
   GTimer          *last_data_written;
 
   /* For connection and idle time display */
@@ -496,6 +501,9 @@ struct world_s
   /* GUI structure */
   WorldGUI        *gui;
 
+  /* UI Builder for the dialogs */
+  GtkBuilder      *ui_builder;
+
   /* World Editor dialog and related objects */
   GtkWidget          *dlgEditWorld;
   GtkTreeModel       *plugin_model;
@@ -531,6 +539,7 @@ struct world_s
   gboolean             cmdfind_upwards;
   gchar               *cmdfind_string;
   GtkTreeRowReference *cmdfind_row;
+  GtkWidget           *dlgCmdHistoryFind;
 
   /* Test Triggers dialog */
   GtkWidget *dlgTestTriggers;
@@ -542,6 +551,7 @@ struct world_s
   gboolean       use_tls;
 #ifdef HAVE_LIBGNUTLS
   gnutls_session tls_session;
+  int            tls_handshake_dir;
 #endif
 };
 typedef struct world_s World;
@@ -766,23 +776,24 @@ enum {
 /********************
  * Global variables *
  ********************/
-extern GtkWidget *wndMain;              /* The main window widget */
-extern GtkWidget *ntbkWorlds;           /* Notebook with the worlds */
-extern GList     *open_worlds;          /* List of open worlds */
-extern World     *currentWorld;         /* The current world */
-extern World     *world_for_perl;       /* The world for Perl to use */
-extern World     *default_world;        /* World with default settings */
-extern gboolean   window_has_focus;     /* Whether the main window has focus */
-extern gint       worlds_with_new_text; /* # of worlds with unread text */
-extern gboolean   debug_matches;        /* Show when a trigger/alias matches */
-extern Prefs      globalPrefs;          /* Holds the global preferences */
-extern FILE      *dumpFile;             /* File to dump decompr. data*/
-extern FILE      *rdumpFile;            /* File to dump raw received data */
-extern GdkColor   deffore;              /* Default foreground color */
-extern GdkColor   defbold;              /* Default bold foreground color */
-extern GdkColor   defback;              /* Default background color */
-extern GdkColor   defboldback;          /* Default bold background color */
-extern GdkColor   defansi[];            /* Default ansi colors */
+extern GtkWidget  *wndMain;              /* The main window widget */
+extern GtkBuilder *main_builder;        /* Created main window & others */
+extern GtkWidget  *ntbkWorlds;           /* Notebook with the worlds */
+extern GList      *open_worlds;          /* List of open worlds */
+extern World      *currentWorld;         /* The current world */
+extern World      *world_for_perl;       /* The world for Perl to use */
+extern World      *default_world;        /* World with default settings */
+extern gboolean    window_has_focus;     /* Whether the main window has focus */
+extern gint        worlds_with_new_text; /* # of worlds with unread text */
+extern gboolean    debug_matches;        /* Show when a trigger/alias matches */
+extern Prefs       globalPrefs;          /* Holds the global preferences */
+extern FILE       *dumpFile;             /* File to dump decompr. data*/
+extern FILE       *rdumpFile;            /* File to dump raw received data */
+extern GdkColor    deffore;              /* Default foreground color */
+extern GdkColor    defbold;              /* Default bold foreground color */
+extern GdkColor    defback;              /* Default background color */
+extern GdkColor    defboldback;          /* Default bold background color */
+extern GdkColor    defansi[];            /* Default ansi colors */
 #ifdef HAVE_LIBGNUTLS
 /* gnutls certificate credentials */
 extern gnutls_certificate_credentials tls_certif_cred;
@@ -794,7 +805,6 @@ extern gnutls_certificate_credentials tls_certif_cred;
  *********************************/
 GtkWidget* create_main_window(WorldGUI *gui);
 World*     get_world_to_connect();
-void       show_preferences_dialog_cb(GtkWidget *widget, gpointer data);
 void       remove_world(World *world, gboolean remove_gui);
 gboolean   remove_world_timer(World *world);
 void       set_focused_world(gint page);
@@ -802,7 +812,7 @@ gint       get_focused_world(void);
 void       set_notebook_tab_position(GtkPositionType pos);
 void       adjust_window_title(void);
 gboolean   open_new_world(WorldGUI    *gui,
-                          const gchar *name,
+                          const gchar *cmdline,
                           gboolean     can_offline,
                           gboolean    *offline_selected);
 gboolean   update_times_cb(gpointer data);
@@ -829,7 +839,6 @@ void      free_world_gui(WorldGUI *gui);
 void      configure_gui(WorldGUI *gui, World *world);
 void      worldgui_configure_spell(WorldGUI *gui);
 void      clear_button_cb(GtkButton *button, gpointer data);
-void      menu_url_open(GtkWidget *widget, gchar *url);
 void      send_naws_size(World *world);
 void      completion_cell_data_function(GtkCellLayout   *cell_layout,
                                         GtkCellRenderer *renderer,
@@ -837,6 +846,7 @@ void      completion_cell_data_function(GtkCellLayout   *cell_layout,
                                         GtkTreeIter     *iter,
                                         gpointer         data);
 void      find_next_cb(GtkButton *button, gpointer data);
+G_MODULE_EXPORT void menu_url_open(GtkAction *action, gchar *url);
 
 
 /***************************************************
@@ -873,12 +883,18 @@ guchar   *strip_ansi(const guchar *original, int len);
 void      set_default_colors(World *world);
 World    *create_new_world(gboolean load_defaults);
 void      free_world(World *world);
+void      remove_plugin_objects(World *world, Plugin *plugin);
 gboolean  load_world_from_file(const char *file,
                                World *world,
                                gboolean forWorldSelector,
                                GError **error);
+void      create_world_from_parameters(World *world,
+                                       gchar *host,
+                                       gchar *port,
+                                       gchar *user,
+                                       gchar *password);
 void      load_command_history(World *world);
-World    *get_world_from_name(const gchar *name);
+World    *get_world_from_cmdline(const gchar *cmdline);
 void      save_world_to_file(World *world);
 void      save_command_history(World *world);
 gboolean  edit_world(World     **world,
@@ -897,46 +913,46 @@ gchar    *replace_kc_escapes(const gchar *pattern,
 /*********************************************
  * Function prototypes - World Editor Dialog *
  *********************************************/
-GtkWidget *get_we_general_tab(GladeXML *gladexml);
-void       fill_we_general_tab(World *world, GladeXML *gladexml);
-void       update_we_general_parameters(World *world);
-GtkWidget *get_we_mainwindow_tab(GladeXML *gladexml);
-void       fill_we_mainwindow_tab(World *world, GladeXML *gladexml);
-GtkWidget *get_we_colors_tab(World *world, GladeXML *gladexml);
-GtkWidget *get_we_statusbar_tab(GladeXML *gladexml);
-void       fill_we_statusbar_tab(World *world, GladeXML *gladexml);
-GtkWidget *get_we_input_tab(GladeXML *gladexml);
-void       fill_we_input_tab(World *world, GladeXML *gladexml);
-void       update_we_input_parameters(World *world);
-GtkWidget *get_we_logging_tab(GladeXML *gladexml, World *world);
-void       fill_we_logging_tab(World *world, GladeXML *gladexml);
-void       update_we_logging_parameters(World *world);
-void       set_input_line_controls(World *world,
-                                   GtkWidget *radSingleLine,
-                                   GtkWidget *radMultiLine,
-                                   GtkWidget *spnNLines);
-GtkWidget *get_we_scripting_tab(GtkWidget *parent, GladeXML *gladexml);
-void       fill_we_scripting_tab(World *world, GladeXML *gladexml);
-void       update_we_scripting_parameters(World *world);
-GtkWidget *get_we_plugins_tab(GladeXML *gladexml, World *world);
-GtkWidget *get_we_triggers_tab(World *world);
-GtkWidget *get_we_aliases_tab(World *world);
-GtkWidget *get_we_macros_tab(World *world);
-GtkWidget *get_we_timers_tab(World *world);
-GtkWidget *get_we_hooks_tab(World *world);
-GtkWidget *get_we_vars_tab(World *world);
-GtkWidget *get_we_protocols_tab(GladeXML *gladexml);
-void       fill_we_protocols_tab(World *world, GladeXML *gladexml);
-void       update_we_protocols_parameters(World *world);
-GtkWidget *get_we_misc_tab(GladeXML *gladexml);
-void       fill_we_misc_tab(World *world, GladeXML *gladexml);
-void       update_we_misc_parameters(World *world);
-GtkWidget *get_we_advanced_tab(World *world, GladeXML *xml);
-void       fill_we_advanced_tab(World *world, GladeXML *gladexml);
-void       update_we_advanced_parameters(World *world, gboolean *newworld);
+GObject   *get_we_general_tab(World *world, GtkBuilder *ui_builder);
+void       fill_we_general_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_general_parameters(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_mainwindow_tab(GtkBuilder *ui_builder);
+void       fill_we_mainwindow_tab(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_colors_tab(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_statusbar_tab(GtkBuilder *ui_builder);
+void       fill_we_statusbar_tab(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_input_tab(GtkBuilder *ui_builder);
+void       fill_we_input_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_input_parameters(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_logging_tab(GtkBuilder *ui_builder, World *world);
+void       fill_we_logging_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_logging_parameters(World *world, GtkBuilder *ui_builder);
+void       set_input_line_controls(World         *world,
+                                   GObject       *radSingleLine,
+                                   GObject       *radMultiLine,
+                                   GtkSpinButton *spnNLines);
+GObject   *get_we_scripting_tab(GtkBuilder *ui_builder);
+void       fill_we_scripting_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_scripting_parameters(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_plugins_tab(GtkBuilder *ui_builder, World *world);
+GObject   *get_we_triggers_tab(World *world);
+GObject   *get_we_aliases_tab(World *world);
+GObject   *get_we_macros_tab(World *world);
+GObject   *get_we_timers_tab(World *world);
+GObject   *get_we_hooks_tab(World *world);
+GObject   *get_we_vars_tab(World *world);
+GObject   *get_we_protocols_tab(GtkBuilder *ui_builder);
+void       fill_we_protocols_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_protocols_parameters(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_misc_tab(GtkBuilder *ui_builder);
+void       fill_we_misc_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_misc_parameters(World *world, GtkBuilder *ui_builder);
+GObject   *get_we_advanced_tab(World *world, GtkBuilder *ui_builder);
+void       fill_we_advanced_tab(World *world, GtkBuilder *ui_builder);
+void       update_we_advanced_parameters(World *world, gboolean *newworld,
+                                         GtkBuilder *ui_builder);
 
-void       we_guied_import_cb(GtkButton *button, gpointer data);
-void       we_guied_export_cb(GtkButton *button, gpointer data);
+G_MODULE_EXPORT void we_guied_export_cb(GtkButton *button, gpointer data);
 
 void       we_guied_bool_func(GtkTreeViewColumn *column,
                               GtkCellRenderer   *renderer,
@@ -1013,10 +1029,11 @@ void do_multi_line_send(MLSendData *mlcontext, gdouble delay);
  * Function prototypes - Network *
  *********************************/
 gboolean  connect_to(World *world, gchar **errmsg);
-gboolean  disconnect_world(gpointer data);
+gboolean  disconnect_world_idle(gpointer data);
+void      disconnect_world(gpointer data, gboolean run_hook);
 gboolean  reconnect_world(gpointer data);
 gboolean  connect_another(gpointer data);
-gchar    *close_connection(World *world);
+gchar    *close_connection(World *world, gboolean run_hook);
 gboolean  flush_buffer_timeout_cb(gpointer data);
 void      perform_auto_logon(World *world);
 
@@ -1027,7 +1044,8 @@ void      perform_auto_logon(World *world);
 void        read_global_preferences(int *x, int *y, int *width, int *height);
 void        save_global_preferences(void);
 const char *create_kildclient_directory(void);
-GtkWidget  *create_combo_with_colors(void);
+G_MODULE_EXPORT void show_preferences_dialog_cb(GtkWidget *widget,
+                                                gpointer data);
 
 
 /********************************
@@ -1123,7 +1141,10 @@ gboolean      execute_hook(World       *world,
                            const gchar *hookdata);
 void          list_hooks(World *world, const gchar *hookname, Plugin *plugin);
 void          list_hook(World *world, const gchar *hookname, int pos);
-gboolean      delete_hook(World *world, const gchar *hookname, int pos);
+gboolean      delete_hook(World *world,
+                          const gchar *hookname,
+                          int pos,
+                          GSList *hook);
 gboolean      move_hook(World *world, const gchar *hookname,
                         gint old_pos, gint new_pos);
 void          free_hook(Hook *hook, gpointer data);
