@@ -1,7 +1,7 @@
-/* $Id: mainwindow.c 1017 2009-02-02 18:34:39Z ekalin $ */
+/* $Id: mainwindow.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 #include <libintl.h>
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gmodule.h>
 #ifndef __MINGW32__
 #  include <netinet/in.h>
@@ -53,13 +52,14 @@
 /******************
  * Initialization *
  ******************/
-World     *currentWorld         = NULL;
-GtkWidget *wndMain              = NULL;
-GtkWidget *ntbkWorlds           = NULL;
-GList     *open_worlds          = NULL;
-gboolean   window_has_focus     = TRUE;
-gint       worlds_with_new_text = 0;
-gboolean   debug_matches        = 0;
+World      *currentWorld         = NULL;
+GtkBuilder *main_builder         = NULL;
+GtkWidget  *wndMain              = NULL;
+GtkWidget  *ntbkWorlds           = NULL;
+GList      *open_worlds          = NULL;
+gboolean    window_has_focus     = TRUE;
+gint        worlds_with_new_text = 0;
+gboolean    debug_matches        = 0;
 
 
 /***********************
@@ -73,7 +73,7 @@ static gint     attach_world_gui(WorldGUI *gui);
 static void     detach_world_gui(WorldGUI *gui);
 static gchar   *format_time_string(World *world, int ctime, int itime);
 static void     format_hms_time(GString *str, int seconds);
-/* Glade callbacks */
+/* XMl UI callbacks */
 G_MODULE_EXPORT gboolean exit_cb(GtkWidget *widget,
                                  GdkEvent *event,
                                  gpointer data);
@@ -92,8 +92,7 @@ G_MODULE_EXPORT void     notebook_page_reordered_cb(GtkNotebook *notebook,
                                                     guint        page_num,
                                                     gpointer     data);
 /* Menus */
-G_MODULE_EXPORT void     menu_world_open_cb(GtkWidget *widget,
-                                            gpointer data);
+G_MODULE_EXPORT void     menu_world_open_cb(gpointer data, GtkWidget *widget);
 G_MODULE_EXPORT void     open_new_world_cb(GtkWidget *widget,
                                            gpointer data);
 G_MODULE_EXPORT void     edit_this_world_cb(GtkWidget *widget,
@@ -141,6 +140,11 @@ G_MODULE_EXPORT void     mnuDebugMatches_activate_cb(GtkWidget *widget,
 G_MODULE_EXPORT void     menu_edit_default_world_cb(GtkWidget *widget,
                                                     gpointer data);
 
+G_MODULE_EXPORT void     menu_display_open_cb(GtkWidget *widget,
+                                              gpointer data);
+G_MODULE_EXPORT void     menu_split_screen_cb(GtkAction *widget,
+                                              gpointer data);
+
 G_MODULE_EXPORT void     menu_manual_cb(GtkWidget *widget,
                                         gpointer data);
 G_MODULE_EXPORT void     about_menu_cb(GtkWidget *widget,
@@ -151,22 +155,28 @@ G_MODULE_EXPORT void     about_menu_cb(GtkWidget *widget,
 GtkWidget*
 create_main_window(WorldGUI *gui)
 {
-  GtkWidget *window;
-  GladeXML  *gladexml;
-  GtkWidget *mnuPreferences;
-  GtkWidget *mnuReconnect;
-  GtkWidget *mnuDisconnect;
-  GtkWidget *mnuConnectAnother;
-  GtkWidget *mnuSave;
-  GtkWidget *mnuClose;
-  GtkWidget *mnuPrevious;
-  GtkWidget *mnuNext;
-  GtkWidget *mnuQuit;
+  GError     *error = NULL;
+  GtkWidget  *window;
+  GObject    *mnuReconnect;
+  GObject    *mnuDisconnect;
+  GObject    *mnuConnectAnother;
+  GObject    *mnuSave;
+  GObject    *mnuClose;
+  GObject    *mnuPrevious;
+  GObject    *mnuNext;
+  GObject    *mnuQuit;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("wndmain.glade"),
-                           "wndMain", NULL);
-  window = glade_xml_get_widget(gladexml, "wndMain");
-  glade_xml_signal_autoconnect(gladexml);
+  main_builder = gtk_builder_new();
+  if (!gtk_builder_add_from_file(main_builder,
+                                 get_kildclient_installed_file("wndmain.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    exit(1); /* No point in continuing */
+  }
+  gtk_builder_connect_signals(main_builder, NULL);
+
+  window = GTK_WIDGET(gtk_builder_get_object(main_builder, "wndMain"));
 #ifndef __WIN32__
   gtk_window_set_icon_from_file(GTK_WINDOW(window),
                                 SYSDATADIR "/pixmaps/kildclient.png",
@@ -176,7 +186,7 @@ create_main_window(WorldGUI *gui)
 #endif
 
   /* User interface */
-  ntbkWorlds = glade_xml_get_widget(gladexml, "ntbkWorlds");
+  ntbkWorlds = GTK_WIDGET(gtk_builder_get_object(main_builder, "ntbkWorlds"));
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(ntbkWorlds),
                            globalPrefs.tab_position);
 
@@ -184,35 +194,30 @@ create_main_window(WorldGUI *gui)
 
   gtk_widget_grab_focus(GTK_WIDGET(gui->cmbEntry));
 
-  /* This signal needs pointers to the objects */
-  mnuPreferences = glade_xml_get_widget(gladexml, "mnuPreferences");
-  g_signal_connect(G_OBJECT(mnuPreferences), "activate",
-                   G_CALLBACK(show_preferences_dialog_cb), NULL);
-
-  /* These signals apparently don't work from Glade */
-  mnuReconnect = glade_xml_get_widget(gladexml, "mnuReconnect");
-  g_signal_connect(G_OBJECT(mnuReconnect), "activate",
+  /* These signals apparently don't work from the XML file */
+  mnuReconnect = gtk_builder_get_object(main_builder, "mnuReconnect");
+  g_signal_connect(mnuReconnect, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->reconnect()");
-  mnuDisconnect = glade_xml_get_widget(gladexml, "mnuDisconnect");
-  g_signal_connect(G_OBJECT(mnuDisconnect), "activate",
+  mnuDisconnect = gtk_builder_get_object(main_builder, "mnuDisconnect");
+  g_signal_connect(mnuDisconnect, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->dc()");
-  mnuConnectAnother = glade_xml_get_widget(gladexml, "mnuConnectAnother");
-  g_signal_connect(G_OBJECT(mnuConnectAnother), "activate",
+  mnuConnectAnother = gtk_builder_get_object(main_builder, "mnuConnectAnother");
+  g_signal_connect(mnuConnectAnother, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->connectother()");
-  mnuSave = glade_xml_get_widget(gladexml, "mnuSave");
-  g_signal_connect(G_OBJECT(mnuSave), "activate",
+  mnuSave = gtk_builder_get_object(main_builder, "mnuSave");
+  g_signal_connect(mnuSave, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->save()");
-  mnuClose = glade_xml_get_widget(gladexml, "mnuClose");
-  g_signal_connect(G_OBJECT(mnuClose), "activate",
+  mnuClose = gtk_builder_get_object(main_builder, "mnuClose");
+  g_signal_connect(mnuClose, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->close()");
-  mnuPrevious = glade_xml_get_widget(gladexml, "mnuPrevious");
-  g_signal_connect(G_OBJECT(mnuPrevious), "activate",
+  mnuPrevious = gtk_builder_get_object(main_builder, "mnuPrevious");
+  g_signal_connect(mnuPrevious, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->prev()");
-  mnuNext = glade_xml_get_widget(gladexml, "mnuNext");
-  g_signal_connect(G_OBJECT(mnuNext), "activate",
+  mnuNext = gtk_builder_get_object(main_builder, "mnuNext");
+  g_signal_connect(mnuNext, "activate",
                    G_CALLBACK(menu_perl_run_cb), "$world->next()");
-  mnuQuit = glade_xml_get_widget(gladexml, "mnuQuit");
-  g_signal_connect(G_OBJECT(mnuQuit), "activate",
+  mnuQuit = gtk_builder_get_object(main_builder, "mnuQuit");
+  g_signal_connect(mnuQuit, "activate",
                    G_CALLBACK(menu_perl_run_cb), "quit");
 
   return window;
@@ -431,7 +436,7 @@ open_new_world_cb(GtkWidget *widget, gpointer data)
 
 gboolean
 open_new_world(WorldGUI    *gui,
-               const gchar *name,
+               const gchar *cmdline_arg,
                gboolean     can_offline,
                gboolean    *offline_selected)
 {
@@ -450,8 +455,8 @@ open_new_world(WorldGUI    *gui,
     prev_world = gui->world;
   }
 
-  if (name) {
-    world = get_world_from_name(name);
+  if (cmdline_arg) {
+    world = get_world_from_cmdline(cmdline_arg);
   } else {
     world = get_world_to_connect();
   }
@@ -517,121 +522,134 @@ edit_this_world_cb(GtkWidget *widget, gpointer data)
 void
 menu_statistics_cb(GtkWidget *widget, gpointer data)
 {
-  GladeXML  *gladexml;
-  GtkWidget *dlgStatistics;
-  GtkWidget *lblName;
-  GtkWidget *lblHost;
-  GtkWidget *lblPort;
-  GtkWidget *lblProxyType;
-  GtkWidget *lblProxyHost;
-  GtkWidget *lblProxyPort;
-  GtkWidget *lblConnTime;
-  GtkWidget *lblIdleTime;
-  GtkWidget *lblCompression;
-  GtkWidget *lblRawBytes;
-  GtkWidget *lblBytes;
-  GtkWidget *lblCRatio;
-  GtkWidget *lblLines;
-  GtkWidget *lblTLS;
-  gchar     *str;
-  time_t     now;
-  GString   *timestr;
+  GtkBuilder *ui_builder;
+  GError     *error = NULL;
+  GtkWidget  *dlgStatistics;
+  GtkLabel   *lblName;
+  GtkLabel   *lblHost;
+  GtkLabel   *lblPort;
+  GtkLabel   *lblProxyType;
+  GtkLabel   *lblProxyHost;
+  GtkLabel   *lblProxyPort;
+  GtkLabel   *lblConnTime;
+  GtkLabel   *lblIdleTime;
+  GtkLabel   *lblCompression;
+  GtkLabel   *lblRawBytes;
+  GtkLabel   *lblBytes;
+  GtkLabel   *lblCRatio;
+  GtkLabel   *lblLines;
+  GtkLabel   *lblTLS;
+  gchar      *str;
+  time_t      now;
+  GString    *timestr;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgStatistics", NULL);
-  dlgStatistics  = glade_xml_get_widget(gladexml, "dlgStatistics");
-  lblName        = glade_xml_get_widget(gladexml, "lblName");
-  lblHost        = glade_xml_get_widget(gladexml, "lblHost");
-  lblPort        = glade_xml_get_widget(gladexml, "lblPort");
-  lblConnTime    = glade_xml_get_widget(gladexml, "lblConnTime");
-  lblIdleTime    = glade_xml_get_widget(gladexml, "lblIdleTime");
-  lblCompression = glade_xml_get_widget(gladexml, "lblCompression");
-  lblRawBytes    = glade_xml_get_widget(gladexml, "lblRawBytes");
-  lblBytes       = glade_xml_get_widget(gladexml, "lblBytes");
-  lblCRatio      = glade_xml_get_widget(gladexml, "lblCRatio");
-  lblLines       = glade_xml_get_widget(gladexml, "lblLines");
-  lblTLS         = glade_xml_get_widget(gladexml, "lblTLS");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_from_file(ui_builder,
+                                 get_kildclient_installed_file("dlgStatistics.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return;
+  }
+
+  dlgStatistics  = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                    "dlgStatistics"));
+  lblName        = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblName"));
+  lblHost        = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblHost"));
+  lblPort        = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblPort"));
+  lblConnTime    = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblConnTime"));
+  lblIdleTime    = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblIdleTime"));
+  lblCompression = GTK_LABEL(gtk_builder_get_object(ui_builder,
+                                                   "lblCompression"));
+  lblRawBytes    = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblRawBytes"));
+  lblBytes       = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblBytes"));
+  lblCRatio      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblCRatio"));
+  lblLines       = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblLines"));
+  lblTLS         = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLS"));
 
   /* Basic information */
-  gtk_label_set_text(GTK_LABEL(lblName), currentWorld->name);
+  gtk_label_set_text(lblName, currentWorld->name);
   if (currentWorld->proxy_used.type == PROXY_NONE && currentWorld->host_ip) {
     str = g_strdup_printf("%s (%s)", currentWorld->host, currentWorld->host_ip);
   } else {
     str = g_strdup(currentWorld->host);
   }
-  gtk_label_set_text(GTK_LABEL(lblHost), str);
+  gtk_label_set_text(lblHost, str);
   g_free(str);
-  gtk_label_set_text(GTK_LABEL(lblPort), currentWorld->port);
+  gtk_label_set_text(lblPort, currentWorld->port);
 
   /* Proxy information */
   if (currentWorld->proxy_used.type != PROXY_NONE) {
-    lblProxyType = glade_xml_get_widget(gladexml, "lblProxyType");
-    lblProxyHost = glade_xml_get_widget(gladexml, "lblProxyHost");
-    lblProxyPort = glade_xml_get_widget(gladexml, "lblProxyPort");
+    lblProxyType
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblProxyType"));
+    lblProxyHost
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblProxyHost"));
+    lblProxyPort
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblProxyPort"));
 
     if (currentWorld->proxy_used.type == PROXY_SOCKS4) {
-      gtk_label_set_text(GTK_LABEL(lblProxyType), "SOCKS4");
+      gtk_label_set_text(lblProxyType, "SOCKS4");
     } else if (currentWorld->proxy_used.type == PROXY_SOCKS5) {
-      gtk_label_set_text(GTK_LABEL(lblProxyType), "SOCKS5");
+      gtk_label_set_text(lblProxyType, "SOCKS5");
     }
 
-    gtk_label_set_text(GTK_LABEL(lblProxyHost), currentWorld->proxy_used.server);
-    gtk_label_set_text(GTK_LABEL(lblProxyPort), currentWorld->proxy_used.port);
+    gtk_label_set_text(lblProxyHost, currentWorld->proxy_used.server);
+    gtk_label_set_text(lblProxyPort, currentWorld->proxy_used.port);
   }
 
   if (currentWorld->connected) {
     now = time(NULL);
     timestr = g_string_new(NULL);
     format_hms_time(timestr, difftime(now, currentWorld->connected_time));
-    gtk_label_set_text(GTK_LABEL(lblConnTime), timestr->str);
+    gtk_label_set_text(lblConnTime, timestr->str);
     g_string_truncate(timestr, 0);
     format_hms_time(timestr, difftime(now, currentWorld->last_command_time));
-    gtk_label_set_text(GTK_LABEL(lblIdleTime), timestr->str);
+    gtk_label_set_text(lblIdleTime, timestr->str);
     g_string_free(timestr, TRUE);
   } else {
-    gtk_label_set_text(GTK_LABEL(lblConnTime), _("Not connected"));
-    gtk_label_set_text(GTK_LABEL(lblIdleTime), _("Not connected"));
+    gtk_label_set_text(lblConnTime, _("Not connected"));
+    gtk_label_set_text(lblIdleTime, _("Not connected"));
   }
 
   /* Compression Information */
   if (currentWorld->zstream) {
     str = g_strdup_printf("MCCP version %d", currentWorld->mccp_ver);
-    gtk_label_set_text(GTK_LABEL(lblCompression), str);
+    gtk_label_set_text(lblCompression, str);
     g_free(str);
     str = g_strdup_printf("%ld", currentWorld->rawbytes);
-    gtk_label_set_text(GTK_LABEL(lblRawBytes), str);
+    gtk_label_set_text(lblRawBytes, str);
     g_free(str);
     str = g_strdup_printf("%ld", currentWorld->bytes);
-    gtk_label_set_text(GTK_LABEL(lblBytes), str);
+    gtk_label_set_text(lblBytes, str);
     g_free(str);
     str = g_strdup_printf("%.2f:1",
-                          (double)currentWorld->bytes/(double)currentWorld->rawbytes);
-    gtk_label_set_text(GTK_LABEL(lblCRatio), str);
+                          (double) currentWorld->bytes/(double) currentWorld->rawbytes);
+    gtk_label_set_text(lblCRatio, str);
     g_free(str);
   } else {
     str = g_strdup_printf("%ld", currentWorld->rawbytes);
-    gtk_label_set_text(GTK_LABEL(lblBytes), str);
+    gtk_label_set_text(lblBytes, str);
     g_free(str);
   }
 
   str = g_strdup_printf("%ld",
                         currentWorld->deleted_lines
                           + gtk_text_buffer_get_line_count(currentWorld->gui->txtBuffer));
-  gtk_label_set_text(GTK_LABEL(lblLines), str);
+  gtk_label_set_text(lblLines, str);
   g_free(str);
 
   /* TLS information */
 #ifdef HAVE_LIBGNUTLS
   if (currentWorld->use_tls) {
-    GtkWidget               *lblTLSCipher;
-    GtkWidget               *lblTLSMAC;
-    GtkWidget               *lblTLSKX;
-    GtkWidget               *lblTLSComp;
-    GtkWidget               *lblTLSCertType;
-    GtkWidget               *lblTLSCertHostname;
-    GtkWidget               *lblTLSCertActivation;
-    GtkWidget               *lblTLSCertExpiration;
-    GtkWidget               *lblTLSCertFingerprint;
+    GtkLabel                *lblTLSCipher;
+    GtkLabel                *lblTLSMAC;
+    GtkLabel                *lblTLSKX;
+    GtkLabel                *lblTLSComp;
+    GtkLabel                *lblTLSCertType;
+    GtkLabel                *lblTLSCertHostname;
+    GtkLabel                *lblTLSCertActivation;
+    GtkLabel                *lblTLSCertExpiration;
+    GtkLabel                *lblTLSCertFingerprint;
     gnutls_certificate_type  cert_type;
     const gnutls_datum      *cert_list;
     guint                    cert_list_size;
@@ -643,42 +661,48 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
     const gchar             *val;
     gchar                   *mval;
     gchar                    buf[MAX_BUFFER];
-    gint                     i;
+    guint                    i;
 
     val = gnutls_protocol_get_name(gnutls_protocol_get_version(currentWorld->tls_session));
-    gtk_label_set_text(GTK_LABEL(lblTLS), val);
+    gtk_label_set_text(lblTLS, val);
 
-    lblTLSCipher = glade_xml_get_widget(gladexml, "lblTLSCipher");
+    lblTLSCipher
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCipher"));
     val = gnutls_cipher_get_name(gnutls_cipher_get(currentWorld->tls_session));
-    gtk_label_set_text(GTK_LABEL(lblTLSCipher), val);
+    gtk_label_set_text(lblTLSCipher, val);
 
-    lblTLSMAC = glade_xml_get_widget(gladexml, "lblTLSMAC");
+    lblTLSMAC = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSMAC"));
     val = gnutls_mac_get_name(gnutls_mac_get(currentWorld->tls_session));
-    gtk_label_set_text(GTK_LABEL(lblTLSMAC), val);
+    gtk_label_set_text(lblTLSMAC, val);
 
-    lblTLSKX = glade_xml_get_widget(gladexml, "lblTLSKX");
+    lblTLSKX = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSKX"));
     val = gnutls_kx_get_name(gnutls_kx_get(currentWorld->tls_session));
-    gtk_label_set_text(GTK_LABEL(lblTLSKX), val);
+    gtk_label_set_text(lblTLSKX, val);
 
-    lblTLSComp = glade_xml_get_widget(gladexml, "lblTLSComp");
+    lblTLSComp = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSComp"));
     val = gnutls_compression_get_name(gnutls_compression_get(currentWorld->tls_session));
-    gtk_label_set_text(GTK_LABEL(lblTLSComp), val);
+    gtk_label_set_text(lblTLSComp, val);
 
-    lblTLSCertType = glade_xml_get_widget(gladexml, "lblTLSCertType");
+    lblTLSCertType = GTK_LABEL(gtk_builder_get_object(ui_builder,
+                                                     "lblTLSCertType"));
     cert_type = gnutls_certificate_type_get(currentWorld->tls_session);
     val = gnutls_certificate_type_get_name(cert_type);
-    gtk_label_set_text(GTK_LABEL(lblTLSCertType), val);
+    gtk_label_set_text(lblTLSCertType, val);
 
     /* Certificate/key information */
-    lblTLSCertHostname    = glade_xml_get_widget(gladexml, "lblTLSCertHostname");
-    lblTLSCertActivation  = glade_xml_get_widget(gladexml, "lblTLSCertActivation");
-    lblTLSCertExpiration  = glade_xml_get_widget(gladexml, "lblTLSCertExpiration");
-    lblTLSCertFingerprint = glade_xml_get_widget(gladexml, "lblTLSCertFingerprint");
+    lblTLSCertHostname
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertHostname"));
+    lblTLSCertActivation
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertActivation"));
+    lblTLSCertExpiration
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertExpiration"));
+    lblTLSCertFingerprint
+      = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertFingerprint"));
     cert_list = gnutls_certificate_get_peers(currentWorld->tls_session,
                                              &cert_list_size);
     if (cert_type == GNUTLS_CRT_X509) {
-      GtkWidget      *lblTLSCertSubDN;
-      GtkWidget      *lblTLSCertIssDN;
+      GtkLabel       *lblTLSCertSubDN;
+      GtkLabel       *lblTLSCertIssDN;
       gnutls_x509_crt crt;
       size_t          size;
 
@@ -698,28 +722,19 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
 
         size = sizeof(buf);
         gnutls_x509_crt_get_dn(crt, buf, &size);
-        lblTLSCertSubDN = glade_xml_get_widget(gladexml, "lblTLSCertSubDN");
-        gtk_label_set_text(GTK_LABEL(lblTLSCertSubDN), buf);
+        lblTLSCertSubDN
+          = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertSubDN"));
+        gtk_label_set_text(lblTLSCertSubDN, buf);
 
         size = sizeof(buf);
         gnutls_x509_crt_get_issuer_dn(crt, buf, &size);
-        lblTLSCertIssDN = glade_xml_get_widget(gladexml, "lblTLSCertIssDN");
-        gtk_label_set_text(GTK_LABEL(lblTLSCertIssDN), buf);
+        lblTLSCertIssDN
+          = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblTLSCertIssDN"));
+        gtk_label_set_text(lblTLSCertIssDN, buf);
       }
-    } else if (cert_type == GNUTLS_CRT_OPENPGP) {
-      gnutls_openpgp_key crt;
-
-      gnutls_openpgp_key_init(&crt);
-      gnutls_openpgp_key_import(crt, &cert_list[0], GNUTLS_OPENPGP_FMT_RAW);
-
-      hostname_matches = gnutls_openpgp_key_check_hostname(crt,
-                                                           currentWorld->host);
-
-      act_time = gnutls_openpgp_key_get_creation_time(crt);
-      exp_time = gnutls_openpgp_key_get_expiration_time(crt);
-
-      digest_size = sizeof(digest);
-      gnutls_openpgp_key_get_fingerprint(crt, digest, &digest_size);
+    } else {
+      gtk_label_set_text(lblTLSCertHostname, "Unknown certificate type");
+      return;
     }
 
     if (hostname_matches) {
@@ -727,7 +742,7 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
     } else {
       mval = g_strdup_printf("Does not match '%s'", currentWorld->host);
     }
-    gtk_label_set_text(GTK_LABEL(lblTLSCertHostname), mval);
+    gtk_label_set_text(lblTLSCertHostname, mval);
     g_free(mval);
 
     now = time(NULL);
@@ -735,12 +750,12 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
     if (act_time > now) {
       strcat(buf, _(": Not yet active"));
     }
-    gtk_label_set_text(GTK_LABEL(lblTLSCertActivation), buf);
+    gtk_label_set_text(lblTLSCertActivation, buf);
     strftime(buf, MAX_BUFFER, "%c", localtime(&exp_time));
     if (exp_time < now) {
       strcat(buf, _(": Expired"));
     }
-    gtk_label_set_text(GTK_LABEL(lblTLSCertExpiration), buf);
+    gtk_label_set_text(lblTLSCertExpiration, buf);
 
     mval = buf;
     for (i = 0; i < digest_size; i++) {
@@ -748,16 +763,16 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
       mval += 3;
     }
     mval[strlen(mval) - 1] = 0; /* Remove last : */
-    gtk_label_set_text(GTK_LABEL(lblTLSCertFingerprint), buf);
+    gtk_label_set_text(lblTLSCertFingerprint, buf);
   } else {
-    gtk_label_set_text(GTK_LABEL(lblTLS), _("None"));
+    gtk_label_set_text(lblTLS, _("None"));
   }
 #endif
 
   /* Run the dialog */
   gtk_dialog_run(GTK_DIALOG(dlgStatistics));
   gtk_widget_destroy(dlgStatistics);
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 }
 
 
@@ -780,7 +795,7 @@ menu_manual_cb(GtkWidget *widget, gpointer data)
 #else
   manualurl = g_strdup_printf(get_kildclient_installed_file("manual\\html\\index.html"));
 #endif
-  menu_url_open(widget, manualurl);
+  menu_url_open(NULL, manualurl);
 }
 
 
@@ -788,18 +803,23 @@ menu_manual_cb(GtkWidget *widget, gpointer data)
 void
 about_menu_cb(GtkWidget *widget, gpointer data)
 {
-  GladeXML         *xml;
+  GError           *error = NULL;
   static GtkWidget *dlgAbout = NULL;
 
   if (!dlgAbout) {
     gtk_about_dialog_set_url_hook(about_dlg_url_hook, NULL, NULL);
 
-    xml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                        "dlgAbout", NULL);
-    dlgAbout = glade_xml_get_widget(xml, "dlgAbout");
+    if (!gtk_builder_add_from_file(main_builder,
+                                   get_kildclient_installed_file("dlgAbout.ui"),
+                                   &error)) {
+      g_warning(_("Error loading UI from XML file: %s"), error->message);
+      g_error_free(error);
+      return;
+    }
+    dlgAbout = GTK_WIDGET(gtk_builder_get_object(main_builder, "dlgAbout"));
     gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dlgAbout), VERSION);
 
-    glade_xml_signal_autoconnect(xml);
+    gtk_builder_connect_signals(main_builder, NULL);
   }
 
   gtk_widget_show_all(dlgAbout);
@@ -812,7 +832,7 @@ void
 about_dlg_url_hook(GtkAboutDialog *about, const gchar *link, gpointer data)
 {
   gchar *url = g_strdup(link);
-  menu_url_open(GTK_WIDGET(about), url);
+  menu_url_open(NULL, url);
 }
 
 
@@ -881,7 +901,7 @@ remove_world(World *world, gboolean remove_gui)
   open_worlds = g_list_remove(open_worlds, world);
 
   if (world->connected || world->connecting) {
-    g_free(close_connection(world));
+    g_free(close_connection(world, TRUE));
   }
 
   if (world->file) {
@@ -921,33 +941,25 @@ set_notebook_tab_position(GtkPositionType pos)
 
 
 void
-menu_world_open_cb(GtkWidget *widget,
-                   gpointer data)
+menu_world_open_cb(gpointer data, GtkWidget *widget)
 {
-  GladeXML  *gladexml;
-  GtkWidget *mnuDisconnect;
-
-  gladexml = glade_get_widget_tree(widget);
-  mnuDisconnect = glade_xml_get_widget(gladexml,
-                                       "mnuDisconnect");
+  GtkAction *mnuDisconnect = GTK_ACTION(data);
 
   if (currentWorld && (currentWorld->connected || currentWorld->connecting)) {
-    gtk_widget_set_sensitive(mnuDisconnect, TRUE);
+    gtk_action_set_sensitive(mnuDisconnect, TRUE);
   } else {
-    gtk_widget_set_sensitive(mnuDisconnect, FALSE);
+    gtk_action_set_sensitive(mnuDisconnect, FALSE);
   }
 }
 
 
 void
-menu_edit_open_cb(GtkWidget *widget,
-                  gpointer data)
+menu_edit_open_cb(GtkWidget *widget, gpointer data)
 {
-  GladeXML     *gladexml;
-  GtkWidget    *mnuCut;
-  GtkWidget    *mnuCopy;
-  GtkWidget    *mnuPaste;
-  GtkWidget    *mnuDelete;
+  GtkAction    *mnuCut;
+  GtkAction    *mnuCopy;
+  GtkAction    *mnuPaste;
+  GtkAction    *mnuDelete;
   GdkDisplay   *display;
   GtkClipboard *clipboard;
   gboolean      input_selected;
@@ -959,32 +971,30 @@ menu_edit_open_cb(GtkWidget *widget,
     return;
   }
 
-  gladexml  = glade_get_widget_tree(widget);
-  mnuCut    = glade_xml_get_widget(gladexml, "mnuCut");
-  mnuCopy   = glade_xml_get_widget(gladexml, "mnuCopy");
-  mnuPaste  = glade_xml_get_widget(gladexml, "mnuPaste");
-  mnuDelete = glade_xml_get_widget(gladexml, "mnuDelete");
+  mnuCut     = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuCut"));
+  mnuCopy    = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuCopy"));
+  mnuPaste   = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuPaste"));
+  mnuDelete  = GTK_ACTION(gtk_builder_get_object(main_builder, "mnuDelete"));
 
   /* We can paste when there is text in the clipboard */
-
-  display = gtk_widget_get_display(GTK_WIDGET(widget));
+  display = gtk_widget_get_display(wndMain);
   clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_CLIPBOARD);
-  gtk_widget_set_sensitive(mnuPaste,
+  gtk_action_set_sensitive(mnuPaste,
                            gtk_clipboard_wait_is_text_available(clipboard));
 
   /* We can cut and delete if there is text selected in the input box */
   input_selected
     = simo_combo_box_get_selection_bounds(currentWorld->gui->cmbEntry,
                                           NULL, NULL);
-  gtk_widget_set_sensitive(mnuCut,    input_selected);
-  gtk_widget_set_sensitive(mnuDelete, input_selected);
+  gtk_action_set_sensitive(mnuCut,    input_selected);
+  gtk_action_set_sensitive(mnuDelete, input_selected);
 
   /* We can copy if there is a selection in the input box or the output
      window */
   output_selected
     = gtk_text_buffer_get_selection_bounds(currentWorld->gui->txtBuffer,
                                            NULL, NULL);
-  gtk_widget_set_sensitive(mnuCopy, input_selected || output_selected);
+  gtk_action_set_sensitive(mnuCopy, input_selected || output_selected);
 }
 
 
@@ -998,7 +1008,7 @@ menu_cut_activate_cb(GtkWidget *widget,
 
 void
 menu_copy_activate_cb(GtkWidget *widget,
-                     gpointer data)
+                      gpointer data)
 {
   GtkTextIter start;
   GtkTextIter end;
@@ -1008,7 +1018,7 @@ menu_copy_activate_cb(GtkWidget *widget,
     GdkDisplay   *display;
     GtkClipboard *clipboard;
 
-    display = gtk_widget_get_display(GTK_WIDGET(widget));
+    display = gtk_widget_get_display(GTK_WIDGET(currentWorld->gui->txtView));
 
     clipboard = gtk_clipboard_get_for_display(display,
                                               GDK_SELECTION_CLIPBOARD);
@@ -1138,30 +1148,32 @@ void
 menu_preferences_open_cb(GtkWidget *widget,
                          gpointer data)
 {
-  GladeXML  *gladexml;
-  GtkWidget *mnuDisableTriggers;
-  GtkWidget *mnuDisableAliases;
-  GtkWidget *mnuDisableMacros;
-  GtkWidget *mnuDisableTimers;
+  GtkToggleAction *mnuDisableTriggers;
+  GtkToggleAction *mnuDisableAliases;
+  GtkToggleAction *mnuDisableMacros;
+  GtkToggleAction *mnuDisableTimers;
 
-  gladexml = glade_get_widget_tree(widget);
-  mnuDisableTriggers = glade_xml_get_widget(gladexml,
-                                            "mnuDisTriggers");
-  mnuDisableAliases  = glade_xml_get_widget(gladexml,
-                                            "mnuDisAliases");
-  mnuDisableMacros   = glade_xml_get_widget(gladexml,
-                                            "mnuDisMacros");
-  mnuDisableTimers   = glade_xml_get_widget(gladexml,
-                                            "mnuDisTimers");
+  if (!currentWorld) {
+    return;
+  }
 
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mnuDisableTriggers),
-                                 currentWorld->disable_triggers);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mnuDisableAliases),
-                                 currentWorld->disable_aliases);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mnuDisableMacros),
-                                 currentWorld->disable_macros);
-  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mnuDisableTimers),
-                                 currentWorld->disable_timers);
+  mnuDisableTriggers
+    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisTriggers"));
+  mnuDisableAliases
+    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisAliases"));
+  mnuDisableMacros
+    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisMacros"));
+  mnuDisableTimers
+    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuDisTimers"));
+
+  gtk_toggle_action_set_active(mnuDisableTriggers,
+                               currentWorld->disable_triggers);
+  gtk_toggle_action_set_active(mnuDisableAliases,
+                               currentWorld->disable_aliases);
+  gtk_toggle_action_set_active(mnuDisableMacros,
+                               currentWorld->disable_macros);
+  gtk_toggle_action_set_active(mnuDisableTimers,
+                               currentWorld->disable_timers);
 }
 
 
@@ -1170,7 +1182,7 @@ menu_disable_triggers_cb(GtkWidget *widget,
                          gpointer data)
 {
   currentWorld->disable_triggers =
-    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
 }
 
 
@@ -1179,7 +1191,7 @@ menu_disable_aliases_cb(GtkWidget *widget,
                         gpointer data)
 {
   currentWorld->disable_aliases =
-    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
 }
 
 
@@ -1188,7 +1200,7 @@ menu_disable_macros_cb(GtkWidget *widget,
                        gpointer data)
 {
   currentWorld->disable_macros =
-    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
 }
 
 
@@ -1197,15 +1209,65 @@ menu_disable_timers_cb(GtkWidget *widget,
                        gpointer data)
 {
   currentWorld->disable_timers =
-    gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+    gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
 }
 
 
 void
 mnuDebugMatches_activate_cb(GtkWidget *widget,
-                   gpointer data)
+                            gpointer data)
 {
-  debug_matches = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget));
+  debug_matches = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(widget));
+}
+
+
+void
+menu_display_open_cb(GtkWidget *widget,
+                     gpointer data)
+{
+  GtkToggleAction *mnuSplitWindow;
+
+  if (!currentWorld) {
+    return;
+  }
+
+  mnuSplitWindow
+    = GTK_TOGGLE_ACTION(gtk_builder_get_object(main_builder, "mnuSplitWindow"));
+
+  g_signal_handlers_block_by_func(G_OBJECT(mnuSplitWindow),
+                                  menu_split_screen_cb,
+                                  NULL);
+  gtk_toggle_action_set_active(mnuSplitWindow,
+                               gtk_paned_get_position(currentWorld->gui->split_pane) > 0);
+  g_signal_handlers_unblock_by_func(G_OBJECT(mnuSplitWindow),
+                                    menu_split_screen_cb,
+                                    NULL);
+}
+
+void
+menu_split_screen_cb(GtkAction *action, gpointer data)
+{
+  if (gtk_paned_get_position(currentWorld->gui->split_pane) == 0) {
+    if (currentWorld->gui->last_split_position == 0) {
+      GtkAllocation allocation;
+
+#if GTK_MINOR_VERSION < 18
+      allocation = GTK_WIDGET(currentWorld->gui->split_pane)->allocation;
+#else
+      gtk_widget_get_allocation(GTK_WIDGET(currentWorld->gui->split_pane),
+                                &allocation);
+#endif
+
+      currentWorld->gui->last_split_position = allocation.height * .65;
+    }
+
+    gtk_paned_set_position(currentWorld->gui->split_pane,
+                           currentWorld->gui->last_split_position);
+  } else {
+    currentWorld->gui->last_split_position
+      = gtk_paned_get_position(currentWorld->gui->split_pane);
+    gtk_paned_set_position(currentWorld->gui->split_pane, 0);
+  }
 }
 
 
