@@ -1,7 +1,7 @@
-/* $Id: multilinesend.c 1027 2009-02-12 18:27:11Z ekalin $ */
+/* $Id: multilinesend.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gmodule.h>
 
 #include "kildclient.h"
@@ -48,9 +47,9 @@ static gchar     *get_text_buffer_line(GtkTextBuffer *buffer,
 static gchar     *get_file_line(FILE *fp, gboolean *last);
 static void       send_line(gchar *line, MLSendData  *context);
 static void       string_free(gpointer string, gpointer user_data);
-static void       start_multi_line_send_cb(GtkButton *btn, gpointer data);
-/* Glade signals */
+/* XML UI signals */
 G_MODULE_EXPORT void multi_line_send_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void start_multi_line_send_cb(GtkButton *btn, gpointer data);
 G_MODULE_EXPORT void clear_multi_line_send_dialog_cb(GtkWidget *widget,
                                                      gpointer   data);
 
@@ -73,15 +72,12 @@ static
 GtkWidget *
 create_multi_line_send_dialog(GtkWindow *parent, World *world)
 {
-  GladeXML  *gladexml;
-  GtkWidget *dlg;
-  GtkWidget *text;
-  GtkWidget *txtInitial;
-  GtkWidget *txtFile;
-  GtkWidget *btnClearFile;
-  GtkWidget *btnSend;
-  GtkWidget *spnDelay;
-  GtkWidget *spnLinesATime;
+  GError        *error = NULL;
+  GtkWidget     *dlg;
+  GtkWidget     *text;
+  GtkWidget     *txtInitial;
+  GtkSpinButton *spnDelay;
+  GtkSpinButton *spnLinesATime;
 
   PangoFontDescription *fontDesc;
   PangoLayout          *layout;
@@ -90,46 +86,48 @@ create_multi_line_send_dialog(GtkWindow *parent, World *world)
   GtkRequisition        requisition;
 
   /* Create the dialog */
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgMLSend", NULL);
-  dlg      = glade_xml_get_widget(gladexml, "dlgMLSend");
+  if (!world->ui_builder) {
+    world->ui_builder = gtk_builder_new();
+  }
+  if (!gtk_builder_add_from_file(world->ui_builder,
+                                 get_kildclient_installed_file("dlgMLSend.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+  dlg = GTK_WIDGET(gtk_builder_get_object(world->ui_builder, "dlgMLSend"));
 
   /* Set the fonts */
   fontDesc = pango_font_description_from_string(world->entryfont);
 
-  txtInitial = glade_xml_get_widget(gladexml, "txtInitial");
+  txtInitial = GTK_WIDGET(gtk_builder_get_object(world->ui_builder,
+                                                 "txtInitial"));
   gtk_widget_modify_font(txtInitial, fontDesc);
 
-  text = glade_xml_get_widget(gladexml, "txtFinal");
+  text = GTK_WIDGET(gtk_builder_get_object(world->ui_builder, "txtFinal"));
   gtk_widget_modify_font(text, fontDesc);
 
-  text  = glade_xml_get_widget(gladexml, "txtStart");
+  text  = GTK_WIDGET(gtk_builder_get_object(world->ui_builder, "txtStart"));
   gtk_widget_modify_font(text, fontDesc);
 
-  text    = glade_xml_get_widget(gladexml, "txtEnd");
+  text    = GTK_WIDGET(gtk_builder_get_object(world->ui_builder, "txtEnd"));
   gtk_widget_modify_font(text, fontDesc);
 
   pango_font_description_free(fontDesc);
 
-  /* Connect signals */
-  glade_xml_signal_autoconnect(gladexml);
-
-  btnSend = glade_xml_get_widget(gladexml, "btnMLSend");
-  g_signal_connect(G_OBJECT(btnSend), "clicked",
-                   G_CALLBACK(start_multi_line_send_cb), world);
-
-  txtFile       = glade_xml_get_widget(gladexml, "txtFile");
-  btnClearFile = glade_xml_get_widget(gladexml, "btnClearFile");
-  g_signal_connect_swapped(G_OBJECT(btnClearFile), "clicked",
-                           G_CALLBACK(gtk_file_chooser_unselect_all), txtFile);
-
   /* Default values */
-  spnDelay = glade_xml_get_widget(gladexml, "spnMLSDelay");
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnDelay),
-                            globalPrefs.multi_cmd_delay);
-  spnLinesATime = glade_xml_get_widget(gladexml, "spnLinesATime");
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnLinesATime),
-                            globalPrefs.multi_cmd_group_size);
+  spnDelay = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                                    "spnMLSDelay"));
+  gtk_spin_button_set_value(spnDelay, globalPrefs.multi_cmd_delay);
+  spnLinesATime
+    = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder,
+                                             "spnLinesATime"));
+  gtk_spin_button_set_value(spnLinesATime, globalPrefs.multi_cmd_group_size);
+
+  /* Connect signals */
+  gtk_builder_connect_signals(world->ui_builder, world);
 
   /* Set width of entries to be slightly larger than 80 columns. */
   layout = gtk_widget_create_pango_layout(txtInitial, "W");
@@ -151,44 +149,46 @@ create_multi_line_send_dialog(GtkWindow *parent, World *world)
 void
 start_multi_line_send_cb(GtkButton *btn, gpointer data)
 {
-  World      *world = (World *) data;
-  GladeXML   *gladexml;
-  GtkWidget  *txtInitial;
-  GtkWidget  *txtFinal;
-  GtkWidget  *txtFile;
-  GtkWidget  *txtStart;
-  GtkWidget  *txtEnd;
-  GtkWidget  *spnDelay;
-  GtkWidget  *spnLinesATime;
-  gdouble     delay;
-  GtkWidget  *chkDontClose;
-  MLSendData *mlcontext;
+  World           *world = (World *) data;
+  GtkBuilder      *ui_builder;
+  GtkTextView     *txtInitial;
+  GtkTextView     *txtFinal;
+  GtkFileChooser  *txtFile;
+  GtkEntry        *txtStart;
+  GtkEntry        *txtEnd;
+  GtkSpinButton   *spnDelay;
+  GtkSpinButton   *spnLinesATime;
+  GtkToggleButton *chkDontClose;
+  gdouble          delay;
+  MLSendData      *mlcontext;
 
-  gladexml = glade_get_widget_tree(GTK_WIDGET(btn));
+  ui_builder = world->ui_builder;
 
-  txtInitial    = glade_xml_get_widget(gladexml, "txtInitial");
-  txtFinal      = glade_xml_get_widget(gladexml, "txtFinal");
-  txtFile       = glade_xml_get_widget(gladexml, "txtFile");
-  txtStart      = glade_xml_get_widget(gladexml, "txtStart");
-  txtEnd        = glade_xml_get_widget(gladexml, "txtEnd");
-  spnDelay      = glade_xml_get_widget(gladexml, "spnMLSDelay");
-  spnLinesATime = glade_xml_get_widget(gladexml, "spnLinesATime");
-  chkDontClose  = glade_xml_get_widget(gladexml, "chkDontClose");
+  txtInitial
+    = GTK_TEXT_VIEW(gtk_builder_get_object(ui_builder, "txtInitial"));
+  txtFinal = GTK_TEXT_VIEW(gtk_builder_get_object(ui_builder, "txtFinal"));
+  txtFile  = GTK_FILE_CHOOSER(gtk_builder_get_object(ui_builder, "txtFile"));
+  txtStart = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtStart"));
+  txtEnd   = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtEnd"));
+  spnDelay
+    = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder, "spnMLSDelay"));
+  spnLinesATime
+    = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder, "spnLinesATime"));
+  chkDontClose
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkDontClose"));
 
-  delay = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spnDelay));
+  delay = gtk_spin_button_get_value(spnDelay);
 
   mlcontext = g_new0(MLSendData, 1);
   mlcontext->state       = ML_TRY_INITIALBUFFER;
   mlcontext->world       = world;
-  mlcontext->textInitial = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txtInitial));
-  mlcontext->textFinal   = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txtFinal));
-  mlcontext->file        = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(txtFile));
-  mlcontext->textStart   = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtStart)));
-  mlcontext->textEnd     = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtEnd)));
-  mlcontext->linesatime  =
-    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spnLinesATime));
-  mlcontext->dont_close
-    = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkDontClose));
+  mlcontext->textInitial = gtk_text_view_get_buffer(txtInitial);
+  mlcontext->textFinal   = gtk_text_view_get_buffer(txtFinal);
+  mlcontext->file        = gtk_file_chooser_get_filename(txtFile);
+  mlcontext->textStart   = g_strdup(gtk_entry_get_text(txtStart));
+  mlcontext->textEnd     = g_strdup(gtk_entry_get_text(txtEnd));
+  mlcontext->linesatime  = gtk_spin_button_get_value_as_int(spnLinesATime);
+  mlcontext->dont_close  = gtk_toggle_button_get_active(chkDontClose);
 
   if (!mlcontext->dont_close) {
     gtk_widget_hide(world->dlgMLSend);
@@ -345,7 +345,7 @@ multi_line_send_timer_cb(gpointer data)
     case ML_FINISH:
       if (context->textInitial && !context->dont_close) {
         clear_multi_line_send_dialog_cb(GTK_WIDGET(context->world->dlgMLSend),
-                                        NULL);
+                                        context->world);
       }
       g_free(context->textStart);
       g_free(context->textEnd);
@@ -370,30 +370,31 @@ multi_line_send_timer_cb(gpointer data)
 void
 clear_multi_line_send_dialog_cb(GtkWidget *widget, gpointer data)
 {
-  GladeXML      *gladexml;
-  GtkWidget     *dlgMLSend;
-  GtkWidget     *txtInitial;
-  GtkWidget     *txtFile;
-  GtkWidget     *txtFinal;
-  GtkWidget     *txtStart;
-  GtkWidget     *txtEnd;
-  GtkTextBuffer *buffer;
+  World          *world = (World *) data;
+  GtkBuilder     *ui_builder;
+  GtkWidget      *dlgMLSend;
+  GtkTextView    *txtInitial;
+  GtkFileChooser *txtFile;
+  GtkTextView    *txtFinal;
+  GtkEntry       *txtStart;
+  GtkEntry       *txtEnd;
+  GtkTextBuffer  *buffer;
 
-  gladexml   = glade_get_widget_tree(widget);
-  dlgMLSend  = glade_xml_get_widget(gladexml, "dlgMLSend");
-  txtInitial = glade_xml_get_widget(gladexml, "txtInitial");
-  txtFile    = glade_xml_get_widget(gladexml, "txtFile");
-  txtFinal   = glade_xml_get_widget(gladexml, "txtFinal");
-  txtStart   = glade_xml_get_widget(gladexml, "txtStart");
-  txtEnd     = glade_xml_get_widget(gladexml, "txtEnd");
+  ui_builder = world->ui_builder;
+  dlgMLSend  = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgMLSend"));
+  txtInitial = GTK_TEXT_VIEW(gtk_builder_get_object(ui_builder, "txtInitial"));
+  txtFile    = GTK_FILE_CHOOSER(gtk_builder_get_object(ui_builder, "txtFile"));
+  txtFinal   = GTK_TEXT_VIEW(gtk_builder_get_object(ui_builder, "txtFinal"));
+  txtStart   = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtStart"));
+  txtEnd     = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtEnd"));
 
-  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txtInitial));
+  buffer = gtk_text_view_get_buffer(txtInitial);
   gtk_text_buffer_set_text(buffer, "", 0);
-  gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(txtFile));
-  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txtFinal));
+  gtk_file_chooser_unselect_all(txtFile);
+  buffer = gtk_text_view_get_buffer(txtFinal);
   gtk_text_buffer_set_text(buffer, "", 0);
-  gtk_entry_set_text(GTK_ENTRY(txtStart), "");
-  gtk_entry_set_text(GTK_ENTRY(txtEnd), "");
+  gtk_entry_set_text(txtStart, "");
+  gtk_entry_set_text(txtEnd, "");
 
   gtk_widget_hide(dlgMLSend);
 }
