@@ -1,7 +1,7 @@
-/* $Id: net.c 1024 2009-02-09 18:01:06Z ekalin $ */
+/* $Id: net.c 1161 2011-04-25 20:29:35Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <zlib.h>
 
 #ifndef __MINGW32__
@@ -59,9 +58,6 @@
  *************************/
 #define MAX_US 250000
 static char *terminal_type = "KildClient";
-#ifdef HAVE_LIBGNUTLS
-const int cert_type_priority[3] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
-#endif
 
 
 /**************
@@ -99,6 +95,12 @@ static gboolean connect_ready_cb(GIOChannel   *iochan,
                                  GIOCondition  cond,
                                  gpointer      data);
 static gboolean connected_to_dispatcher(World *world);
+#ifdef HAVE_LIBGNUTLS
+static gboolean tls_handshake_cb(GIOChannel   *iochan,
+                                 GIOCondition  cond,
+                                 gpointer      data);
+static gboolean tls_handshake_timeout_cb(gpointer data);
+#endif
 static void     disconnected_decide_action(World *world, gint action);
 static gboolean data_ready_cb(GIOChannel   *iochan,
                               GIOCondition  cond,
@@ -171,9 +173,7 @@ connect_to(World *world, gchar **errmsg)
 #else /* defined HAVE_LIBGNUTLS */
   if (world->use_tls) {
     gnutls_init(&world->tls_session, GNUTLS_CLIENT);
-    gnutls_set_default_priority(world->tls_session);
-    gnutls_certificate_type_set_priority(world->tls_session,
-                                         cert_type_priority);
+    gnutls_priority_set_direct(world->tls_session, "NORMAL", NULL);
     gnutls_credentials_set(world->tls_session,
                            GNUTLS_CRD_CERTIFICATE, tls_certif_cred);
   }
@@ -267,6 +267,7 @@ connect_to(World *world, gchar **errmsg)
     resolve_data->callback = continue_connecting_cb;
     resolve_data->ai_family = AF_UNSPEC;
   }
+
   if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
     *errmsg = g_strdup(_("Could not start name-resolution thread."));
     return FALSE;
@@ -323,7 +324,7 @@ continue_connecting_cb(gpointer data)
       world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
                                       resolve_data->host, msg);
       g_free(msg);
-      disconnect_world(world);
+      disconnect_world(world, FALSE);
     }
     g_free(resolve_data->host);
     g_free(resolve_data);
@@ -376,7 +377,7 @@ continue_connecting_cb(gpointer data)
     world->errmsg = g_strdup_printf(_("Could not create socket: error #%d"),
                                     WSAGetLastError());
 #endif
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 #ifndef __MINGW32__
@@ -411,6 +412,12 @@ continue_connecting_cb(gpointer data)
 #endif
   }
 
+  /* Enable TCP Keep Alive if selected */
+  if (world->keep_alive) {
+    int optval = 1;
+    setsockopt(world->sock, SOL_SOCKET, SO_KEEPALIVE, &optval, sizeof(optval));
+  }
+
   world->noecho = FALSE;
 #ifndef __MINGW32__
   world->iochan = g_io_channel_unix_new(world->sock);
@@ -447,8 +454,8 @@ connect_ready_cb(GIOChannel   *iochan,
 
   ret = connect(world->sock,
                 world->addrinfo->ai_addr, world->addrinfo->ai_addrlen);
-  if (ret < 0) {
-    if (errno == EALREADY || ret == EINPROGRESS) {
+  if (ret < 0 && errno != EISCONN) {
+    if (errno == EALREADY || errno == EINPROGRESS) {
       return TRUE;  /* Still connecting. */
     }
 
@@ -461,7 +468,7 @@ connect_ready_cb(GIOChannel   *iochan,
                                       g_strerror(errno));
     }
 
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   } else {
     connected_to_dispatcher(world);
@@ -491,7 +498,8 @@ connect_ready_cb(GIOChannel   *iochan,
     return FALSE;
   }
 
-  ret = getsockopt(world->sock, SOL_SOCKET, SO_ERROR, &sockerror, &optlen);
+  ret = getsockopt(world->sock, SOL_SOCKET, SO_ERROR,
+                   (char *) &sockerror, &optlen);
   if (ret != 0) {
     sockerror = WSAGetLastError();
   }
@@ -503,7 +511,7 @@ connect_ready_cb(GIOChannel   *iochan,
       world->errmsg = g_strdup_printf(_("Unable to connect to proxy host %s: Error #%d."),
                                       world->proxy_used.server, sockerror);
     }
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -521,7 +529,7 @@ connect_ready_cb(GIOChannel   *iochan,
     world->errmsg = g_strdup_printf(_("Unable to connect to proxy host %s: Error #%d."),
                                     world->proxy_used.server, sockerror);
   }
-  disconnect_world(world);
+  disconnect_world(world, FALSE);
   return FALSE;
 }
 #endif
@@ -539,12 +547,127 @@ connected_to_dispatcher(World *world)
     return proxy_socks4_connected_to(world);
   } else if (world->proxy_used.type == PROXY_SOCKS5) {
     return proxy_socks5_connected_to(world);
+  } else {
+    return perform_tls_handshake(world);
+  }
+}
+
+
+#ifdef HAVE_LIBGNUTLS
+gboolean
+perform_tls_handshake(World *world)
+{
+  gchar *status_text;
+
+  if (world->use_tls) {
+    status_text = g_strdup_printf(_("Performing SSL handshake..."));
+    ansitextview_append_string_with_fgcolor(world->gui, status_text,
+                                            globalPrefs.idxInfoMsgColor);
+    ansitextview_append_string(world->gui, "\n");
+    gtk_label_set_text(world->gui->lblStatus, status_text);
+    g_free(status_text);
+
+#ifndef __x86_64      /* This is to remove a warning about pointer sizes */
+    gnutls_transport_set_ptr(world->tls_session,
+                             (gnutls_transport_ptr) world->sock);
+#else
+    gnutls_transport_set_ptr(world->tls_session,
+                             (gnutls_transport_ptr) (long) world->sock);
+#endif
+    /* Register callback to perform the TLS handshake
+     * The first action is we sending data (ClientHello) */
+    world->tls_handshake_dir = 1;
+    world->io_watch_id = g_io_add_watch(world->iochan,
+                                        G_IO_OUT | G_IO_HUP | G_IO_ERR,
+                                        tls_handshake_cb,
+                                        world);
+    g_timeout_add_seconds(60, tls_handshake_timeout_cb, world);
+    return TRUE;
   } else {
     return connected_to(world);
   }
 }
 
 
+static
+gboolean
+tls_handshake_cb(GIOChannel   *iochan,
+                 GIOCondition  cond,
+                 gpointer      data)
+{
+  World *world = (World *) data;
+  int ret;
+
+  ret = gnutls_handshake(world->tls_session);
+
+  if (ret == GNUTLS_E_SUCCESS) {
+    /* Sucess */
+    connected_to(world);
+    /* Do not call the function again */
+    return FALSE;
+  }
+
+  if (gnutls_error_is_fatal(ret) != 0) {
+    /* Fatal error */
+    gnutls_deinit(world->tls_session);
+    world->errmsg = g_strdup_printf(_("SSL Handshake failed: %s"),
+                                    gnutls_strerror(ret));
+    disconnect_world(world, FALSE);
+    /* Do not call the function again */
+    return FALSE;
+  }
+
+  /* Continue when more data is available, watching the direction. */
+  int dir = gnutls_record_get_direction(world->tls_session);
+  if (dir != world->tls_handshake_dir) {
+    GIOCondition cond = G_IO_HUP | G_IO_ERR;
+    if (dir == 0) {
+      cond |= G_IO_IN;
+    } else {
+      cond |= G_IO_OUT;
+    }
+
+    world->io_watch_id = g_io_add_watch(world->iochan,
+                                        cond,
+                                        tls_handshake_cb,
+                                        world);
+    world->tls_handshake_dir = dir;
+    /* A new callback has been set up for the new direction, remove this one */
+    return FALSE;
+  } else {
+    /* Continue */
+    return TRUE;
+  }
+}
+
+
+static
+gboolean
+tls_handshake_timeout_cb(gpointer data)
+{
+  /* Called 60 seconds after a TLS handshake has started. If the connection
+   * has not completed, aborts it. */
+  World *world = (World *) data;
+
+  // The world might have been closed
+  if (g_list_index(open_worlds, world) != -1 && world->connecting) {
+    gnutls_deinit(world->tls_session);
+    world->errmsg = g_strdup_printf(_("SSL Handshake took too long."));
+    disconnect_world(world, FALSE);
+  }
+
+  /* Never called again */
+  return FALSE;
+}
+#else
+gboolean
+perform_tls_handshake(World *world)
+{
+  return connected_to(world);
+}
+#endif
+
+
 gboolean
 connected_to(World *world)
 {
@@ -552,48 +675,6 @@ connected_to(World *world)
      or indirectly. */
   gchar *status_text;
 
-#ifdef HAVE_LIBGNUTLS
-  if (world->use_tls) {
-#ifndef __MINGW32__
-    int    flags;
-#else /* defined __MINGW32__ */
-    u_long flags;
-#endif
-    int ret;
-
-    status_text = g_strdup_printf(_("Performing SSL handshake..."));
-    ansitextview_append_string_with_fgcolor(world->gui, status_text,
-                                            globalPrefs.idxInfoMsgColor);
-    ansitextview_append_string(world->gui, "\n");
-    gtk_label_set_text(world->gui->lblStatus, status_text);
-    g_free(status_text);
-
-#ifndef __MINGW32__
-    flags = fcntl(world->sock, F_GETFL);
-    flags &= ~O_NONBLOCK;
-    fcntl(world->sock, F_SETFL, flags);
-#else /* defined __MINGW32__ */
-    flags = 0;
-    ioctlsocket(sock, FIONBIO, &flags);
-#endif
-    gnutls_transport_set_ptr(world->tls_session,
-#ifndef __x86_64      /* This is to remove a warning about pointer sizes */
-                               (gnutls_transport_ptr) world->sock);
-#else
-                               (gnutls_transport_ptr) (long) world->sock);
-#endif
-    /* Perform the TLS handshake */
-    ret = gnutls_handshake(world->tls_session);
-    if (ret < 0) {
-      gnutls_deinit(world->tls_session);
-      world->errmsg = g_strdup_printf(_("SSL Handshake failed: %s"),
-                                      gnutls_strerror(ret));
-      disconnect_world(world);
-      return TRUE;
-    }
-  }
-#endif
-
   world->connected         = TRUE;
   world->connecting        = FALSE;
   world->connected_time    = time(NULL);
@@ -654,7 +735,16 @@ connected_to(World *world)
 
 
 gboolean
-disconnect_world(gpointer data)
+disconnect_world_idle(gpointer data)
+{
+  disconnect_world(data, TRUE);
+
+  return FALSE;
+}
+
+
+void
+disconnect_world(gpointer data, gboolean run_hook)
 {
   /* This function is called as an idle function, so it must return FALSE. */
   World *world;
@@ -664,7 +754,7 @@ disconnect_world(gpointer data)
   world = (World *) data;
 
   if (!world->connected && !world->connecting) {
-    return FALSE;
+    return;
   }
 
   /* In case we are still connecting and it has not been free'd */
@@ -673,26 +763,17 @@ disconnect_world(gpointer data)
     world->addrinfo = NULL;
   }
 
-  msg = close_connection(world);
+  msg = close_connection(world, run_hook);
 
   execute_hook(world, "OnDisconnect", NULL);
 
   currentWorld = NULL;
 
-  /* Remove the tooltip */
-  worldgui_destroy_tooltip_window(world->gui);
-  if (world->gui->tooltip_timeout_id) {
-    g_source_remove(world->gui->tooltip_timeout_id);
-    world->gui->tooltip_timeout_id = 0;
-  }
-
   /* Here we decide what to do */
   action = disconnected_msgbox(msg, TRUE);
   g_free(msg);
 
   disconnected_decide_action(world, action);
-
-  return FALSE;
 }
 
 
@@ -702,7 +783,7 @@ reconnect_world(gpointer data)
   World *world = (World *) data;
 
   if (world->connected || world->connecting)
-    g_free(close_connection(world));
+    g_free(close_connection(world, TRUE));
 
   /* This function is called as an idle function, so it must return FALSE. */
   disconnected_decide_action((World *) data, RESP_RECONNECT);
@@ -759,13 +840,15 @@ disconnected_decide_action(World *world, gint action)
 
 
 gchar *
-close_connection(World *world)
+close_connection(World *world, gboolean run_hook)
 {
   /* Note: this function should only be called if the world is
      connected or connecting. */
   gchar *msg;
 
-  execute_hook(world, "OnCloseConnected", NULL);
+  if (run_hook) {
+    execute_hook(world, "OnCloseConnected", NULL);
+  }
 
   flush_buffer(world, 2);
   if (world->errmsg) {
@@ -778,6 +861,11 @@ close_connection(World *world)
                                           globalPrefs.idxInfoMsgColor);
   ansitextview_append_string(world->gui, "\n");
 
+#ifdef USE_LIBGNUTLS
+  if (world->use_tls) {
+    gnutls_bye(world->tls_session);
+  }
+#endif
 #ifndef __MINGW32__
   close(world->sock);
 #else /* defined __MINGW32__ */
@@ -834,15 +922,40 @@ data_ready_cb(GIOChannel   *iochan,
                   MAX_BUFFER - world->pbuf_size);
   if (nread <= 0) {      /* Connection closed */
     if (nread != 0) {    /* Error */
+#ifdef HAVE_LIBGNUTLS
+      if (world->use_tls) {
+        if (gnutls_error_is_fatal(nread) == 0) {
+          return TRUE;
+        }
+
+        world->errmsg = g_strdup_printf(_("Disconnected from host %s: %s."),
+                                        world->host, gnutls_strerror(nread));
+      } else {
+#endif
 #ifndef __MINGW32__
-      world->errmsg = g_strdup_printf(_("Disconnected from host %s: %s."),
+        if (errno == EAGAIN || errno == EWOULDBLOCK) {
+          return TRUE;
+        }
+
+        world->errmsg = g_strdup_printf(_("Disconnected from host %s: %s."),
                                       world->host, g_strerror(errno));
 #else /* defined __MINGW32__ */
-      world->errmsg = g_strdup_printf(_("Disconnected from host %s: error #%d."),
-                                      world->host, WSAGetLastError());
+        int errcode = WSAGetLastError();
+
+        if (errcode == WSAEINTR
+            || errcode == WSAEINPROGRESS
+            || errcode == WSAEWOULDBLOCK) {
+          return TRUE;
+        }
+
+        world->errmsg = g_strdup_printf(_("Disconnected from host %s: error #%d."),
+                                        world->host, errcode);
+#endif
+#ifdef HAVE_LIBGNUTLS
+      }
 #endif
     }
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
