@@ -1,7 +1,7 @@
-/* $Id: perlscript.c 1033 2009-02-19 21:34:07Z ekalin $ */
+/* $Id: perlscript.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <time.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 /* Perl includes */
 #include <EXTERN.h>
@@ -299,7 +298,8 @@ perl_substitute(PerlInterpreter *interpreter,
                 const char      *line,
                 SV              *pattern_re,
                 const char      *substitution,
-                int              perl_eval)
+                int              perl_eval,
+                int              global)
 {
   SV  *string;
   SV  *re;
@@ -318,8 +318,9 @@ perl_substitute(PerlInterpreter *interpreter,
   re = get_sv("_re", TRUE);
   sv_setsv(re, pattern_re);
   /* Create the command to run */
-  command = newSVpvf("$__aliasmatched = ($aliascmd =~ s\x05$_re\x05%s\x05%s);",
+  command = newSVpvf("$__aliasmatched = ($aliascmd =~ s\x05$_re\x05%s\x05%s%s);",
                      substitution,
+                     global    ? "g" : "",
                      perl_eval ? "e" : "");
 
   /* Run the command */
@@ -492,7 +493,7 @@ XS_dc(pTHX_ CV *perlcv)
      can destroy the Perl interpreter if another world is loaded, and
      this would cause a segfault in the function that evaluates
      Perl commands. */
-  g_idle_add(disconnect_world, world);
+  g_idle_add(disconnect_world_idle, world);
 
   XSRETURN_EMPTY;
 }
@@ -508,7 +509,7 @@ XS_reconnect(pTHX_ CV *perlcv)
   FETCHWORLD;
 
   if (world->connected) {
-    g_free(close_connection(world));
+    g_free(close_connection(world, TRUE));
   }
 
   /* We don't call the function directly because disconnect_world
@@ -531,7 +532,7 @@ XS_connectother(pTHX_ CV *perlcv)
   FETCHWORLD;
 
   if (world->connected) {
-    g_free(close_connection(world));
+    g_free(close_connection(world, TRUE));
   }
 
   /* We don't call the function directly because disconnect_world
@@ -2994,7 +2995,7 @@ XS__delhook(pTHX_ CV *perlcv)
 
   hookname = SvPV_nolen(ST(1));
   i = SvIV(ST(2));
-  if (delete_hook(world, hookname, i)) {
+  if (delete_hook(world, hookname, i, NULL)) {
     XSRETURN_YES;
   } else {
     XSRETURN_NO;
@@ -3167,7 +3168,7 @@ static
 void
 XS_listplugin(pTHX_ CV *perlcv)
 {
-  int      i;
+  guint    i;
   guint    total_width;
   guint    field1_width;
   guint    field2_width;
@@ -3360,6 +3361,8 @@ XS__stopplugin(pTHX_ CV *perlcv)
                                     plugin);
     we_lplugin_insert_plugin(world, plugin);
   } else {
+    remove_plugin_objects(world, plugin);
+
     g_free(plugin);
   }
 
