@@ -1,7 +1,7 @@
-/* $Id: perlscript.h 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: perlscript.h 1178 2011-05-29 13:21:14Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,6 +96,7 @@
 #undef recv
 #undef close
 #undef getsockopt
+#undef setsockopt
 
 #undef gettimeofday
 #undef malloc
@@ -132,7 +133,8 @@ char *perl_substitute(PerlInterpreter *interpreter,
                       const char *line,
                       SV         *pattern_re,
                       const char *substitution,
-                      int         perl_eval);
+                      int         perl_eval,
+                      int         global);
 SV   *precompute_re(PerlInterpreter *interpreter,
                     const char      *re_string,
                     int              ignore_case);
