@@ -1,7 +1,7 @@
-/* $Id: prefs.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: prefs.c 1175 2011-05-28 14:24:01Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gmodule.h>
 
 #include "kildclient.h"
@@ -48,18 +47,6 @@ Prefs globalPrefs;
 /*************************
  * File Global variables *
  *************************/
-static char *colorNames[] = {
-  N_("Black"),
-  N_("Red"),
-  N_("Green"),
-  N_("Yellow"),
-  N_("Blue"),
-  N_("Magenta"),
-  N_("Cyan"),
-  N_("White"),
-  N_("Default"),
-};
-#define N_COLOR_NAMES 9
 #define MAX_OLDSAVE_COLOR 8
 
 
@@ -75,8 +62,7 @@ static void     combo_changed_cb(GtkComboBox *combo, gpointer data);
 static gboolean txt_cmd_focus_out_cb(GtkWidget     *widget,
                                      GdkEventFocus *event,
                                      gpointer       data);
-static void     cmbProxyType_changed_cb(GtkComboBox *combo, gpointer data);
-/* Glade signals */
+/* XML UI signals */
 G_MODULE_EXPORT void     cmd_group_size_changed_cb(GtkSpinButton *spin,
                                                    gpointer       data);
 G_MODULE_EXPORT void     cmd_delay_changed_cb(GtkSpinButton *spin,
@@ -87,134 +73,135 @@ G_MODULE_EXPORT void     chk_hide_single_tab_toggled_cb(GtkToggleButton *button,
                                                         gpointer data);
 G_MODULE_EXPORT void     chk_urgency_toggled_cb(GtkToggleButton *button,
                                                 gpointer data);
+G_MODULE_EXPORT void     cmbProxyType_changed_cb(GtkComboBox *combo, gpointer data);
 
 
 void
 show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
 {
   static GtkWidget *dialog = NULL;
-  GladeXML         *xml;
   GtkNotebook      *notebook;
-  GtkWidget        *cmbInfo;
-  GtkWidget        *cmbCmdEcho;
-  GtkWidget        *cmbTabPos;
-  GtkWidget        *chkHideSingleTab;
-  GtkWidget        *chkUrgency;
-  GtkWidget        *spnGroupSize;
-  GtkWidget        *spnDelay;
+  GObject          *cmbInfo;
+  GObject          *cmbCmdEcho;
+  GtkComboBox      *cmbTabPos;
+  GtkToggleButton  *chkHideSingleTab;
+  GtkToggleButton  *chkUrgency;
+  GtkSpinButton    *spnGroupSize;
+  GtkSpinButton    *spnDelay;
   GtkComboBox      *cmbProxyType;
-  GtkWidget        *txtProxyServer;
-  GtkWidget        *txtProxyPort;
-  GtkWidget        *txtProxyUser;
-  GtkWidget        *txtProxyPassword;
+  GObject          *txtProxyServer;
+  GObject          *txtProxyPort;
+  GObject          *txtProxyUser;
+  GObject          *txtProxyPassword;
 #ifndef __WIN32__
-  GtkWidget        *txtBrowserCommand;
-  GtkWidget        *txtAudioPlayerCommand;
+  GObject          *txtBrowserCommand;
+  GObject          *txtAudioPlayerCommand;
 #else
   GtkWidget        *tabPrograms;
 #endif
-  GtkBox           *vbox;
 
   if (!dialog) {
-    xml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                        "dlgPreferences", NULL);
-    dialog = glade_xml_get_widget(xml, "dlgPreferences");
-    notebook = (GtkNotebook *) glade_xml_get_widget(xml, "ntbkPrefs");
-    vbox = GTK_BOX(glade_xml_get_widget(xml, "vbox_int_kc_colors"));
-    glade_xml_signal_autoconnect(xml);
+    GError *error     = NULL;
 
-    /* Cannot connect swapped signals from Glade */
-    g_signal_connect_swapped(G_OBJECT(dialog), "response",
-                             G_CALLBACK(gtk_widget_hide), dialog);
+    if (!gtk_builder_add_from_file(main_builder,
+                                   get_kildclient_installed_file("dlgPreferences.ui"),
+                                   &error)) {
+      g_warning(_("Error loading UI from XML file: %s"), error->message);
+      g_error_free(error);
+      return;
+    }
 
-    /* For some reason, this does not work from Glade */
-    cmbInfo = create_combo_with_colors();
+    dialog = GTK_WIDGET(gtk_builder_get_object(main_builder, "dlgPreferences"));
+    notebook = GTK_NOTEBOOK(gtk_builder_get_object(main_builder, "ntbkPrefs"));
+
+    /* Combos with colors */
+    cmbInfo = gtk_builder_get_object(main_builder, "cmbInfoColor");
     gtk_combo_box_set_active(GTK_COMBO_BOX(cmbInfo),
                              globalPrefs.idxInfoMsgColor);
-    g_signal_connect(G_OBJECT(cmbInfo), "changed",
+    g_signal_connect(cmbInfo, "changed",
                      G_CALLBACK(combo_changed_cb),
                      &globalPrefs.idxInfoMsgColor);
-    gtk_box_pack_start(vbox, cmbInfo, TRUE, TRUE, 0);
-    gtk_box_reorder_child(vbox, cmbInfo, 1);
 
-    cmbCmdEcho = create_combo_with_colors();
+    cmbCmdEcho = gtk_builder_get_object(main_builder, "cmbCmdEchoColor");
     gtk_combo_box_set_active(GTK_COMBO_BOX(cmbCmdEcho),
                              globalPrefs.idxCmdEchoColor);
-    g_signal_connect(G_OBJECT(cmbCmdEcho), "changed",
+    g_signal_connect(cmbCmdEcho, "changed",
                      G_CALLBACK(combo_changed_cb),
                      &globalPrefs.idxCmdEchoColor);
-    gtk_box_pack_start(vbox, cmbCmdEcho, TRUE, TRUE, 0);
 
     /* Load current tab position */
-    cmbTabPos = glade_xml_get_widget(xml, "cmbTabPos");
-    gtk_combo_box_set_active(GTK_COMBO_BOX(cmbTabPos),
-                             globalPrefs.tab_position);
+    cmbTabPos = GTK_COMBO_BOX(gtk_builder_get_object(main_builder, "cmbTabPos"));
+    gtk_combo_box_set_active(cmbTabPos, globalPrefs.tab_position);
 
-    chkHideSingleTab = glade_xml_get_widget(xml, "chkHideSingleTab");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkHideSingleTab),
+    chkHideSingleTab
+      = GTK_TOGGLE_BUTTON(gtk_builder_get_object(main_builder,
+                                                 "chkHideSingleTab"));
+    gtk_toggle_button_set_active(chkHideSingleTab,
                                  globalPrefs.hide_single_tab);
 
     /* Load use of urgency hint */
-    chkUrgency = glade_xml_get_widget(xml, "chkUrgency");
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkUrgency),
-                                 globalPrefs.urgency_hint);
+    chkUrgency
+      = GTK_TOGGLE_BUTTON(gtk_builder_get_object(main_builder, "chkUrgency"));
+    gtk_toggle_button_set_active(chkUrgency, globalPrefs.urgency_hint);
 
     /* Load delays */
-    spnGroupSize = glade_xml_get_widget(xml, "spnGroupSize");
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnGroupSize),
-                              globalPrefs.multi_cmd_group_size);
-    spnDelay = glade_xml_get_widget(xml, "spnPrefsDelay");
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnDelay),
-                              globalPrefs.multi_cmd_delay);
+    spnGroupSize
+      = GTK_SPIN_BUTTON(gtk_builder_get_object(main_builder, "spnGroupSize"));
+    gtk_spin_button_set_value(spnGroupSize, globalPrefs.multi_cmd_group_size);
+    spnDelay
+      = GTK_SPIN_BUTTON(gtk_builder_get_object(main_builder, "spnPrefsDelay"));
+    gtk_spin_button_set_value(spnDelay, globalPrefs.multi_cmd_delay);
 
     /* Load proxy settings */
-    cmbProxyType = GTK_COMBO_BOX(glade_xml_get_widget(xml, "cmbGProxyType"));
-    txtProxyServer   = glade_xml_get_widget(xml, "txtGProxyServer");
+    txtProxyServer   = gtk_builder_get_object(main_builder, "txtGProxyServer");
     gtk_entry_set_text(GTK_ENTRY(txtProxyServer), globalPrefs.proxy.server);
-    g_signal_connect(G_OBJECT(txtProxyServer), "focus_out_event",
+    g_signal_connect(txtProxyServer, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.proxy.server);
-    txtProxyPort     = glade_xml_get_widget(xml, "txtGProxyPort");
+    txtProxyPort     = gtk_builder_get_object(main_builder, "txtGProxyPort");
     gtk_entry_set_text(GTK_ENTRY(txtProxyPort), globalPrefs.proxy.port);
-    g_signal_connect(G_OBJECT(txtProxyPort), "focus_out_event",
+    g_signal_connect(txtProxyPort, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.proxy.port);
-    txtProxyUser     = glade_xml_get_widget(xml, "txtGProxyUser");
+    txtProxyUser     = gtk_builder_get_object(main_builder, "txtGProxyUser");
     gtk_entry_set_text(GTK_ENTRY(txtProxyUser), globalPrefs.proxy.user);
-    g_signal_connect(G_OBJECT(txtProxyUser), "focus_out_event",
+    g_signal_connect(txtProxyUser, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.proxy.user);
-    txtProxyPassword = glade_xml_get_widget(xml, "txtGProxyPassword");
+    txtProxyPassword = gtk_builder_get_object(main_builder, "txtGProxyPassword");
     gtk_entry_set_text(GTK_ENTRY(txtProxyPassword), globalPrefs.proxy.password);
-    g_signal_connect(G_OBJECT(txtProxyPassword), "focus_out_event",
+    g_signal_connect(txtProxyPassword, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.proxy.password);
     /* Remove "Use Global Settings" - these are the global settings */
+    cmbProxyType
+      = GTK_COMBO_BOX(gtk_builder_get_object(main_builder, "cmbGProxyType"));
     gtk_combo_box_remove_text(cmbProxyType, 0);
     gtk_combo_box_set_active(cmbProxyType, globalPrefs.proxy.type);
-    g_signal_connect(G_OBJECT(cmbProxyType), "changed",
-                     G_CALLBACK(cmbProxyType_changed_cb), NULL);
 
 #ifndef __WIN32__
     /* Load commands */
-    txtBrowserCommand = glade_xml_get_widget(xml, "txtBrowserCommand");
+    txtBrowserCommand = gtk_builder_get_object(main_builder, "txtBrowserCommand");
     gtk_entry_set_text(GTK_ENTRY(txtBrowserCommand),
                        globalPrefs.browser_command);
-    g_signal_connect(G_OBJECT(txtBrowserCommand), "focus_out_event",
+    g_signal_connect(txtBrowserCommand, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.browser_command);
-    txtAudioPlayerCommand = glade_xml_get_widget(xml, "txtAudioPlayerCommand");
+    txtAudioPlayerCommand
+      = gtk_builder_get_object(main_builder, "txtAudioPlayerCommand");
     gtk_entry_set_text(GTK_ENTRY(txtAudioPlayerCommand),
                        globalPrefs.audio_player_command);
-    g_signal_connect(G_OBJECT(txtAudioPlayerCommand), "focus_out_event",
+    g_signal_connect(txtAudioPlayerCommand, "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.audio_player_command);
 #else /* defined __WIN32__ */
-    tabPrograms = glade_xml_get_widget(xml, "prefs_panel_progs");
+    tabPrograms
+      = GTK_WIDGET(gtk_builder_get_object(main_builder, "prefs_panel_progs"));
     gtk_notebook_remove_page(notebook,
                              gtk_notebook_page_num(notebook, tabPrograms));
 #endif
-    g_object_unref(xml);
+
+    gtk_builder_connect_signals(main_builder, NULL);
 
     /* For some reason the proxy page insists on being the default selected. */
     gtk_widget_show_all(dialog);
@@ -296,7 +283,6 @@ txt_cmd_focus_out_cb(GtkWidget     *widget,
 }
 
 
-static
 void
 cmbProxyType_changed_cb(GtkComboBox *combo, gpointer data)
 {
@@ -550,26 +536,3 @@ create_kildclient_directory(void)
 
   return kilddir;
 }
-
-
-GtkWidget *
-create_combo_with_colors(void)
-{
-  GtkWidget   *combow;
-  GtkComboBox *combo;
-  int i;
-
-  combow = gtk_combo_box_new_text();
-  combo = GTK_COMBO_BOX(combow);
-
-  for (i = 0; i < N_COLOR_NAMES; ++i) {
-    gtk_combo_box_append_text(combo, _(colorNames[i]));
-  }
-  for (i = 0; i < N_COLOR_NAMES; ++i) {
-    gchar *str = g_strdup_printf(_("%s (Bold)"), _(colorNames[i]));
-    gtk_combo_box_append_text(combo, str);
-    g_free(str);
-  }
-
-  return combow;
-}
