@@ -1,7 +1,7 @@
-/* $Id: proxy.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: proxy.c 1154 2011-03-26 22:57:37Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <zlib.h>
 
 #ifndef __MINGW32__
@@ -191,7 +190,7 @@ proxy_socks4_resolved_real_host_cb(gpointer data)
       world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
                                       world->host, msg);
       g_free(msg);
-      disconnect_world(world);
+      disconnect_world(world, FALSE);
     }
     g_free(resolve_data);
 
@@ -223,7 +222,7 @@ proxy_socks4_resolved_real_host_cb(gpointer data)
   resolve_data->callback  = continue_connecting_cb;
   if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
     world->errmsg = g_strdup(_("Could not start name-resolution thread."));
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -296,7 +295,7 @@ proxy_socks4_data_ready_cb(GIOChannel   *iochan,
   if (nread == -1) {
     world->errmsg = g_strdup(_("Error while reading from proxy host."));
     world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -311,7 +310,7 @@ proxy_socks4_data_ready_cb(GIOChannel   *iochan,
   switch (world->inbuffer[1]) {
   case SOCKS4_REP_OK:
     world->pbuf_size = 0;
-    connected_to(world);
+    perform_tls_handshake(world);
     return FALSE; /* Not to be called again. */
 
   case SOCKS4_REP_FAIL:
@@ -328,7 +327,7 @@ proxy_socks4_data_ready_cb(GIOChannel   *iochan,
   }
 
   world->errmsg = g_strdup_printf(_("Proxy request failed: %s."), msg);
-  disconnect_world(world);
+  disconnect_world(world, FALSE);
   return FALSE;
 }
 
@@ -425,7 +424,7 @@ proxy_socks5_data_ready_initial(World *world)
   if (nread == -1) {
     world->errmsg = g_strdup(_("Error while reading from proxy host."));
     world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -440,7 +439,7 @@ proxy_socks5_data_ready_initial(World *world)
   case SOCKS5_AUTH_REJECTED:
     world->errmsg = _("No acceptable proxy authentication methods.");
     world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
 
   case SOCKS5_AUTH_NONE:
@@ -470,7 +469,7 @@ proxy_socks5_data_ready_auth(World *world)
   if (nread == -1) {
     world->errmsg = g_strdup(_("Error while reading from proxy host."));
     world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -488,7 +487,7 @@ proxy_socks5_data_ready_auth(World *world)
 
   world->errmsg = g_strdup(_("Proxy authentication failed."));
   world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-  disconnect_world(world);
+  disconnect_world(world, FALSE);
   return FALSE;
 }
 
@@ -531,7 +530,7 @@ proxy_socks5_data_ready_connection(World *world)
     default: /* Should never happen */
       world->errmsg = g_strdup(_("Unrecognized response from proxy host."));
       world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-      disconnect_world(world);
+      disconnect_world(world, FALSE);
       return FALSE;
     }
   }
@@ -544,7 +543,7 @@ proxy_socks5_data_ready_connection(World *world)
   if (nread == -1) {
     world->errmsg = g_strdup(_("Error while reading from proxy host."));
     world->pbuf_size = 0; /* disconnect_world() flushes the buffer. */
-    disconnect_world(world);
+    disconnect_world(world, FALSE);
     return FALSE;
   }
 
@@ -558,18 +557,18 @@ proxy_socks5_data_ready_connection(World *world)
   /* Determine the reply. */
   world->pbuf_size = 0;
   if (world->inbuffer[1] == SOCKS5_CONN_REP_SUCCESS) {
-    connected_to(world);
+    perform_tls_handshake(world);
     return FALSE; /* Not to be called again. */
   }
 
   if (world->inbuffer[1] > SOCKS5_CONN_MAX_FAILED_REASON) {
-    msg = _("unkown reason code");
+    msg = _("unknown reason code");
   } else {
     msg = _(proxy_socks5_rejection_reasons[world->inbuffer[1]]);
   }
 
   world->errmsg = g_strdup_printf(_("Proxy request failed: %s."), msg);
-  disconnect_world(world);
+  disconnect_world(world, FALSE);
   return FALSE;
 }
 
