@@ -252,12 +252,10 @@ simo_combo_box_class_init(SimoComboBoxClass *klass)
   GObjectClass      *g_object_class;
   GtkObjectClass    *gtk_object_class;
   GtkWidgetClass    *widget_class;
-  GtkContainerClass *container_class;
 
   g_object_class   = G_OBJECT_CLASS (klass);
   gtk_object_class = (GtkObjectClass*) klass;
   widget_class     = (GtkWidgetClass*) klass;
-  container_class  = (GtkContainerClass*) klass;
 
   //widget_class->show = simo_combo_box_show;
   //widget_class->hide = simo_combo_box_hide;
@@ -281,42 +279,42 @@ simo_combo_box_class_init(SimoComboBoxClass *klass)
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, clicked),
                                             NULL, NULL,
-                                            gtk_marshal_VOID__VOID,
+                                            g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[POPUP_DISPLAYED] = g_signal_new("popup-displayed",
                                                     G_OBJECT_CLASS_TYPE(gtk_object_class),
                                                     G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                                     G_STRUCT_OFFSET(SimoComboBoxClass, popup_displayed),
                                                     NULL, NULL,
-                                                    gtk_marshal_VOID__VOID,
+                                                    g_cclosure_marshal_VOID__VOID,
                                                     G_TYPE_NONE, 0);
   combo_box_signals[POPUP_CLOSED] = g_signal_new("popup-closed",
                                                  G_OBJECT_CLASS_TYPE(gtk_object_class),
                                                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                                  G_STRUCT_OFFSET(SimoComboBoxClass, popup_closed),
                                                  NULL, NULL,
-                                                 gtk_marshal_VOID__VOID,
+                                                 g_cclosure_marshal_VOID__VOID,
                                                  G_TYPE_NONE, 0);
   combo_box_signals[CHANGED] = g_signal_new("changed",
                                             G_OBJECT_CLASS_TYPE(gtk_object_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, changed),
                                             NULL, NULL,
-                                            gtk_marshal_VOID__VOID,
+                                            g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[ACTIVATE] = g_signal_new("activate",
                                             G_OBJECT_CLASS_TYPE(gtk_object_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, activate),
                                             NULL, NULL,
-                                            gtk_marshal_VOID__VOID,
+                                            g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
   combo_box_signals[SIZE_CHANGED] = g_signal_new("size_changed",
                                             G_OBJECT_CLASS_TYPE(gtk_object_class),
                                             G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                             G_STRUCT_OFFSET(SimoComboBoxClass, size_changed),
                                             NULL, NULL,
-                                            gtk_marshal_VOID__VOID,
+                                            g_cclosure_marshal_VOID__VOID,
                                             G_TYPE_NONE, 0);
 
   /* properties */
@@ -449,7 +447,7 @@ simo_combo_box_init(SimoComboBox *combo_box)
   gtk_tree_view_append_column( combo_box->treeview, combo_box->column );
   //#ifdef NDEBUG // only enable for release until warnnigs are fixed
 #ifdef GDK_WINDOWING_WIN32 /* once Linux version becomes 2.6 we'll remove the macro */
-  g_object_set( cell, "ellipsize", PANGO_ELLIPSIZE_END, 0 );
+  g_object_set( cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
 #endif
   //#endif
   selection = gtk_tree_view_get_selection( combo_box->treeview );
@@ -468,9 +466,10 @@ simo_combo_box_init(SimoComboBox *combo_box)
   gtk_window_set_decorated( GTK_WINDOW( mthis->popup_window ), FALSE );
   gtk_window_set_resizable( GTK_WINDOW( mthis->popup_window ), TRUE );
   gtk_widget_realize( mthis->popup_window );
-  simo_cursor_set_delay(mthis->popup_window->window, mthis->hand );
-  mask = gdk_window_get_events(mthis->popup_window->window);
-  gdk_window_set_events(mthis->popup_window->window,
+  simo_cursor_set_delay(gtk_widget_get_window(mthis->popup_window),
+                        mthis->hand );
+  mask = gdk_window_get_events(gtk_widget_get_window(mthis->popup_window));
+  gdk_window_set_events(gtk_widget_get_window(mthis->popup_window),
                         mask | GDK_BUTTON_PRESS_MASK
                         | GDK_BUTTON_RELEASE_MASK | GDK_KEY_PRESS_MASK
                         | GDK_KEY_RELEASE_MASK);
@@ -527,13 +526,15 @@ simo_combo_box_init(SimoComboBox *combo_box)
   combo_box->editable = TRUE;
 
   // give an initial allocation size to the entry box
-  combo_box->priv->entry_alloc.width = COMBO_MIN_WIDTH - 25;
-  combo_box->entry->allocation.width = COMBO_MIN_WIDTH - 25;
-  combo_box->entry->allocation.height = COMBO_MIN_HEIGHT;
+  // Removed by Eduardo Kalinowski. Seems not to be necessary, and
+  // does not compile when GSEAL_ENABLE is defined.
+  //combo_box->priv->entry_alloc.width = COMBO_MIN_WIDTH - 25;
+  //combo_box->entry->allocation.width = COMBO_MIN_WIDTH - 25;
+  //combo_box->entry->allocation.height = COMBO_MIN_HEIGHT;
   //	g_print( "setting fixed entry size to default\n" );
-  combo_box->priv->fixed_entry_size = -1;
-  mthis->label_button->allocation.width = COMBO_MIN_HEIGHT;
-  mthis->label_button->allocation.height = COMBO_MIN_HEIGHT;
+  //combo_box->priv->fixed_entry_size = -1;
+  //mthis->label_button->allocation.width = COMBO_MIN_HEIGHT;
+  //mthis->label_button->allocation.height = COMBO_MIN_HEIGHT;
 
   //gtk_container_remove( GTK_CONTAINER( combo_box->button ),
   //                     gtk_bin_get_child( GTK_BIN( combo_box->button ) ) );
@@ -832,6 +833,8 @@ simo_combo_box_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
   GtkAllocation ealloc;
   SimoComboBox *combo = SIMO_COMBO_BOX( widget );
+  GtkAllocation combobtnAlloc;
+  GtkAllocation vboxarrowsAlloc;
 
   //	g_print( "ENTER: simo_combo_box_size_allocate\n" );
 
@@ -840,11 +843,20 @@ simo_combo_box_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
   GTK_WIDGET_CLASS( parent_class )->size_allocate( widget, allocation );
   //		g_print( "combo box allocate: %d:%d:%d:%d:%d\n", allocation->x, allocation->y, allocation->width, allocation->height, combo->priv->fixed_entry_size );
+
+#if GTK_MINOR_VERSION < 18
+  combobtnAlloc   = combo->button->allocation;
+  vboxarrowsAlloc = combo->priv->vbox_arrows->allocation;
+#else
+  gtk_widget_get_allocation(combo->button, &combobtnAlloc);
+  gtk_widget_get_allocation(combo->priv->vbox_arrows, &vboxarrowsAlloc);
+#endif
+
   ealloc.x = allocation->x;
   ealloc.y = allocation->y;
   ealloc.width = allocation->width
-    - combo->button->allocation.width
-    - combo->priv->vbox_arrows->allocation.width;
+    - combobtnAlloc.width
+    - vboxarrowsAlloc.width;
   ealloc.height = allocation->height;
 
   if (combo->priv->n_lines > 1) {
@@ -1009,6 +1021,7 @@ simo_combo_box_update_pressed( GtkWidget *widget,
   gdouble x, y;
   gint x1, x2, y1, y2;
   gint xoffset, yoffset;
+  GtkAllocation alloc;
   //	GtkWidget *event_widget;
 
   if( event->button != 1 ){
@@ -1016,15 +1029,22 @@ simo_combo_box_update_pressed( GtkWidget *widget,
   }
   x = event->x_root;
   y = event->y_root;
-  gdk_window_get_root_origin( widget->window, &xoffset, &yoffset );
+  gdk_window_get_root_origin( gtk_widget_get_window(widget),
+                              &xoffset, &yoffset );
 
-  xoffset += widget->allocation.x;
-  yoffset += widget->allocation.y;
+#if GTK_MINOR_VERSION < 18
+  alloc = widget->allocation;
+#else
+  gtk_widget_get_allocation(widget, &alloc);
+#endif
 
-  x1 = widget->allocation.x + xoffset;
-  y1 = widget->allocation.y + yoffset;
-  x2 = x1 + widget->allocation.width;
-  y2 = y1 + widget->allocation.height;
+  xoffset += alloc.x;
+  yoffset += alloc.y;
+
+  x1 = alloc.x + xoffset;
+  y1 = alloc.y + yoffset;
+  x2 = x1 + alloc.width;
+  y2 = y1 + alloc.height;
 
   if( x > x1 && x < x2 && y > y1 && y < y2 ){
     return TRUE;
@@ -1071,18 +1091,28 @@ simo_combo_box_popup_close( SimoComboBox *combo_box )
   g_signal_handler_unblock( G_OBJECT( combo_box->button ), combo_box->priv->arrow_sig );
   g_signal_handler_unblock( G_OBJECT( combo_box->priv->label_button ), combo_box->priv->bl_sig );
 
-  gtk_widget_hide( GTK_WIDGET( combo_box->priv->popup_window ) );
+  gtk_widget_hide(GTK_WIDGET(combo_box->priv->popup_window));
   // the label should have focus after the combobox is clicked
   // make sure our widget has focus after being clicked on
-  if( GTK_WIDGET_CAN_FOCUS( combo_box ) ){
+#if GTK_MINOR_VERSION < 18
+  if (GTK_WIDGET_CAN_FOCUS(combo_box)) {
     //    gtk_widget_grab_focus( GTK_WIDGET( combo_box ) );
-    if( GTK_WIDGET_CAN_FOCUS( combo_box->entry ) ){
-      gtk_widget_grab_focus( GTK_WIDGET( combo_box->entry ) );
-    }
-    else if( GTK_WIDGET_CAN_FOCUS( combo_box->priv->label_button ) ){
-      gtk_widget_grab_focus( GTK_WIDGET( combo_box->priv->label_button ) );
+    if (GTK_WIDGET_CAN_FOCUS(combo_box->entry)) {
+      gtk_widget_grab_focus(GTK_WIDGET(combo_box->entry));
+    } else if (GTK_WIDGET_CAN_FOCUS(combo_box->priv->label_button)) {
+      gtk_widget_grab_focus(GTK_WIDGET( combo_box->priv->label_button));
     }
   }
+#else
+  if (gtk_widget_get_can_focus(GTK_WIDGET(combo_box))) {
+    //    gtk_widget_grab_focus( GTK_WIDGET( combo_box ) );
+    if (gtk_widget_get_can_focus(combo_box->entry)) {
+      gtk_widget_grab_focus(GTK_WIDGET(combo_box->entry));
+    } else if (gtk_widget_get_can_focus(combo_box->priv->label_button)) {
+      gtk_widget_grab_focus(GTK_WIDGET( combo_box->priv->label_button));
+    }
+  }
+#endif
 }
 
 
@@ -1151,7 +1181,7 @@ simo_combo_box_compute_tree_size( SimoComboBox *combo,
   for( i = gtk_tree_view_get_columns( combo->treeview ); i; i = g_list_next( i ) ){
     spacing += gtk_tree_view_column_get_spacing( GTK_TREE_VIEW_COLUMN( i->data ) );
     // loop over each cell renderer
-    for( j = gtk_tree_view_column_get_cell_renderers( GTK_TREE_VIEW_COLUMN( i->data ) );
+    for( j = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( i->data ) );
          j; j = g_list_next( j ) ){
 
       gtk_cell_renderer_get_size( GTK_CELL_RENDERER( j->data ), GTK_WIDGET( combo->treeview ),
@@ -1194,7 +1224,7 @@ simo_combo_box_popup( SimoComboBox *combo_box )
   gint width, height;
   gint screen_height, screen_width;
   GtkWidget *widget;
-  GtkAllocation *alloc;
+  GtkAllocation alloc;
   SimoComboBoxPrivate *mthis;
   //	GtkTreeViewColumn *column;
   gint tree_req_height, cell_width, cell_height, count;
@@ -1210,11 +1240,17 @@ simo_combo_box_popup( SimoComboBox *combo_box )
   widget = GTK_WIDGET( combo_box );
   mthis = combo_box->priv;
 
-  if( mthis->parent == NULL ){
-    mthis->parent = gtk_widget_get_toplevel( widget );
-    if( !GTK_WIDGET_TOPLEVEL( mthis->parent ) ){
+  if (mthis->parent == NULL) {
+    mthis->parent = gtk_widget_get_toplevel(widget);
+#if GTK_MINOR_VERSION < 18
+   if (!GTK_WIDGET_TOPLEVEL(mthis->parent)) {
       return;
     }
+#else
+   if (!gtk_widget_is_toplevel(mthis->parent)) {
+      return;
+    }
+#endif
   }
 
   g_signal_emit( combo_box, combo_box_signals[POPUP_DISPLAYED], 0 );
@@ -1224,24 +1260,36 @@ simo_combo_box_popup( SimoComboBox *combo_box )
   //	g_signal_connect( G_OBJECT( mthis->popup_window ), "event_after",
   //										G_CALLBACK( simo_combo_box_button_after ), combo_box );
 
-  alloc = &widget->allocation;
+#if GTK_MINOR_VERSION < 18
+  alloc = widget->allocation;
+#else
+  gtk_widget_get_allocation(widget, &alloc);
+#endif
 
   // begin determining the window position
 
-  gdk_window_get_origin( widget->window, &bx, &by );
-  bx += alloc->x;
-  by += alloc->y + alloc->height;
+  gdk_window_get_origin( gtk_widget_get_window(widget), &bx, &by );
+  bx += alloc.x;
+  by += alloc.y + alloc.height;
   screen_height = gdk_screen_height() - by;
   screen_width = gdk_screen_width();
 
+#if GTK_MINOR_VERSION < 18
+  alloc = mthis->popup_window->allocation;
+#else
+  gtk_widget_get_allocation(mthis->popup_window, &alloc);
+  height = alloc.height;
+  width = alloc.width;
+#endif
 
-  alloc = &(mthis->popup_window->allocation);
-  height = alloc->height;
-  width = alloc->width;
-  alloc = &widget->allocation;
+#if GTK_MINOR_VERSION < 18
+  alloc = widget->allocation;
+#else
+  gtk_widget_get_allocation(widget, &alloc);
+#endif
 
   gtk_window_move( GTK_WINDOW( mthis->popup_window ), -500, -500 );
-  gtk_widget_set_size_request( mthis->popup_window, alloc->width, 20 );
+  gtk_widget_set_size_request( mthis->popup_window, alloc.width, 20 );
 
   gtk_window_resize( GTK_WINDOW( mthis->popup_window ), 500, 500 );
 
@@ -1295,7 +1343,7 @@ simo_combo_box_popup( SimoComboBox *combo_box )
         width = widget->allocation.width + combo_box->max_popup_width;
         }
         else*/
-  width = alloc->width;   /* Never wider than the widget */
+  width = alloc.width;   /* Never wider than the widget */
 
   /* Always show popup below widget */
   if( height > screen_height ){
@@ -1333,14 +1381,24 @@ simo_combo_box_popup( SimoComboBox *combo_box )
 
   //grab_on_window( mthis->popup_window->window, gtk_get_current_event_time() );
   //	grab_on_window( GTK_WIDGET(combo_box->treeview)->window, gtk_get_current_event_time() );
-  if( !GTK_WIDGET_HAS_FOCUS( combo_box->treeview ) ){
-    GTK_WIDGET_SET_FLAGS (combo_box->treeview, GTK_CAN_DEFAULT);
-    gdk_keyboard_grab( mthis->popup_window->window, FALSE, GDK_CURRENT_TIME );
-    gtk_widget_grab_focus( GTK_WIDGET( combo_box->treeview ) );
+#if GTK_MINOR_VERSION < 18
+  if (!GTK_WIDGET_HAS_FOCUS(combo_box->treeview)) {
+    GTK_WIDGET_SET_FLAGS(combo_box->treeview, GTK_CAN_DEFAULT);
+    gdk_keyboard_grab(gtk_widget_get_window(mthis->popup_window),
+                      FALSE, GDK_CURRENT_TIME);
+    gtk_widget_grab_focus(GTK_WIDGET(combo_box->treeview));
   }
+#else
+  if (!gtk_widget_has_focus(GTK_WIDGET(combo_box->treeview))) {
+    gtk_widget_set_can_default(GTK_WIDGET(combo_box->treeview), TRUE);
+    gdk_keyboard_grab(gtk_widget_get_window(mthis->popup_window),
+                      FALSE, GDK_CURRENT_TIME);
+    gtk_widget_grab_focus(GTK_WIDGET(combo_box->treeview));
+  }
+#endif
 
   gtk_grab_add( mthis->popup_window );
-  gdk_pointer_grab( mthis->popup_window->window, TRUE,
+  gdk_pointer_grab( gtk_widget_get_window(mthis->popup_window), TRUE,
                     GDK_BUTTON_PRESS_MASK |
                     GDK_BUTTON_RELEASE_MASK |
                     GDK_POINTER_MOTION_MASK,
@@ -1423,11 +1481,18 @@ simo_combo_box_treeview_release_event( GtkWidget *widget,
 {
   GtkTreePath *path;
   GtkTreeViewColumn *col;
+  GtkAllocation      alloc;
 
-  if( (event->x > widget->allocation.x) &&
-      (event->x < (widget->allocation.x + widget->allocation.width)) &&
-      (event->y > widget->allocation.y) &&
-      (event->y < (widget->allocation.y + widget->allocation.height)) ){
+#if GTK_MINOR_VERSION < 18
+  alloc = widget->allocation;
+#else
+  gtk_widget_get_allocation(widget, &alloc);
+#endif
+
+  if( (event->x > alloc.x) &&
+      (event->x < (alloc.x + alloc.width)) &&
+      (event->y > alloc.y) &&
+      (event->y < (alloc.y + alloc.height)) ){
 
     if( gtk_tree_view_get_path_at_pos( combo_box->treeview, (gint)event->x, (gint)event->y,
                                        &path, &col, NULL, NULL ) ){
@@ -1601,7 +1666,7 @@ simo_combo_box_set_editable( SimoComboBox *combo_box, gboolean editable_state )
   //	g_object_set( G_OBJECT( combo_box->entry ), "can-focus", editable_state, NULL );
 
   if( editable_state ){
-    if( widget == button->parent ){
+    if( widget == gtk_widget_get_parent(button) ){
       const gchar *text = gtk_label_get_text( GTK_LABEL( combo_box->label ) );
 
       g_object_ref( G_OBJECT( button ) );
@@ -1620,7 +1685,7 @@ simo_combo_box_set_editable( SimoComboBox *combo_box, gboolean editable_state )
     }
   }
   else
-    if( widget == combo_box->entry->parent ){
+    if( widget == gtk_widget_get_parent(combo_box->entry) ){
       const gchar *text = gtk_entry_get_text( GTK_ENTRY( combo_box->entry ) );
       // save the entry allocation
       //memcpy( &(combo_box->priv->entry_alloc), &(combo_box->entry->allocation), sizeof( GtkAllocation ) );
