@@ -25,9 +25,7 @@
 #ifndef SIMO_COMBO_BOX_H
 #define SIMO_COMBO_BOX_H
 
-#include <gtk/gtkbin.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
