@@ -1,7 +1,7 @@
-/* $Id: timers.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: timers.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -154,7 +153,7 @@ free_timer(Timer *timer, gpointer data)
 void
 list_timers(World *world, Plugin *plugin)
 {
-  int     i;
+  guint   i;
   guint   interval_width;
   guint   repeat_width;
   guint   total_width;
