@@ -1,7 +1,7 @@
-/* $Id: triggers.c 1024 2009-02-09 18:01:06Z ekalin $ */
+/* $Id: triggers.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <libintl.h>
 #include <locale.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "ansi.h"
@@ -42,9 +41,9 @@ static void       apply_highlight(World       *world,
                                   Trigger     *trigger,
                                   GtkTextIter *end_iter);
 static GtkWidget *create_test_triggers_dialog(GtkWindow *parent, World *world);
-static void       test_trigger_send_cb(GtkWidget *widget, gpointer data);
-/* Glade signals */
+/* XML UI signals */
 G_MODULE_EXPORT void test_triggers_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void test_trigger_send_cb(GtkWidget *widget, gpointer data);
 
 
 
@@ -140,7 +139,9 @@ match_triggers(World     *world,
                 trigger->action);
       }
 
-      parse_commands(world, trigger->action, strlen(trigger->action));
+      if (trigger->action) {
+        parse_commands(world, trigger->action, strlen(trigger->action));
+      }
 
       g_free(*colorline);
       tmp = SvPV(Pcolorline, *output_len);
@@ -248,11 +249,27 @@ match_triggers(World     *world,
       }
 
       if (trigger->action) {
+        /* Substitute $1, $2, etc. for matched parts. */
+        gchar *action = perl_substitute(world->perl_interpreter,
+                                        trigger->action,
+                                        world->trigger_action_re,
+                                        "$_[$1]",
+                                        FALSE,
+                                        TRUE);
         if (!trigger_response) {
-          parse_commands(world, trigger->action, strlen(trigger->action));
+          if (action) {
+            parse_commands(world, action, strlen(action));
+            g_free(action);
+          } else {
+            parse_commands(world, trigger->action, strlen(trigger->action));
+          }
         } else {
-          *trigger_response = g_slist_append(*trigger_response,
-                                             g_strdup(trigger->action));
+          if (action) {
+            *trigger_response = g_slist_append(*trigger_response, action);
+          } else {
+            *trigger_response = g_slist_append(*trigger_response,
+                                               g_strdup(trigger->action));
+          }
         }
       }
 
@@ -359,6 +376,11 @@ remove_trigger(World *world, GSList *triggeritem)
 {
   Trigger *trigger = (Trigger *) triggeritem->data;
 
+  /* Necessary when free_trigger dereferences trigger->pattern_re */
+  if (world->perl_interpreter) {
+    PERL_SET_CONTEXT(world->perl_interpreter);
+  }
+
   we_trigger_delete_trigger(world, trigger);
 
   world->triggers = g_slist_remove_link(world->triggers, triggeritem);
@@ -398,6 +420,7 @@ free_trigger(Trigger *trigger, gpointer data)
   g_free(trigger->name);
   g_free(trigger->pattern);
   g_free(trigger->action);
+  SvREFCNT_dec(trigger->pattern_re);
 
   g_free(trigger);
 }
@@ -406,7 +429,7 @@ free_trigger(Trigger *trigger, gpointer data)
 void
 list_triggers(World *world, Plugin *plugin)
 {
-  int      i;
+  guint    i;
   guint    total_width;
   guint    field_width;
   GSList  *trigptr;
@@ -480,6 +503,14 @@ trigger_precompute_res(World *world)
 
     triggerptr = triggerptr->next;
   }
+
+  /* Precompute a special pattern to substitute $1, $2, etc in trigger
+     actions. */
+  if (!world->trigger_action_re) {
+    world->trigger_action_re = precompute_re(world->perl_interpreter,
+                                             "\\$([0-9]+)",
+                                             FALSE);
+  }
 }
 
 
@@ -541,41 +572,39 @@ static
 GtkWidget *
 create_test_triggers_dialog(GtkWindow *parent, World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *dlg;
-  GtkWidget        *btnClose;
-  GtkWidget        *btnSend;
+  GError           *error = NULL;
+  GObject          *dlg;
 
   /* Create the dialog */
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgTestTriggers", NULL);
-  dlg      = glade_xml_get_widget(gladexml, "dlgTestTriggers");
+  if (!world->ui_builder) {
+    world->ui_builder = gtk_builder_new();
+  }
+  if (!gtk_builder_add_from_file(world->ui_builder,
+                                 get_kildclient_installed_file("dlgTestTriggers.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+  dlg      = gtk_builder_get_object(world->ui_builder, "dlgTestTriggers");
 
   /* Signals */
-  glade_xml_signal_autoconnect(gladexml);
-  btnClose = glade_xml_get_widget(gladexml, "btnTTClose");
-  g_signal_connect_swapped(G_OBJECT(btnClose), "clicked",
-                           G_CALLBACK(gtk_widget_hide), dlg);
+  gtk_builder_connect_signals(world->ui_builder, world);
 
-  btnSend = glade_xml_get_widget(gladexml, "btnTriggerSend");
-  g_signal_connect(G_OBJECT(btnSend), "clicked",
-                   G_CALLBACK(test_trigger_send_cb), world);
-
-  return dlg;
+  return GTK_WIDGET(dlg);
 }
 
 
-static
 void
 test_trigger_send_cb(GtkWidget *widget, gpointer data)
 {
   World         *world = (World *) data;
-  GladeXML      *gladexml;
-  GtkWidget     *txtLine;
-  GtkWidget     *lblMatches;
-  GtkWidget     *txtCommands;
-  GtkWidget     *txtOutput;
-  GtkWidget     *lblGagLog;
+  GtkEntry      *txtLine;
+  GtkLabel      *lblMatches;
+  GtkTextView   *txtCommands;
+  GtkEntry      *txtOutput;
+  GtkLabel      *lblGagLog;
   GtkTextBuffer *bufCommands;
   gboolean       disable_triggers;
   guchar        *line;
@@ -589,15 +618,19 @@ test_trigger_send_cb(GtkWidget *widget, gpointer data)
   GSList        *cmdptr;
   GtkTextIter    iter;
 
-  gladexml    = glade_get_widget_tree(widget);
-  txtLine     = glade_xml_get_widget(gladexml, "txtLine");
-  lblMatches  = glade_xml_get_widget(gladexml, "lblMatches");
-  txtOutput   = glade_xml_get_widget(gladexml, "txtOutput");
-  txtCommands = glade_xml_get_widget(gladexml, "txtCommands");
-  lblGagLog   = glade_xml_get_widget(gladexml, "lblGagLog");
-  bufCommands = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txtCommands));
+  txtLine     = GTK_ENTRY(gtk_builder_get_object(world->ui_builder,
+                                                 "txtLine"));
+  lblMatches  = GTK_LABEL(gtk_builder_get_object(world->ui_builder,
+                                                 "lblMatches"));
+  txtOutput   = GTK_ENTRY(gtk_builder_get_object(world->ui_builder,
+                                                 "txtOutput"));
+  txtCommands = GTK_TEXT_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                     "txtCommands"));
+  lblGagLog   = GTK_LABEL(gtk_builder_get_object(world->ui_builder,
+                                                 "lblGagLog"));
+  bufCommands = gtk_text_view_get_buffer(txtCommands);
 
-  line     = (guchar *) g_strdup(gtk_entry_get_text(GTK_ENTRY(txtLine)));
+  line     = (guchar *) g_strdup(gtk_entry_get_text(txtLine));
   stripped = strip_ansi(line, strlen((gchar *) line));
 
   /* We need the triggers to be tried. */
@@ -612,7 +645,7 @@ test_trigger_send_cb(GtkWidget *widget, gpointer data)
 
   /* Show output */
   n_matches_str = g_strdup_printf("%d", n_matches);
-  gtk_label_set_text(GTK_LABEL(lblMatches), n_matches_str);
+  gtk_label_set_text(lblMatches, n_matches_str);
   g_free(n_matches_str);
 
   gtk_text_buffer_set_text(bufCommands, "", 0);
@@ -627,13 +660,12 @@ test_trigger_send_cb(GtkWidget *widget, gpointer data)
   g_slist_free(trigger_response);
 
   if (!gag_output) {
-    gtk_entry_set_text(GTK_ENTRY(txtOutput), (gchar *) line);
+    gtk_entry_set_text(txtOutput, (gchar *) line);
   } else {
-    gtk_entry_set_text(GTK_ENTRY(txtOutput), "");
+    gtk_entry_set_text(txtOutput, "");
   }
 
-  gtk_label_set_text(GTK_LABEL(lblGagLog),
-                     gag_log ? _("No") : _("Yes"));
+  gtk_label_set_text(lblGagLog, gag_log ? _("No") : _("Yes"));
 
   g_free(line);
   g_free(stripped);
