@@ -1,7 +1,7 @@
-/* $Id: we_advanced.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: we_advanced.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <sys/stat.h>
 #include <libintl.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -37,18 +36,19 @@
 /***********************
  * Function prototypes *
  ***********************/
-static void confirm_delete_changed_cb(GtkToggleButton *chkbox,
-                                      gpointer         data);
-static void show_plugin_toggled_cb(GtkToggleButton *button,
-                                   gpointer         data);
 static GtkTreeModel *get_charsets_model(void);
 static char         *get_file_name_for_world(const char *name);
+/* XML UI signals */
+G_MODULE_EXPORT void confirm_delete_changed_cb(GtkToggleButton *chkbox,
+                                               gpointer         data);
+G_MODULE_EXPORT void show_plugin_toggled_cb(GtkToggleButton *button,
+                                            gpointer         data);
 
 
 /*************************
  * File global variables *
  *************************/
-static const gchar     *charsets[] = {
+static const gchar *charsets[] = {
     "ISO-8859-1",
     "UTF-8",
     "ANSI_X3.110-1983",
@@ -216,26 +216,26 @@ static const gchar     *charsets[] = {
 
 
 
-GtkWidget *
-get_we_advanced_tab(World *world, GladeXML *gladexml)
+GObject *
+get_we_advanced_tab(World *world, GtkBuilder *ui_builder)
 {
-  GtkWidget       *panel;
+  GObject         *panel;
   GtkCellRenderer *renderer;
-  GtkWidget       *cmbCharset;
+  GtkComboBox     *cmbCharset;
   GtkWidget       *vbox_file;
 
-  panel = glade_xml_get_widget(gladexml, "we_panel_advanced");
+  panel = gtk_builder_get_object(ui_builder, "we_panel_advanced");
 
   if (world == default_world) {
-    vbox_file = glade_xml_get_widget(gladexml, "vbox_file");
+    vbox_file = GTK_WIDGET(gtk_builder_get_object(ui_builder, "vbox_file"));
     gtk_widget_destroy(vbox_file);
   }
 
-  cmbCharset = glade_xml_get_widget(gladexml, "cmbCharset");
-  gtk_combo_box_set_model(GTK_COMBO_BOX(cmbCharset), get_charsets_model());
+  cmbCharset = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbCharset"));
+  gtk_combo_box_set_model(cmbCharset, get_charsets_model());
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(cmbCharset), renderer, TRUE);
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cmbCharset), 0);
+  gtk_combo_box_set_active(cmbCharset, 0);
   gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(cmbCharset),
                                  renderer,
                                  "text", 0,
@@ -246,30 +246,38 @@ get_we_advanced_tab(World *world, GladeXML *gladexml)
 
 
 void
-fill_we_advanced_tab(World *world, GladeXML *gladexml)
+fill_we_advanced_tab(World *world, GtkBuilder *ui_builder)
 {
-  GtkWidget *txtWorldFile;
-  GtkWidget *cmbCharset;
-  GtkWidget *chkConfirmDelete;
-  GtkWidget *chkShowPlugin;
+  GtkEntry        *txtWorldFile;
+  GtkComboBox     *cmbCharset;
+  GtkToggleButton *chkConfirmDelete;
+  GtkToggleButton *chkShowPlugin;
+  GtkToggleButton *chkKeepAlive;
+  GtkToggleButton *chkIgnoreUpDown;
 
   if (!world) {
     return;
   }
 
-  cmbCharset       = glade_xml_get_widget(gladexml, "cmbCharset");
-  chkConfirmDelete = glade_xml_get_widget(gladexml, "chkConfirmDelete");
-  chkShowPlugin    = glade_xml_get_widget(gladexml, "chkShowPlugin");
+  cmbCharset
+    = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbCharset"));
+  chkConfirmDelete
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkConfirmDelete"));
+  chkShowPlugin
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkShowPlugin"));
+  chkKeepAlive
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkKeepAlive"));
+  chkIgnoreUpDown
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkIgnoreUpDown"));
 
   if (world != default_world) {
-    txtWorldFile = glade_xml_get_widget(gladexml, "txtWorldFile");
+    txtWorldFile = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtWorldFile"));
     if (world->file)
-      gtk_entry_set_text(GTK_ENTRY(txtWorldFile), world->file);
+      gtk_entry_set_text(txtWorldFile, world->file);
   }
 
   if (world->charset_index != -1) {
-    gtk_combo_box_set_active(GTK_COMBO_BOX(cmbCharset),
-                             world->charset_index);
+    gtk_combo_box_set_active(cmbCharset, world->charset_index);
   } else {
     GtkTreeIter   iter;
     GtkTreeModel *model;
@@ -282,44 +290,44 @@ fill_we_advanced_tab(World *world, GladeXML *gladexml)
                          0, &charset,
                          -1);
       if (strcmp(charset, world->charset) == 0) {
-        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(cmbCharset),
-                                      &iter);
-        world->charset_index = gtk_combo_box_get_active(GTK_COMBO_BOX(cmbCharset));
+        gtk_combo_box_set_active_iter(cmbCharset, &iter);
+        world->charset_index = gtk_combo_box_get_active(cmbCharset);
         break;
       }
     } while (gtk_tree_model_iter_next(model, &iter));
   }
 
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfirmDelete),
-                               world->confirm_delete);
-  g_signal_connect(G_OBJECT(chkConfirmDelete), "toggled",
-                   G_CALLBACK(confirm_delete_changed_cb), world);
-
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkShowPlugin),
-                               world->show_plugin_items);
-  g_signal_connect(G_OBJECT(chkShowPlugin), "toggled",
-                   G_CALLBACK(show_plugin_toggled_cb), world);
-
+  gtk_toggle_button_set_active(chkConfirmDelete, world->confirm_delete);
+  gtk_toggle_button_set_active(chkShowPlugin,    world->show_plugin_items);
+  gtk_toggle_button_set_active(chkKeepAlive,     world->keep_alive);
+  gtk_toggle_button_set_active(chkIgnoreUpDown,  world->ignore_up_down_keys);
 }
 
 
 void
-update_we_advanced_parameters(World *world, gboolean *newworld)
+update_we_advanced_parameters(World *world, gboolean *newworld,
+                              GtkBuilder *ui_builder)
 {
-  GladeXML    *gladexml;
-  GtkWidget   *txtWorldFile;
-  GtkWidget   *cmbCharset;
-  const gchar *newval;
-  gchar       *newfile;
-  gchar       *newcharset;
-  GtkTreeIter  iter;
+  GtkEntry        *txtWorldFile;
+  GtkComboBox     *cmbCharset;
+  GtkToggleButton *chkKeepAlive;
+  const gchar     *newval;
+  gchar           *newfile;
+  gchar           *newcharset;
+  GtkTreeIter      iter;
+  GtkToggleButton *chkIgnoreUpDown;
 
-  gladexml   = glade_get_widget_tree(world->dlgEditWorld);
-  cmbCharset = glade_xml_get_widget(gladexml, "cmbCharset");
+  cmbCharset      = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder,
+                                                         "cmbCharset"));
+  chkKeepAlive    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                             "chkKeepAlive"));
+  chkIgnoreUpDown = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                             "chkIgnoreUpDown"));
 
   if (world != default_world) {
-    txtWorldFile = glade_xml_get_widget(gladexml, "txtWorldFile");
-    newval = gtk_entry_get_text(GTK_ENTRY(txtWorldFile));
+    txtWorldFile = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                    "txtWorldFile"));
+    newval = gtk_entry_get_text(txtWorldFile);
     if (strcmp(newval, "") == 0) {
       newfile = get_file_name_for_world(world->name);
     } else {
@@ -334,7 +342,7 @@ update_we_advanced_parameters(World *world, gboolean *newworld)
     }
   }
 
-  gtk_combo_box_get_active_iter(GTK_COMBO_BOX(cmbCharset), &iter);
+  gtk_combo_box_get_active_iter(cmbCharset, &iter);
   gtk_tree_model_get(get_charsets_model(), &iter,
                      0, &newcharset,
                      -1);
@@ -344,11 +352,14 @@ update_we_advanced_parameters(World *world, gboolean *newworld)
   } else {
     g_free(newcharset);
   }
-  world->charset_index = gtk_combo_box_get_active(GTK_COMBO_BOX(cmbCharset));
+  world->charset_index = gtk_combo_box_get_active(cmbCharset);
+
+  world->keep_alive = gtk_toggle_button_get_active(chkKeepAlive);
+
+  world->ignore_up_down_keys = gtk_toggle_button_get_active(chkIgnoreUpDown);
 }
 
 
-static
 void
 confirm_delete_changed_cb(GtkToggleButton *chkbox,
                           gpointer         data)
@@ -358,7 +369,7 @@ confirm_delete_changed_cb(GtkToggleButton *chkbox,
   world->confirm_delete = gtk_toggle_button_get_active(chkbox);
 }
 
-static
+
 void
 show_plugin_toggled_cb(GtkToggleButton *button, gpointer data)
 {
