@@ -1,7 +1,7 @@
-/* $Id: we_general.c 1019 2009-02-03 19:31:11Z ekalin $ */
+/* $Id: we_general.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -36,117 +35,39 @@
 /***********************
  * Function prototypes *
  ***********************/
-static void     we_general_add_edit_char_cb(GtkWidget *widget, gpointer data);
-static void     view_row_dblclick_cb(GtkTreeView        *view,
-                                     GtkTreePath        *path,
-                                     GtkTreeViewColumn  *col,
-                                     gpointer            data);
 static gboolean edit_char(World        *world,
                           GtkTreeModel *model,
                           GtkTreeIter  *iter);
-static void     we_general_delete_char_cb(GtkWidget *widget, gpointer data);
-
-static gboolean we_general_view_keypress_cb(GtkWidget   *widget,
-                                            GdkEventKey *evt,
-                                            gpointer     data);
-static void     we_general_move_char_cb(GtkWidget *widget, gpointer data);
+/* XML UI signals */
+G_MODULE_EXPORT void     we_general_add_edit_char_cb(GtkWidget *widget,
+                                                     gpointer data);
+G_MODULE_EXPORT void     we_general_delete_char_cb(GtkWidget *widget,
+                                                   gpointer data);
+G_MODULE_EXPORT void     we_general_move_char_cb(gpointer data,
+                                                 GtkWidget *widget);
+G_MODULE_EXPORT void     we_general_view_row_dblclick_cb(GtkTreeView  *view,
+                                                         GtkTreePath  *path,
+                                                         GtkTreeViewColumn *col,
+                                                         gpointer     data);
+G_MODULE_EXPORT gboolean we_general_view_keypress_cb(GtkWidget   *widget,
+                                                     GdkEventKey *evt,
+                                                     gpointer     data);
 
 
 
-GtkWidget *
-get_we_general_tab(GladeXML *gladexml)
+GObject   *
+get_we_general_tab(World *world, GtkBuilder *ui_builder)
 {
-  GtkWidget *panel;
-
-  panel = glade_xml_get_widget(gladexml, "we_panel_general");
-
-  return panel;
-}
-
-
-void
-fill_we_general_tab(World *world, GladeXML *gladexml)
-{
-  GtkWidget         *txtName;
-  GtkWidget         *txtHost;
-  GtkWidget         *txtPort;
-  GtkWidget         *chkTLS;
-  GtkWidget         *cmbProxyType;
-  GtkWidget         *txtProxyServer;
-  GtkWidget         *txtProxyPort;
-  GtkWidget         *txtProxyUser;
-  GtkWidget         *txtProxyPassword;
-  GtkWidget         *cmbConnStyle;
+  GObject           *panel;
   GtkTreeView       *viewCharacters;
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
-  GtkWidget         *btnAdd;
-  GtkWidget         *btnEdit;
-  GtkWidget         *btnDelete;
-  GtkWidget         *btnUp;
-  GtkWidget         *btnDown;
 
-  if (!world) {
-    return;
-  }
-
-  txtName          = glade_xml_get_widget(gladexml, "txtName");
-  txtHost          = glade_xml_get_widget(gladexml, "txtHost");
-  txtPort          = glade_xml_get_widget(gladexml, "txtPort");
-  chkTLS           = glade_xml_get_widget(gladexml, "chkTLS");
-  cmbProxyType     = glade_xml_get_widget(gladexml, "cmbProxyType");
-  txtProxyServer   = glade_xml_get_widget(gladexml, "txtProxyServer");
-  txtProxyPort     = glade_xml_get_widget(gladexml, "txtProxyPort");
-  txtProxyUser     = glade_xml_get_widget(gladexml, "txtProxyUser");
-  txtProxyPassword = glade_xml_get_widget(gladexml, "txtProxyPassword");
-  cmbConnStyle     = glade_xml_get_widget(gladexml, "cmbConnStyle");
-  btnAdd           = glade_xml_get_widget(gladexml, "btnCharAdd");
-  btnEdit          = glade_xml_get_widget(gladexml, "btnCharEdit");
-  btnDelete        = glade_xml_get_widget(gladexml, "btnCharDelete");
-  btnUp            = glade_xml_get_widget(gladexml, "btnCharUp");
-  btnDown          = glade_xml_get_widget(gladexml, "btnCharDown");
-  viewCharacters   = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCharacters"));
-
-  /* General connection settings */
-  if (world->name) {
-    gtk_entry_set_text(GTK_ENTRY(txtName), world->name);
-  }
-  if (world->host) {
-    gtk_entry_set_text(GTK_ENTRY(txtHost), world->host);
-  }
-  gtk_entry_set_text(GTK_ENTRY(txtPort), world->port);
-#ifdef HAVE_LIBGNUTLS
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkTLS), world->use_tls);
-#else
-  gtk_widget_destroy(chkTLS);
-#endif
-
-  /* Proxy */
-  /* -1 is "global settings in enumeration */
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cmbProxyType),
-                           world->proxy.type + 1);
-  if (world->proxy.server) {
-    gtk_entry_set_text(GTK_ENTRY(txtProxyServer),
-                       world->proxy.server);
-  }
-  if (world->proxy.port) {
-    gtk_entry_set_text(GTK_ENTRY(txtProxyPort),
-                       world->proxy.port);
-  }
-  if (world->proxy.user) {
-    gtk_entry_set_text(GTK_ENTRY(txtProxyUser),
-                       world->proxy.user);
-  }
-  if (world->proxy.password) {
-    gtk_entry_set_text(GTK_ENTRY(txtProxyPassword),
-                       world->proxy.password);
-  }
-
-  /* Auto-login*/
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cmbConnStyle),
-                           world->connection_style);
+  panel = gtk_builder_get_object(ui_builder, "we_panel_general");
 
   /* Configure view */
+  viewCharacters
+    = GTK_TREE_VIEW(gtk_builder_get_object(ui_builder, "viewCharacters"));
   gtk_tree_view_set_model(viewCharacters, world->logon_characters);
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_attributes(viewCharacters,
@@ -162,72 +83,152 @@ fill_we_general_tab(World *world, GladeXML *gladexml)
   gtk_tree_view_column_set_title(column, _("Password"));
   gtk_tree_view_column_pack_start(column, renderer, TRUE);
   gtk_tree_view_append_column(viewCharacters, column);
-  g_signal_connect(G_OBJECT(viewCharacters), "row-activated",
-                   G_CALLBACK(view_row_dblclick_cb), world);
-  g_signal_connect(G_OBJECT(viewCharacters), "key-press-event",
-                   G_CALLBACK(we_general_view_keypress_cb), world);
 
-  /* Button signals */
-  g_object_set_data(G_OBJECT(btnAdd), "isnew", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnAdd), "clicked",
-                   G_CALLBACK(we_general_add_edit_char_cb), world);
-  g_object_set_data(G_OBJECT(btnEdit), "isnew", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
-                   G_CALLBACK(we_general_add_edit_char_cb), world);
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
-                   G_CALLBACK(we_general_delete_char_cb), world);
-  g_object_set_data(G_OBJECT(btnUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnUp), "clicked",
-                   G_CALLBACK(we_general_move_char_cb), viewCharacters);
-  g_object_set_data(G_OBJECT(btnDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnDown), "clicked",
-                   G_CALLBACK(we_general_move_char_cb), viewCharacters);
+  return panel;
 }
 
 
 void
-update_we_general_parameters(World *world)
+fill_we_general_tab(World *world, GtkBuilder *ui_builder)
 {
-  const gchar *newval;
-  GladeXML    *gladexml;
-  GtkWidget   *txtName;
-  GtkWidget   *txtHost;
-  GtkWidget   *txtPort;
-  GtkWidget   *cmbProxyType;
-  GtkWidget   *txtProxyServer;
-  GtkWidget   *txtProxyPort;
-  GtkWidget   *txtProxyUser;
-  GtkWidget   *txtProxyPassword;
-  GtkWidget   *cmbConnStyle;
+  GtkEntry          *txtName;
+  GtkEntry          *txtHost;
+  GtkEntry          *txtPort;
+  GtkWidget         *chkTLS;
+  GtkComboBox       *cmbProxyType;
+  GtkEntry          *txtProxyServer;
+  GtkEntry          *txtProxyPort;
+  GtkEntry          *txtProxyUser;
+  GtkEntry          *txtProxyPassword;
+  GtkComboBox       *cmbConnStyle;
+  GObject           *btnAdd;
+  GObject           *btnEdit;
+  GObject           *btnUp;
+  GObject           *btnDown;
+
+  if (!world) {
+    return;
+  }
+
+  txtName          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtName"));
+  txtHost          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtHost"));
+  txtPort          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtPort"));
+  chkTLS           = GTK_WIDGET(gtk_builder_get_object(ui_builder, "chkTLS"));
+  cmbProxyType     = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder,
+                                                          "cmbProxyType"));
+  txtProxyServer   = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyServer"));
+  txtProxyPort     = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyPort"));
+  txtProxyUser     = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyUser"));
+  txtProxyPassword = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyPassword"));
+  cmbConnStyle     = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder,
+                                                          "cmbConnStyle"));
+  btnAdd           = gtk_builder_get_object(ui_builder, "btnCharAdd");
+  btnEdit          = gtk_builder_get_object(ui_builder, "btnCharEdit");
+  btnUp            = gtk_builder_get_object(ui_builder, "btnCharUp");
+  btnDown          = gtk_builder_get_object(ui_builder, "btnCharDown");
+
+  /* General connection settings */
+  if (world->name) {
+    gtk_entry_set_text(txtName, world->name);
+  }
+  if (world->host) {
+    gtk_entry_set_text(txtHost, world->host);
+  }
+  if (world->port) {
+    gtk_entry_set_text(txtPort, world->port);
+  }
 #ifdef HAVE_LIBGNUTLS
-  GtkWidget   *chkTLS;
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkTLS), world->use_tls);
+#else
+  gtk_widget_destroy(chkTLS);
 #endif
 
-  gladexml         = glade_get_widget_tree(world->dlgEditWorld);
-  txtName          = glade_xml_get_widget(gladexml, "txtName");
-  txtHost          = glade_xml_get_widget(gladexml, "txtHost");
-  txtPort          = glade_xml_get_widget(gladexml, "txtPort");
-  cmbProxyType     = glade_xml_get_widget(gladexml, "cmbProxyType");
-  txtProxyServer   = glade_xml_get_widget(gladexml, "txtProxyServer");
-  txtProxyPort     = glade_xml_get_widget(gladexml, "txtProxyPort");
-  txtProxyUser     = glade_xml_get_widget(gladexml, "txtProxyUser");
-  txtProxyPassword = glade_xml_get_widget(gladexml, "txtProxyPassword");
-  cmbConnStyle     = glade_xml_get_widget(gladexml, "cmbConnStyle");
+  /* Proxy */
+  /* -1 is "global settings in enumeration */
+  gtk_combo_box_set_active(cmbProxyType, world->proxy.type + 1);
+  if (world->proxy.server) {
+    gtk_entry_set_text(txtProxyServer, world->proxy.server);
+  } else {
+    gtk_entry_set_text(txtProxyServer, "");
+  }
+  if (world->proxy.port) {
+    gtk_entry_set_text(txtProxyPort, world->proxy.port);
+  } else {
+    gtk_entry_set_text(txtProxyPort, "");
+  }
+  if (world->proxy.user) {
+    gtk_entry_set_text(txtProxyUser, world->proxy.user);
+  } else {
+    gtk_entry_set_text(txtProxyUser, "");
+  }
+  if (world->proxy.password) {
+    gtk_entry_set_text(txtProxyPassword, world->proxy.password);
+  } else {
+    gtk_entry_set_text(txtProxyPassword, "");
+  }
+
+  /* Auto-login*/
+  gtk_combo_box_set_active(cmbConnStyle, world->connection_style);
+
+  /* Information used in the callbacks */
+  g_object_set_data(btnAdd, "isnew", GINT_TO_POINTER(TRUE));
+  g_object_set_data(btnEdit, "isnew", GINT_TO_POINTER(FALSE));
+  g_object_set_data(btnUp, "isup", GINT_TO_POINTER(TRUE));
+  g_object_set_data(btnDown, "isup", GINT_TO_POINTER(FALSE));
+}
+
+
+void
+update_we_general_parameters(World *world, GtkBuilder *ui_builder)
+{
+  const gchar     *newval;
+  GtkEntry        *txtName;
+  GtkEntry        *txtHost;
+  GtkEntry        *txtPort;
+  GtkComboBox     *cmbProxyType;
+  GtkEntry        *txtProxyServer;
+  GtkEntry        *txtProxyPort;
+  GtkEntry        *txtProxyUser;
+  GtkEntry        *txtProxyPassword;
+  GtkComboBox     *cmbConnStyle;
+#ifdef HAVE_LIBGNUTLS
+  GtkToggleButton *chkTLS;
+#endif
+
+  txtName          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtName"));
+  txtHost          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtHost"));
+  txtPort          = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtPort"));
+  cmbProxyType     = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder,
+                                                          "cmbProxyType"));
+  txtProxyServer   = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyServer"));
+  txtProxyPort     = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyPort"));
+  txtProxyUser     = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyUser"));
+  txtProxyPassword = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtProxyPassword"));
+  cmbConnStyle     = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder,
+                                                          "cmbConnStyle"));
 
   /* General settings */
-  newval = gtk_entry_get_text(GTK_ENTRY(txtName));
+  newval = gtk_entry_get_text(txtName);
   if (!world->name || strcmp(world->name, newval) != 0) {
     g_free(world->name);
     world->name = g_strdup(newval);
   }
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtHost));
+  newval = gtk_entry_get_text(txtHost);
   if (!world->host || strcmp(world->host, newval) != 0) {
     g_free(world->host);
     world->host = g_strdup(newval);
   }
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtPort));
+  newval = gtk_entry_get_text(txtPort);
   if (!world->port || strcmp(world->port, newval) != 0) {
     g_free(world->port);
     world->port = g_strdup(newval);
@@ -235,56 +236,54 @@ update_we_general_parameters(World *world)
 
   /* Proxy */
   /* -1 is "global settings" in enumeration */
-  world->proxy.type = gtk_combo_box_get_active(GTK_COMBO_BOX(cmbProxyType)) - 1;
+  world->proxy.type = gtk_combo_box_get_active(cmbProxyType) - 1;
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtProxyServer));
+  newval = gtk_entry_get_text(txtProxyServer);
   if (!world->proxy.server || strcmp(world->proxy.server, newval) != 0) {
     g_free(world->proxy.server);
     world->proxy.server = g_strdup(newval);
   }
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtProxyPort));
+  newval = gtk_entry_get_text(txtProxyPort);
   if (!world->proxy.port || strcmp(world->proxy.port, newval) != 0) {
     g_free(world->proxy.port);
     world->proxy.port = g_strdup(newval);
   }
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtProxyUser));
+  newval = gtk_entry_get_text(txtProxyUser);
   if (!world->proxy.user || strcmp(world->proxy.user, newval) != 0) {
     g_free(world->proxy.user);
     world->proxy.user = g_strdup(newval);
   }
 
-  newval = gtk_entry_get_text(GTK_ENTRY(txtProxyPassword));
+  newval = gtk_entry_get_text(txtProxyPassword);
   if (!world->proxy.password || strcmp(world->proxy.password, newval) != 0) {
     g_free(world->proxy.password);
     world->proxy.password = g_strdup(newval);
   }
 
   /* Auto-logon */
-  world->connection_style = gtk_combo_box_get_active(GTK_COMBO_BOX(cmbConnStyle));
+  world->connection_style = gtk_combo_box_get_active(cmbConnStyle);
 
 #ifdef HAVE_LIBGNUTLS
-  chkTLS = glade_xml_get_widget(gladexml, "chkTLS");
-  world->use_tls = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkTLS));
+  chkTLS = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "chkTLS"));
+  world->use_tls = gtk_toggle_button_get_active(chkTLS);
 #endif
 }
 
 
-static
 void
 we_general_add_edit_char_cb(GtkWidget *widget, gpointer data)
 {
   World            *world = (World *) data;
-  GladeXML         *gladexml;
   gboolean          is_new;
   GtkTreeView      *view;
   GtkTreeModel     *model;
   GtkTreeSelection *selection;
   GtkTreeIter       iter;
 
-  gladexml = glade_get_widget_tree(widget);
-  view = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCharacters"));
+  view = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                              "viewCharacters"));
   model = gtk_tree_view_get_model(view);
 
   is_new = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "isnew"));
@@ -303,12 +302,11 @@ we_general_add_edit_char_cb(GtkWidget *widget, gpointer data)
 }
 
 
-static
 void
-view_row_dblclick_cb(GtkTreeView        *view,
-                     GtkTreePath        *path,
-                     GtkTreeViewColumn  *col,
-                     gpointer            data)
+we_general_view_row_dblclick_cb(GtkTreeView        *view,
+                                GtkTreePath        *path,
+                                GtkTreeViewColumn  *col,
+                                gpointer            data)
 {
   World        *world;
   GtkTreeModel *model;
@@ -327,7 +325,9 @@ static
 gboolean
 edit_char(World *world, GtkTreeModel *model, GtkTreeIter *iter)
 {
-  GladeXML    *gladexml;
+  GtkBuilder  *ui_builder;
+  gchar       *objects[] = { "dlgEditChar", NULL };
+  GError      *error = NULL;
   GtkWidget   *dlgEditChar;
   GtkEntry    *txtName;
   GtkEntry    *txtPassword;
@@ -336,17 +336,24 @@ edit_char(World *world, GtkTreeModel *model, GtkTreeIter *iter)
   const gchar *newname;
   const gchar *newpassword;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditChar", NULL);
-  dlgEditChar  = glade_xml_get_widget(gladexml, "dlgEditChar");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return FALSE;
+  }
+
+  dlgEditChar = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgEditChar"));
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditChar), GTK_RESPONSE_OK);
-  txtName     = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                  "txtCharName");
-  txtPassword = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                  "txtCharPassword");
+  txtName     = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtCharName"));
+  txtPassword = GTK_ENTRY( gtk_builder_get_object(ui_builder,
+                                                  "txtCharPassword"));
 
   /* We don't need it anymore */
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 
   /* Fill-in values */
   gtk_tree_model_get(model, iter,
@@ -411,19 +418,17 @@ edit_char(World *world, GtkTreeModel *model, GtkTreeIter *iter)
 }
 
 
-static
 void
 we_general_delete_char_cb(GtkWidget *widget, gpointer data)
 {
   World            *world = (World *) data;
-  GladeXML         *gladexml;
   GtkTreeView      *view;
   GtkTreeModel     *model;
   GtkTreeSelection *selection;
   GtkTreeIter       iter;
 
-  gladexml = glade_get_widget_tree(widget);
-  view = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCharacters"));
+  view = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                              "viewCharacters"));
   selection = gtk_tree_view_get_selection(view);
   if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
     GtkWidget *msgdlg;
@@ -441,7 +446,6 @@ we_general_delete_char_cb(GtkWidget *widget, gpointer data)
 }
 
 
-static
 gboolean
 we_general_view_keypress_cb(GtkWidget   *widget,
                             GdkEventKey *evt,
@@ -456,9 +460,8 @@ we_general_view_keypress_cb(GtkWidget   *widget,
 }
 
 
-static
 void
-we_general_move_char_cb(GtkWidget *widget, gpointer data)
+we_general_move_char_cb(gpointer data, GtkWidget *widget)
 {
   GtkTreeView      *view;
   GtkTreeModel     *model;
