@@ -1,7 +1,7 @@
-/* $Id: we_generic_guied.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: we_generic_guied.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +27,12 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
 
 
-/********************
+/********************7
  * Type definitions *
  ********************/
 typedef void (*UpdateFunc)();
@@ -44,12 +43,12 @@ typedef void (*UpdateFunc)();
  ***********************/
 static void          we_guied_do_export(World       *world,
                                         const gchar *file,
-                                        GladeXML    *gladexml);
+                                        GtkBuilder  *ui_builder);
 static void          we_guied_export_selected_hooks(GString      *str,
                                                     GSList       *list,
                                                     const gchar  *name,
                                                     GtkTreeModel *model,
-                                                    GladeXML     *gladexml);
+                                                    GtkBuilder   *ui_builder);
 static void          we_guied_do_import(World *world, const gchar *filename);
 static void          we_guied_try_import(GSList      **oldlist,
                                          GSList       *newlist,
@@ -60,25 +59,39 @@ static void          we_guied_try_import(GSList      **oldlist,
 static void          we_guied_try_import_vars(World *world, World *newworld);
 static gboolean     *we_guied_get_pointer_to_bool_value(gpointer object,
                                                         gint column);
+/* XML UI signals */
+G_MODULE_EXPORT void we_guied_import_cb(GtkButton *button, gpointer data);
 
 
 
 void
 we_guied_export_cb(GtkButton *button, gpointer data)
 {
-  World     *world = (World *) data;
-  GladeXML  *gladexml;
-  GtkWidget *ExtraFilechooser;
-  GtkWidget *preselect;
-  GtkWidget *filechooser;
+  World           *world = (World *) data;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "ExtraFilechooser", NULL };
+  GError          *error = NULL;
+  GtkWidget       *ExtraFilechooser;
+  GtkToggleButton *preselect;
+  GtkWidget  *     filechooser;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "ExtraFilechooser", NULL);
-  ExtraFilechooser = glade_xml_get_widget(gladexml, "ExtraFilechooser");
-  preselect = glade_xml_get_widget(gladexml,
-                                   g_object_get_data(G_OBJECT(button),
-                                                     "preselect"));
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preselect), TRUE);
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return;
+  }
+
+  ExtraFilechooser = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                       "ExtraFilechooser"));
+  preselect
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                               g_object_get_data(G_OBJECT(button),
+                                                                 "preselect")));
+  gtk_toggle_button_set_active(preselect, TRUE);
 
   filechooser = gtk_file_chooser_dialog_new(_("Export"),
                                             GTK_WINDOW(world->dlgEditWorld),
@@ -95,17 +108,17 @@ we_guied_export_cb(GtkButton *button, gpointer data)
 
     gtk_widget_hide(filechooser);
     filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filechooser));
-    we_guied_do_export(world, filename, gladexml);
+    we_guied_do_export(world, filename, ui_builder);
     g_free(filename);
   }
   gtk_widget_destroy(filechooser);
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 }
 
 
 static
 void
-we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
+we_guied_do_export(World *world, const gchar *file, GtkBuilder *ui_builder)
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
@@ -123,7 +136,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
               "<kcworld-export>\n");
 
   /* Triggers */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "Trigger")))) {
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "Trigger")))) {
     selection = gtk_tree_view_get_selection(world->viewTrigger);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
@@ -147,7 +160,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
   }
 
   /* Aliases */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "Alias")))) {
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "Alias")))) {
     selection = gtk_tree_view_get_selection(world->viewAlias);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
@@ -171,7 +184,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
   }
 
   /* Macros */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "Macro")))) {
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "Macro")))) {
     selection = gtk_tree_view_get_selection(world->viewMacro);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
@@ -195,7 +208,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
   }
 
   /* Timers */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "Timer")))) {
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "Timer")))) {
     selection = gtk_tree_view_get_selection(world->viewTimer);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
@@ -219,7 +232,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
   }
 
   /* Permanent Variables */
-  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "Var")))) {
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "Var")))) {
     selection = gtk_tree_view_get_selection(world->viewVar);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
@@ -250,37 +263,37 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
                                  world->selected_OnConnect_hooks,
                                  "OnConnect",
                                  GTK_TREE_MODEL(world->hooks.OnConnect_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnDisconnect_hooks,
                                  "OnDisconnect",
                                  GTK_TREE_MODEL(world->hooks.OnDisconnect_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnReceivedText_hooks,
                                  "OnReceivedText",
                                  GTK_TREE_MODEL(world->hooks.OnReceivedText_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnSentCommand_hooks,
                                  "OnSentCommand",
                                  GTK_TREE_MODEL(world->hooks.OnSentCommand_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnGetFocus_hooks,
                                  "OnGetFocus",
                                  GTK_TREE_MODEL(world->hooks.OnGetFocus_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnLoseFocus_hooks,
                                  "OnLoseFocus",
                                  GTK_TREE_MODEL(world->hooks.OnLoseFocus_model_filter),
-                                 gladexml);
+                                 ui_builder);
   we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnCloseConnected_hooks,
                                  "OnCloseConnected",
                                  GTK_TREE_MODEL(world->hooks.OnCloseConnected_model_filter),
-                                 gladexml);
+                                 ui_builder);
   g_string_append(file_contents, "  </hooksv2>\n");
 
   g_string_append(file_contents, "</kcworld-export>\n");
@@ -310,13 +323,13 @@ we_guied_export_selected_hooks(GString      *str,
                                GSList       *list,
                                const gchar  *name,
                                GtkTreeModel *model,
-                               GladeXML     *gladexml)
+                               GtkBuilder   *ui_builder)
 {
   GSList      *listiter;
   GtkTreeIter  iter;
   Hook        *hook;
 
-  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, name)))) {
+  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, name)))) {
     return;
   }
 
