@@ -1,7 +1,7 @@
-/* $Id: we_hooks.c 1016 2009-02-02 12:10:40Z ekalin $ */
+/* $Id: we_hooks.c 1153 2011-03-20 15:41:50Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -38,7 +37,7 @@
  ***********************/
 static void          create_we_hooks_models(World *world);
 static GtkTreeModel *create_we_hooks_model(GSList *hookptr);
-static void          configure_view(GtkTreeView *view);
+static void          we_hook_configure_view(GtkTreeView *view);
 static void          we_hook_combo_changed_cb(GtkComboBox *combo,
                                               gpointer     data);
 static void          we_hook_save_currently_selected(GtkTreeModel     *model,
@@ -46,10 +45,10 @@ static void          we_hook_save_currently_selected(GtkTreeModel     *model,
 static void          we_hook_add_cb(GtkButton *button, gpointer data);
 static void          we_hook_edit_cb(GtkButton *button, gpointer data);
 static void          we_hook_delete_cb(GtkButton *button, gpointer data);
-static void          view_row_dblclick_cb(GtkTreeView        *view,
-                                          GtkTreePath        *path,
-                                          GtkTreeViewColumn  *col,
-                                          gpointer            data);
+static void          we_hook_view_row_dblclick_cb(GtkTreeView        *view,
+                                                  GtkTreePath        *path,
+                                                  GtkTreeViewColumn  *col,
+                                                  gpointer            data);
 static gboolean      edit_hook(World *world, const gchar *event, Hook *hook);
 static gboolean      we_hook_view_keypress_cb(GtkWidget   *widget,
                                               GdkEventKey *evt,
@@ -59,92 +58,104 @@ static void          we_hook_export_cb(GtkButton *button, gpointer data);
 
 
 
-GtkWidget *
+GObject *
 get_we_hooks_tab(World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *panel;
-  GtkWidget        *lblType;
-  GtkWidget        *viewObjects;
-  GtkWidget        *btnAdd;
-  GtkWidget        *btnEdit;
-  GtkWidget        *btnDelete;
-  GtkWidget        *btnUp;
-  GtkWidget        *btnDown;
-  GtkWidget        *btnImport;
-  GtkWidget        *btnExport;
+  GtkBuilder       *ui_builder;
+  gchar            *objects[] = { "we_panel_generic_graphical_editor", NULL };
+  GError           *error = NULL;
+  GObject          *panel;
+  GObject          *viewObjects;
+  GtkLabel         *lblType;
+  GObject          *btnAdd;
+  GObject          *btnEdit;
+  GObject          *btnDelete;
+  GObject          *btnUp;
+  GObject          *btnDown;
+  GObject          *btnExport;
   GtkTreeSelection *selection;
 
-  GtkWidget *label;
-  GtkWidget *cmbWhichHook;
-  GtkWidget *hboxWhich;
-  GtkWidget *vbox_int;
+  GtkWidget   *label;
+  GtkComboBox *cmbWhichHook;
+  GtkWidget   *hboxWhich;
+  GtkBox      *vbox_int;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "we_panel_generic_graphical_editor", NULL);
-  panel = glade_xml_get_widget(gladexml, "we_panel_generic_graphical_editor");
-  viewObjects = glade_xml_get_widget(gladexml, "viewObjects");
-  g_object_set_data(G_OBJECT(viewObjects), "forworld", world);
-  g_signal_connect(G_OBJECT(viewObjects), "row-activated",
-                   G_CALLBACK(view_row_dblclick_cb), world);
-  g_signal_connect(G_OBJECT(viewObjects), "key-press-event",
+  /* Create the dialog */
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+
+  panel = gtk_builder_get_object(ui_builder,
+                                 "we_panel_generic_graphical_editor");
+  /* So it's not destroyed when the builder is destroyed */
+  g_object_ref(panel);
+
+  viewObjects = gtk_builder_get_object(ui_builder, "viewObjects");
+  g_object_set_data(viewObjects, "forworld", world);
+  g_signal_connect(viewObjects, "row-activated",
+                   G_CALLBACK(we_hook_view_row_dblclick_cb), world);
+  g_signal_connect(viewObjects, "key-press-event",
                    G_CALLBACK(we_hook_view_keypress_cb), world);
 
-  lblType = glade_xml_get_widget(gladexml, "lblGEType");
-  gtk_label_set_markup(GTK_LABEL(lblType),
+  lblType = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblGEType"));
+  gtk_label_set_markup(lblType,
                        "<span weight=\"bold\">Hooks</span>");
 
-  btnAdd = glade_xml_get_widget(gladexml, "btnGEAdd");
-  g_signal_connect(G_OBJECT(btnAdd), "clicked",
+  btnAdd = gtk_builder_get_object(ui_builder, "btnGEAdd");
+  g_signal_connect(btnAdd, "clicked",
                    G_CALLBACK(we_hook_add_cb), viewObjects);
-  btnEdit = glade_xml_get_widget(gladexml, "btnGEEdit");
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
+  btnEdit = gtk_builder_get_object(ui_builder, "btnGEEdit");
+  g_signal_connect(btnEdit, "clicked",
                    G_CALLBACK(we_hook_edit_cb), viewObjects);
-  btnDelete = glade_xml_get_widget(gladexml, "btnGEDelete");
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
+  btnDelete = gtk_builder_get_object(ui_builder, "btnGEDelete");
+  g_signal_connect(btnDelete, "clicked",
                    G_CALLBACK(we_hook_delete_cb), viewObjects);
 
-  btnUp = glade_xml_get_widget(gladexml, "btnGEUp");
-  g_object_set_data(G_OBJECT(btnUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnUp), "clicked",
+  btnUp = gtk_builder_get_object(ui_builder, "btnGEUp");
+  g_object_set_data(btnUp, "isup", GINT_TO_POINTER(TRUE));
+  g_signal_connect(btnUp, "clicked",
                    G_CALLBACK(we_hook_move_cb), viewObjects);
-  btnDown = glade_xml_get_widget(gladexml, "btnGEDown");
-  g_object_set_data(G_OBJECT(btnDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnDown), "clicked",
+  btnDown = gtk_builder_get_object(ui_builder, "btnGEDown");
+  g_object_set_data(btnDown, "isup", GINT_TO_POINTER(FALSE));
+  g_signal_connect(btnDown, "clicked",
                    G_CALLBACK(we_hook_move_cb), viewObjects);
-  btnImport = glade_xml_get_widget(gladexml, "btnGEImport");
-  g_signal_connect(G_OBJECT(btnImport), "clicked",
-                   G_CALLBACK(we_guied_import_cb), world);
-  btnExport = glade_xml_get_widget(gladexml, "btnGEExport");
-  g_signal_connect(G_OBJECT(btnExport), "clicked",
-                   G_CALLBACK(we_hook_export_cb), viewObjects);
+  btnExport = gtk_builder_get_object(ui_builder, "btnGEExport");
 
   /* The manually added combo for selecting which hook */
   label = gtk_label_new_with_mnemonic(_("_Event:"));
-  cmbWhichHook = gtk_combo_box_new_text();
+  cmbWhichHook = GTK_COMBO_BOX(gtk_combo_box_new_text());
   g_signal_connect(G_OBJECT(cmbWhichHook), "changed",
                    G_CALLBACK(we_hook_combo_changed_cb), viewObjects);
-  gtk_label_set_mnemonic_widget(GTK_LABEL(label), cmbWhichHook);
+  gtk_label_set_mnemonic_widget(GTK_LABEL(label), GTK_WIDGET(cmbWhichHook));
   hboxWhich = gtk_hbox_new(FALSE, 8);
   gtk_box_pack_start(GTK_BOX(hboxWhich), label, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(hboxWhich), cmbWhichHook, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hboxWhich), GTK_WIDGET(cmbWhichHook),
+                     FALSE, FALSE, 0);
 
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnConnect");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnDisconnect");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnReceivedText");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnSentCommand");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnGetFocus");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnLoseFocus");
-  gtk_combo_box_append_text(GTK_COMBO_BOX(cmbWhichHook), "OnCloseConnected");
+  gtk_combo_box_append_text(cmbWhichHook, "OnConnect");
+  gtk_combo_box_append_text(cmbWhichHook, "OnDisconnect");
+  gtk_combo_box_append_text(cmbWhichHook, "OnReceivedText");
+  gtk_combo_box_append_text(cmbWhichHook, "OnSentCommand");
+  gtk_combo_box_append_text(cmbWhichHook, "OnGetFocus");
+  gtk_combo_box_append_text(cmbWhichHook, "OnLoseFocus");
+  gtk_combo_box_append_text(cmbWhichHook, "OnCloseConnected");
 
-  vbox_int = glade_xml_get_widget(gladexml,
-                                  "vbox_int_generic_graphical_editor");
-  gtk_box_pack_start(GTK_BOX(vbox_int), hboxWhich, FALSE, FALSE, 0);
-  gtk_box_reorder_child(GTK_BOX(vbox_int), hboxWhich, 0);
+  vbox_int
+    = GTK_BOX(gtk_builder_get_object(ui_builder,
+                                     "vbox_int_generic_graphical_editor"));
+  gtk_box_pack_start(vbox_int, hboxWhich, FALSE, FALSE, 0);
+  gtk_box_reorder_child(vbox_int, hboxWhich, 0);
 
   create_we_hooks_models(world);
 
-  configure_view(GTK_TREE_VIEW(viewObjects));
+  we_hook_configure_view(GTK_TREE_VIEW(viewObjects));
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(viewObjects));
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
   g_object_set_data(G_OBJECT(selection), "btnEdit",   btnEdit);
@@ -156,9 +167,18 @@ get_we_hooks_tab(World *world)
                    G_CALLBACK(we_guied_selection_changed_cb), NULL);
 
   /* Select first, will cause a model to be associated with the view */
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cmbWhichHook), 0);
+  gtk_combo_box_set_active(cmbWhichHook, 0);
 
-  g_object_unref(gladexml);
+  gtk_builder_connect_signals(ui_builder, world);
+  /* The callback for the Export buttons in the Hooks tab is a
+     different one - so we disconnect the handler from the XML UI file
+     and connect the correct one. */
+  g_signal_handlers_disconnect_by_func(btnExport, we_guied_export_cb, world);
+  g_signal_connect(btnExport, "clicked",
+                   G_CALLBACK(we_hook_export_cb), viewObjects);
+
+
+  g_object_unref(ui_builder);
   return panel;
 }
 
@@ -279,7 +299,7 @@ create_we_hooks_model(GSList *hookptr)
 
 static
 void
-configure_view(GtkTreeView *view)
+we_hook_configure_view(GtkTreeView *view)
 {
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
@@ -525,7 +545,7 @@ we_hook_delete_cb(GtkButton *button, gpointer data)
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &hook, -1);
         pos = g_slist_index(hooklist, hook);
-        delete_hook(world, event, pos);
+        delete_hook(world, event, pos, NULL);
 
         listiter = listiter->prev;
       }
@@ -540,10 +560,10 @@ we_hook_delete_cb(GtkButton *button, gpointer data)
 
 static
 void
-view_row_dblclick_cb(GtkTreeView        *view,
-                     GtkTreePath        *path,
-                     GtkTreeViewColumn  *col,
-                     gpointer            data)
+we_hook_view_row_dblclick_cb(GtkTreeView        *view,
+                             GtkTreePath        *path,
+                             GtkTreeViewColumn  *col,
+                             gpointer            data)
 {
   World        *world;
   GtkTreeModel *model;
@@ -570,7 +590,9 @@ static
 gboolean
 edit_hook(World *world, const gchar *event, Hook *hook)
 {
-  GladeXML        *gladexml;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "dlgEditHook", NULL };
+  GError          *error = NULL;
   GtkWidget       *dlgEditHook;
   GtkLabel        *lblEvent;
   GtkEntry        *txtName;
@@ -579,32 +601,40 @@ edit_hook(World *world, const gchar *event, Hook *hook)
   const gchar     *newname;
   const gchar     *newaction;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditHook", NULL);
-  dlgEditHook    = glade_xml_get_widget(gladexml, "dlgEditHook");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return FALSE;
+  }
+
+  dlgEditHook = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgEditHook"));
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditHook),
                                   GTK_RESPONSE_OK);
-  lblEvent   = (GtkLabel *) glade_xml_get_widget(gladexml,
-                                                 "lblHookEvent");
-  txtName    = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                 "txtHookName");
-  txtAction  = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                 "txtHookAction");
-  chkEnabled = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                        "chkHookEnabled");
+  lblEvent    = GTK_LABEL(gtk_builder_get_object(ui_builder,
+                                                 "lblHookEvent"));
+  txtName     = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                 "txtHookName"));
+  txtAction   = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                 "txtHookAction"));
+  chkEnabled  = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                         "chkHookEnabled"));
 
   /* Only display note when editing a plugin's trigger */
   if (!hook->owner_plugin) {
-    GtkWidget *label;
+    GObject *label;
 
-    label = glade_xml_get_widget(gladexml, "lblNoteHook");
-    gtk_widget_destroy(label);
-    label = glade_xml_get_widget(gladexml, "lblNoteTextHook");
-    gtk_widget_destroy(label);
+    label = gtk_builder_get_object(ui_builder, "lblNoteHook");
+    gtk_widget_destroy(GTK_WIDGET(label));
+    label = gtk_builder_get_object(ui_builder, "lblNoteTextHook");
+    gtk_widget_destroy(GTK_WIDGET(label));
   }
 
   /* We don't need it anymore */
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 
   /* Fill-in values */
   gtk_label_set_text(lblEvent, event);
