@@ -1,7 +1,7 @@
-/* $Id: we_logging.c 1024 2009-02-09 18:01:06Z ekalin $ */
+/* $Id: we_logging.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <string.h>
 #include <libintl.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <gmodule.h>
 
 #include "simocombobox.h"
@@ -38,45 +37,41 @@
 /***********************
  * Function prototypes *
  ***********************/
-static void we_log_start_cb(GtkWidget *widget, gpointer data);
-static void we_log_stop_cb(GtkWidget *widget, gpointer data);
-/* Glade signals */
+/* XML UI signals */
+G_MODULE_EXPORT void we_log_start_cb(GtkWidget *widget, gpointer data);
+G_MODULE_EXPORT void we_log_stop_cb(GtkWidget *widget, gpointer data);
 G_MODULE_EXPORT void select_logfile_cb(gpointer data, GtkWidget *widget);
 G_MODULE_EXPORT void revert_logformat_cb(gpointer data, GtkWidget *widget);
 
 
 
-GtkWidget *
-get_we_logging_tab(GladeXML *gladexml, World *world)
+GObject *
+get_we_logging_tab(GtkBuilder *ui_builder, World *world)
 {
-  GtkWidget *panel;
-  GtkWidget *btnStart;
-  GtkWidget *btnStop;
-  GtkWidget *image;
+  GObject *panel;
+  GObject *btnStart;
 
-  panel = glade_xml_get_widget(gladexml, "we_panel_logging");
+  panel = gtk_builder_get_object(ui_builder, "we_panel_logging");
 
   if (world != default_world) {
-    btnStart = glade_xml_get_widget(gladexml, "btnLogStart");
+    GtkWidget *image;
+
+    /* FIXME: Someday it should be possible to do this directly in the
+       XML file. */
+    btnStart = gtk_builder_get_object(ui_builder, "btnLogStart");
     image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
                                      GTK_ICON_SIZE_BUTTON);
     gtk_button_set_image(GTK_BUTTON(btnStart), image);
-    g_signal_connect(G_OBJECT(btnStart), "clicked",
-                     G_CALLBACK(we_log_start_cb), world);
-
-    btnStop = glade_xml_get_widget(gladexml, "btnLogStop");
-    g_signal_connect(G_OBJECT(btnStop), "clicked",
-                     G_CALLBACK(we_log_stop_cb), world);
   } else {
     /* These parts make no sense for the default world, so let's hide them */
-    GtkWidget *vbox_log_status;
-    GtkWidget *vbox_log_file;
+    GObject *vbox_log_status;
+    GObject *vbox_log_file;
 
-    vbox_log_status = glade_xml_get_widget(gladexml, "vbox_log_status");
-    vbox_log_file   = glade_xml_get_widget(gladexml, "vbox_log_file");
+    vbox_log_status = gtk_builder_get_object(ui_builder, "vbox_log_status");
+    vbox_log_file   = gtk_builder_get_object(ui_builder, "vbox_log_file");
 
-    g_object_set(G_OBJECT(vbox_log_status), "no-show-all", TRUE, NULL);
-    g_object_set(G_OBJECT(vbox_log_file),   "no-show-all", TRUE, NULL);
+    g_object_set(vbox_log_status, "no-show-all", TRUE, NULL);
+    g_object_set(vbox_log_file,   "no-show-all", TRUE, NULL);
   }
 
   return panel;
@@ -84,12 +79,12 @@ get_we_logging_tab(GladeXML *gladexml, World *world)
 
 
 void
-fill_we_logging_tab(World *world, GladeXML *gladexml)
+fill_we_logging_tab(World *world, GtkBuilder *ui_builder)
 {
   GtkLabel        *lblLogStatus;
   GtkWidget       *btnLogStart;
   GtkWidget       *btnLogStop;
-  GtkEntry        *txtLogFile;
+  GObject         *txtLogFile;
   GtkToggleButton *chkLogAuto;
   GtkToggleButton *chkLogAddTime;
   GtkEntry        *txtLogTimeFormat;
@@ -98,26 +93,29 @@ fill_we_logging_tab(World *world, GladeXML *gladexml)
     return;
   }
 
-  if (!gladexml) {
-  /* Called from Perl's XS_logfile. In this case there is no gladexml,
-     and the World Editor might not have been created yet. */
+  if (!ui_builder) {
+    /* Called from Perl's XS_logfile. In this case there is no ui_builder,
+       and the World Editor might not have been created yet. */
     if (!world->dlgEditWorld) {
       return;
     }
 
-    gladexml = glade_get_widget_tree(world->dlgEditWorld);
+    ui_builder = world->ui_builder;
   }
 
-  lblLogStatus     = GTK_LABEL(glade_xml_get_widget(gladexml, "lblLogStatus"));
-  btnLogStart      = glade_xml_get_widget(gladexml, "btnLogStart");
-  btnLogStop       = glade_xml_get_widget(gladexml, "btnLogStop");
-  txtLogFile       = GTK_ENTRY(glade_xml_get_widget(gladexml, "txtLogFile"));
-  chkLogAuto       = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
-                                                            "chkLogAuto"));
-  chkLogAddTime    = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
-                                                            "chkLogAddTime"));
-  txtLogTimeFormat = GTK_ENTRY(glade_xml_get_widget(gladexml,
-                                                    "txtLogTimeFormat"));
+  lblLogStatus     = GTK_LABEL(gtk_builder_get_object(ui_builder,
+                                                      "lblLogStatus"));
+  btnLogStart      = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                       "btnLogStart"));
+  btnLogStop       = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                       "btnLogStop"));
+  txtLogFile       = gtk_builder_get_object(ui_builder, "txtLogFile");
+  chkLogAuto       = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                              "chkLogAuto"));
+  chkLogAddTime    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                              "chkLogAddTime"));
+  txtLogTimeFormat = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                      "txtLogTimeFormat"));
 
   if (world->connected) {
     if (world->log_file) {
@@ -140,21 +138,28 @@ fill_we_logging_tab(World *world, GladeXML *gladexml)
   }
 
   if (world->log_file_name) {
-    gtk_entry_set_text(txtLogFile, world->log_file_name);
+    gtk_entry_set_text(GTK_ENTRY(txtLogFile), world->log_file_name);
+  } else {
+    gtk_entry_set_text(GTK_ENTRY(txtLogFile), "");
   }
 
   gtk_toggle_button_set_active(chkLogAuto, world->log_autostart);
   gtk_toggle_button_set_active(chkLogAddTime, world->log_add_time);
   if (world->log_timeformat) {
     gtk_entry_set_text(txtLogTimeFormat, world->log_timeformat);
+  } else {
+    gtk_entry_set_text(txtLogTimeFormat, "");
   }
+
+  /* Store pointer to dlgEditWorld so it can be used as a parent
+     in the select_logfile_cb callback */
+  g_object_set_data(txtLogFile, "dlgEditWorld", world->dlgEditWorld);
 }
 
 
 void
-update_we_logging_parameters(World *world)
+update_we_logging_parameters(World *world, GtkBuilder *ui_builder)
 {
-  GladeXML        *gladexml;
   GtkEntry        *txtLogFile;
   GtkToggleButton *chkLogAuto;
   GtkToggleButton *chkLogAddTime;
@@ -162,13 +167,12 @@ update_we_logging_parameters(World *world)
   const gchar     *filename;
   const gchar     *format;
 
-  gladexml         = glade_get_widget_tree(world->dlgEditWorld);
-  txtLogFile       = GTK_ENTRY(glade_xml_get_widget(gladexml, "txtLogFile"));
-  chkLogAuto       = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
+  txtLogFile       = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtLogFile"));
+  chkLogAuto       = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
                                                             "chkLogAuto"));
-  chkLogAddTime    = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
+  chkLogAddTime    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
                                                             "chkLogAddTime"));
-  txtLogTimeFormat = GTK_ENTRY(glade_xml_get_widget(gladexml,
+  txtLogTimeFormat = GTK_ENTRY(gtk_builder_get_object(ui_builder,
                                                     "txtLogTimeFormat"));
 
   g_free(world->log_file_name);
@@ -190,33 +194,30 @@ update_we_logging_parameters(World *world)
 }
 
 
-static
 void
 we_log_start_cb(GtkWidget *widget, gpointer data)
 {
   World           *world = (World *) data;
   gint             lines = 0;
   gchar           *errmsg = NULL;
-  GladeXML        *gladexml;
   GtkToggleButton *chkLogIncludePrevious;
   GtkSpinButton   *spnLogPreviousLines;
 
   /* Copy parameters in the World structure */
-  update_we_logging_parameters(world);
+  update_we_logging_parameters(world, world->ui_builder);
 
   /* Has the user asked for inclusion of lines in the window buffer? */
-  gladexml              = glade_get_widget_tree(world->dlgEditWorld);
   chkLogIncludePrevious
-    = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "chkLogIncludePrevious"));
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(world->ui_builder, "chkLogIncludePrevious"));
   spnLogPreviousLines
-    = GTK_SPIN_BUTTON(glade_xml_get_widget(gladexml, "spnLogPreviousLines"));
+    = GTK_SPIN_BUTTON(gtk_builder_get_object(world->ui_builder, "spnLogPreviousLines"));
 
   if (gtk_toggle_button_get_active(chkLogIncludePrevious)) {
     lines = (int) gtk_spin_button_get_value(spnLogPreviousLines);
   }
 
   if (start_log(world, lines, &errmsg)) {
-    fill_we_logging_tab(world, NULL);
+    fill_we_logging_tab(world, world->ui_builder);
   } else {
     GtkWidget *msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
                                                GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -231,13 +232,12 @@ we_log_start_cb(GtkWidget *widget, gpointer data)
 }
 
 
-static
 void
 we_log_stop_cb(GtkWidget *widget, gpointer data)
 {
-  World *world = (World *) data;
+  World      *world = (World *) data;
 
-  update_we_logging_parameters(world);
+  update_we_logging_parameters(world, world->ui_builder);
   stop_log(world);
   fill_we_logging_tab(world, NULL);
 }
@@ -247,14 +247,12 @@ void
 select_logfile_cb(gpointer data, GtkWidget *widget)
 {
   GtkEntry    *txtLogFile = GTK_ENTRY(data);
-  GladeXML    *gladexml;
   GtkWindow   *dlgEditWorld;
   GtkWidget   *filedlg;
   const gchar *filename;
 
-  gladexml = glade_get_widget_tree(widget);
-  dlgEditWorld = (GtkWindow *) g_object_get_data(G_OBJECT(gladexml),
-                                                 "dlgEditWorld");
+  dlgEditWorld = GTK_WINDOW(g_object_get_data(G_OBJECT(txtLogFile),
+                                              "dlgEditWorld"));
 
   filedlg = gtk_file_chooser_dialog_new(_("Select log file"),
                                         dlgEditWorld,
