@@ -1,7 +1,7 @@
-/* $Id: we_macros.c 1016 2009-02-02 12:10:40Z ekalin $ */
+/* $Id: we_macros.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -37,80 +36,91 @@
  * Function prototypes *
  ***********************/
 static GtkTreeModel *create_we_macros_model(World *world);
-static void          configure_view(GtkTreeView *view, GtkTreeModel *model);
+static void          we_macro_configure_view(GtkTreeView *view,
+                                             GtkTreeModel *model);
 static void          we_macro_add_cb(GtkButton *button, gpointer data);
 static void          we_macro_edit_cb(GtkButton *button, gpointer data);
 static void          we_macro_delete_cb(GtkButton *button, gpointer data);
-static void          view_row_dblclick_cb(GtkTreeView        *view,
-                                          GtkTreePath        *path,
-                                          GtkTreeViewColumn  *col,
-                                          gpointer            data);
+static void          we_macro_view_row_dblclick_cb(GtkTreeView        *view,
+                                                   GtkTreePath        *path,
+                                                   GtkTreeViewColumn  *col,
+                                                   gpointer            data);
 static gboolean      edit_macro(World *world, Macro *macro);
 static gboolean      we_macro_view_keypress_cb(GtkWidget   *widget,
                                                GdkEventKey *evt,
                                                gpointer     data);
 static void          we_macro_move_cb(GtkButton *button, gpointer data);
-/* Glade signals */
+/* XML UI signals */
 G_MODULE_EXPORT gboolean we_macro_txtname_keypress_cb(GtkWidget   *widget,
                                                       GdkEventKey *evt,
                                                       gpointer     data);
 
 
-GtkWidget *
+GObject *
 get_we_macros_tab(World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *panel;
-  GtkWidget        *lblType;
-  GtkWidget        *btnAdd;
-  GtkWidget        *btnEdit;
-  GtkWidget        *btnDelete;
-  GtkWidget        *btnUp;
-  GtkWidget        *btnDown;
-  GtkWidget        *btnImport;
-  GtkWidget        *btnExport;
+  GtkBuilder       *ui_builder;
+  gchar            *objects[] = { "we_panel_generic_graphical_editor", NULL };
+  GError           *error = NULL;
+  GObject          *panel;
+  GtkLabel         *lblType;
+  GObject          *btnAdd;
+  GObject          *btnEdit;
+  GObject          *btnDelete;
+  GObject          *btnUp;
+  GObject          *btnDown;
+  GObject          *btnExport;
   GtkTreeSelection *selection;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "we_panel_generic_graphical_editor", NULL);
-  panel = glade_xml_get_widget(gladexml, "we_panel_generic_graphical_editor");
-  world->viewMacro = GTK_TREE_VIEW(glade_xml_get_widget(gladexml,
+  /* Create the dialog */
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+
+  panel = gtk_builder_get_object(ui_builder,
+                                 "we_panel_generic_graphical_editor");
+  /* So it's not destroyed when the builder is destroyed */
+  g_object_ref(panel);
+
+  world->viewMacro = GTK_TREE_VIEW(gtk_builder_get_object(ui_builder,
                                                         "viewObjects"));
   g_object_set_data(G_OBJECT(world->viewMacro), "forworld", world);
   g_signal_connect(G_OBJECT(world->viewMacro), "row-activated",
-                   G_CALLBACK(view_row_dblclick_cb), world);
+                   G_CALLBACK(we_macro_view_row_dblclick_cb), world);
   g_signal_connect(G_OBJECT(world->viewMacro), "key-press-event",
                    G_CALLBACK(we_macro_view_keypress_cb), world);
 
-  lblType = glade_xml_get_widget(gladexml, "lblGEType");
-  gtk_label_set_markup(GTK_LABEL(lblType),
-                       "<span weight=\"bold\">Macros</span>");
+  lblType = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblGEType"));
+  gtk_label_set_markup(lblType, "<span weight=\"bold\">Macros</span>");
 
-  btnAdd = glade_xml_get_widget(gladexml, "btnGEAdd");
-  g_signal_connect(G_OBJECT(btnAdd), "clicked",
+  btnAdd = gtk_builder_get_object(ui_builder, "btnGEAdd");
+  g_signal_connect(btnAdd, "clicked",
                    G_CALLBACK(we_macro_add_cb), world->viewMacro);
-  btnEdit = glade_xml_get_widget(gladexml, "btnGEEdit");
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
+  btnEdit = gtk_builder_get_object(ui_builder, "btnGEEdit");
+  g_signal_connect(btnEdit, "clicked",
                    G_CALLBACK(we_macro_edit_cb), world->viewMacro);
-  btnDelete = glade_xml_get_widget(gladexml, "btnGEDelete");
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
+  btnDelete = gtk_builder_get_object(ui_builder, "btnGEDelete");
+  g_signal_connect(btnDelete, "clicked",
                    G_CALLBACK(we_macro_delete_cb), world->viewMacro);
 
-  btnUp = glade_xml_get_widget(gladexml, "btnGEUp");
-  g_object_set_data(G_OBJECT(btnUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnUp), "clicked",
+  btnUp = gtk_builder_get_object(ui_builder, "btnGEUp");
+  g_object_set_data(btnUp, "isup", GINT_TO_POINTER(TRUE));
+  g_signal_connect(btnUp, "clicked",
                    G_CALLBACK(we_macro_move_cb), world->viewMacro);
-  btnDown = glade_xml_get_widget(gladexml, "btnGEDown");
-  g_object_set_data(G_OBJECT(btnDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnDown), "clicked",
+  btnDown = gtk_builder_get_object(ui_builder, "btnGEDown");
+  g_object_set_data(btnDown, "isup", GINT_TO_POINTER(FALSE));
+  g_signal_connect(btnDown, "clicked",
                    G_CALLBACK(we_macro_move_cb), world->viewMacro);
-  btnImport = glade_xml_get_widget(gladexml, "btnGEImport");
-  g_signal_connect(G_OBJECT(btnImport), "clicked",
-                   G_CALLBACK(we_guied_import_cb), world);
-  btnExport = glade_xml_get_widget(gladexml, "btnGEExport");
-  g_object_set_data(G_OBJECT(btnExport), "preselect", "Macro");
-  g_signal_connect(G_OBJECT(btnExport), "clicked",
-                   G_CALLBACK(we_guied_export_cb), world);
+
+  btnExport = gtk_builder_get_object(ui_builder, "btnGEExport");
+  g_object_set_data(btnExport, "preselect", "Macro");
 
   world->macro_model = create_we_macros_model(world);
   world->macro_model_filter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(world->macro_model, NULL));
@@ -120,7 +130,8 @@ get_we_macros_tab(World *world)
                                          world,
                                          NULL);
 
-  configure_view(world->viewMacro, GTK_TREE_MODEL(world->macro_model_filter));
+  we_macro_configure_view(world->viewMacro,
+                          GTK_TREE_MODEL(world->macro_model_filter));
   selection = gtk_tree_view_get_selection(world->viewMacro);
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
   g_object_set_data(G_OBJECT(selection), "btnEdit",   btnEdit);
@@ -131,7 +142,8 @@ get_we_macros_tab(World *world)
   g_signal_connect(G_OBJECT(selection), "changed",
                    G_CALLBACK(we_guied_selection_changed_cb), NULL);
 
-  g_object_unref(gladexml);
+  gtk_builder_connect_signals(ui_builder, world);
+  g_object_unref(ui_builder);
   return panel;
 }
 
@@ -165,7 +177,7 @@ create_we_macros_model(World *world)
 
 static
 void
-configure_view(GtkTreeView *view, GtkTreeModel *model)
+we_macro_configure_view(GtkTreeView *view, GtkTreeModel *model)
 {
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
@@ -360,10 +372,10 @@ we_macro_delete_cb(GtkButton *button, gpointer data)
 
 static
 void
-view_row_dblclick_cb(GtkTreeView        *view,
-                     GtkTreePath        *path,
-                     GtkTreeViewColumn  *col,
-                     gpointer            data)
+we_macro_view_row_dblclick_cb(GtkTreeView        *view,
+                              GtkTreePath        *path,
+                              GtkTreeViewColumn  *col,
+                              gpointer            data)
 {
   World        *world;
   GtkTreeModel *model;
@@ -388,7 +400,9 @@ static
 gboolean
 edit_macro(World *world, Macro *macro)
 {
-  GladeXML        *gladexml;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "dlgEditMacro", NULL };
+  GError          *error = NULL;
   GtkWidget       *dlgEditMacro;
   GtkEntry        *txtName;
   GtkEntry        *txtKey;
@@ -400,33 +414,41 @@ edit_macro(World *world, Macro *macro)
   GdkModifierType  newmodifiers;
   const gchar     *newaction;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditMacro", NULL);
-  glade_xml_signal_autoconnect(gladexml);
-  dlgEditMacro    = glade_xml_get_widget(gladexml, "dlgEditMacro");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return FALSE;
+  }
+  gtk_builder_connect_signals(ui_builder, NULL);
+
+  dlgEditMacro = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgEditMacro"));
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditMacro),
                                   GTK_RESPONSE_OK);
-  txtName    = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                 "txtMacroName");
-  txtKey     = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                 "txtMacroKey");
-  txtAction  = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                 "txtMacroAction");
-  chkEnabled = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                        "chkMacroEnabled");
+  txtName      = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtMacroName"));
+  txtKey       = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtMacroKey"));
+  txtAction    = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtMacroAction"));
+  chkEnabled   = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkMacroEnabled"));
 
   /* Only display note when editing a plugin's macro */
   if (!macro->owner_plugin) {
-    GtkWidget *label;
+    GObject *label;
 
-    label = glade_xml_get_widget(gladexml, "lblNoteMacro");
-    gtk_widget_destroy(label);
-    label = glade_xml_get_widget(gladexml, "lblNoteTextMacro");
-    gtk_widget_destroy(label);
+    label = gtk_builder_get_object(ui_builder, "lblNoteMacro");
+    gtk_widget_destroy(GTK_WIDGET(label));
+    label = gtk_builder_get_object(ui_builder, "lblNoteTextMacro");
+    gtk_widget_destroy(GTK_WIDGET(label));
   }
 
   /* We don't need it anymore */
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 
   /* Fill-in values */
   if (macro->name) {
