@@ -1,7 +1,7 @@
-/* $Id: we_plugins.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: we_plugins.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -37,12 +36,13 @@
  * Function prototypes - Startup Plugins *
  *****************************************/
 static void     configure_startup_view(GtkTreeView *view, World *world);
-static void     we_splugin_add_cb(GtkButton *button, gpointer data);
-static void     we_splugin_remove_cb(GtkButton *button, gpointer data);
-static gboolean we_splugin_view_keypress_cb(GtkWidget   *widget,
-                                            GdkEventKey *evt,
-                                            gpointer     data);
-static void     we_splugin_move_cb(GtkButton *button, gpointer data);
+/* XML UI signals */
+G_MODULE_EXPORT void     we_splugin_add_cb(gpointer data, GtkButton *button);
+G_MODULE_EXPORT void     we_splugin_remove_cb(gpointer data, GtkButton *button);
+G_MODULE_EXPORT void     we_splugin_move_cb(gpointer data, GtkButton *button);
+G_MODULE_EXPORT gboolean we_splugin_view_keypress_cb(GtkWidget   *widget,
+                                                     GdkEventKey *evt,
+                                                     gpointer     data);
 
 /**************************************************
  * Function prototypes - Currently Loaded Plugins *
@@ -53,8 +53,9 @@ static void          configure_loaded_view(GtkTreeView  *view,
 static void          we_lplugin_ena_toggled_cb(GtkCellRendererToggle *renderer,
                                                gchar                 *pathstr,
                                                gpointer               data);
-static void          we_lplugin_load_cb(GtkButton *button, gpointer data);
-static void          we_lplugin_help_cb(GtkButton *button, gpointer data);
+/* XML UI signals */
+G_MODULE_EXPORT void we_lplugin_load_cb(gpointer data, GtkButton *button);
+G_MODULE_EXPORT void we_lplugin_help_cb(gpointer data, GtkButton *button);
 
 
 /*********************************
@@ -64,63 +65,41 @@ static GtkWidget *we_plugin_get_load_filechooser(GtkWindow *parent);
 
 
 
-GtkWidget *
-get_we_plugins_tab(GladeXML *gladexml, World *world)
+GObject *
+get_we_plugins_tab(GtkBuilder *ui_builder, World *world)
 {
-  GtkWidget *panel;
+  GObject *panel;
 
-  GtkWidget *lstStartupPlugins;
-  GtkWidget *btnSPluginAdd;
-  GtkWidget *btnSPluginRemove;
-  GtkWidget *btnSPluginUp;
-  GtkWidget *btnSPluginDown;
+  GObject *lstLoadedPlugins;
+  GObject *btnLPluginLoad;
 
-  GtkWidget *lstLoadedPlugins;
-  GtkWidget *btnLPluginLoad;
-  GtkWidget *btnLPluginHelp;
+  GObject *lstStartupPlugins;
+  GObject *btnSPluginUp;
+  GObject *btnSPluginDown;
 
-  panel = glade_xml_get_widget(gladexml, "we_panel_plugins");
+  panel = gtk_builder_get_object(ui_builder, "we_panel_plugins");
 
   /* Currently loaded plugins part */
-  lstLoadedPlugins = glade_xml_get_widget(gladexml, "lstLoadedPlugins");
-  g_object_set_data(G_OBJECT(lstLoadedPlugins), "forworld", world);
+  lstLoadedPlugins = gtk_builder_get_object(ui_builder, "lstLoadedPlugins");
+  g_object_set_data(lstLoadedPlugins, "forworld", world);
 
-  btnLPluginLoad = glade_xml_get_widget(gladexml, "btnLPluginLoad");
-  g_signal_connect(G_OBJECT(btnLPluginLoad), "clicked",
-                   G_CALLBACK(we_lplugin_load_cb), lstLoadedPlugins);
+  btnLPluginLoad = gtk_builder_get_object(ui_builder, "btnLPluginLoad");
   if (!world->connected) {
-    gtk_widget_set_sensitive(btnLPluginLoad, FALSE);
+    gtk_widget_set_sensitive(GTK_WIDGET(btnLPluginLoad), FALSE);
   }
 
-  btnLPluginHelp = glade_xml_get_widget(gladexml, "btnLPluginHelp");
-  g_signal_connect(G_OBJECT(btnLPluginHelp), "clicked",
-                   G_CALLBACK(we_lplugin_help_cb), lstLoadedPlugins);
-
   world->plugin_model = create_we_lplugins_model(world);
   configure_loaded_view(GTK_TREE_VIEW(lstLoadedPlugins), world->plugin_model);
 
 
   /* Startup plugins part */
-  lstStartupPlugins = glade_xml_get_widget(gladexml, "lstStartupPlugins");
-  g_object_set_data(G_OBJECT(lstStartupPlugins), "forworld", world);
-  g_signal_connect(G_OBJECT(lstStartupPlugins), "key-press-event",
-                   G_CALLBACK(we_splugin_view_keypress_cb), world);
+  lstStartupPlugins = gtk_builder_get_object(ui_builder, "lstStartupPlugins");
+  g_object_set_data(lstStartupPlugins, "forworld", world);
 
-  btnSPluginAdd = glade_xml_get_widget(gladexml, "btnSPluginAdd");
-  g_signal_connect(G_OBJECT(btnSPluginAdd), "clicked",
-                   G_CALLBACK(we_splugin_add_cb), lstStartupPlugins);
-  btnSPluginRemove = glade_xml_get_widget(gladexml, "btnSPluginRemove");
-  g_signal_connect(G_OBJECT(btnSPluginRemove), "clicked",
-                   G_CALLBACK(we_splugin_remove_cb), lstStartupPlugins);
-
-  btnSPluginUp = glade_xml_get_widget(gladexml, "btnSPluginUp");
-  g_object_set_data(G_OBJECT(btnSPluginUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnSPluginUp), "clicked",
-                   G_CALLBACK(we_splugin_move_cb), lstStartupPlugins);
-  btnSPluginDown = glade_xml_get_widget(gladexml, "btnSPluginDown");
-  g_object_set_data(G_OBJECT(btnSPluginDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnSPluginDown), "clicked",
-                   G_CALLBACK(we_splugin_move_cb), lstStartupPlugins);
+  btnSPluginUp = gtk_builder_get_object(ui_builder, "btnSPluginUp");
+  g_object_set_data(btnSPluginUp, "isup", GINT_TO_POINTER(TRUE));
+  btnSPluginDown = gtk_builder_get_object(ui_builder, "btnSPluginDown");
+  g_object_set_data(btnSPluginDown, "isup", GINT_TO_POINTER(FALSE));
 
   configure_startup_view(GTK_TREE_VIEW(lstStartupPlugins), world);
 
@@ -193,9 +172,8 @@ configure_startup_view(GtkTreeView *view, World *world)
 }
 
 
-static
 void
-we_splugin_add_cb(GtkButton *button, gpointer data)
+we_splugin_add_cb(gpointer data, GtkButton *button)
 {
   World       *world;
   GtkTreeView *view;
@@ -240,9 +218,8 @@ we_splugin_add_cb(GtkButton *button, gpointer data)
 }
 
 
-static
 void
-we_splugin_remove_cb(GtkButton *button, gpointer data)
+we_splugin_remove_cb(gpointer data, GtkButton *button)
 {
   GtkTreeView      *view;
   GtkTreeModel     *model;
@@ -257,9 +234,8 @@ we_splugin_remove_cb(GtkButton *button, gpointer data)
 }
 
 
-static
 void
-we_splugin_move_cb(GtkButton *button, gpointer data)
+we_splugin_move_cb(gpointer data, GtkButton *button)
 {
   GtkTreeView      *view;
   GtkTreeModel     *model;
@@ -291,14 +267,13 @@ we_splugin_move_cb(GtkButton *button, gpointer data)
 }
 
 
-static
 gboolean
-we_splugin_view_keypress_cb(GtkWidget   *widget,
+we_splugin_view_keypress_cb(GtkWidget   *widget, /* The treeview */
                             GdkEventKey *evt,
                             gpointer     data)
 {
   if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
-    we_splugin_remove_cb(NULL, widget);
+    we_splugin_remove_cb(widget, NULL);
     return TRUE;
   }
 
@@ -454,9 +429,8 @@ we_lplugin_ena_toggled_cb(GtkCellRendererToggle *renderer,
 }
 
 
-static
 void
-we_lplugin_load_cb(GtkButton *button, gpointer data)
+we_lplugin_load_cb(gpointer data, GtkButton *button)
 {
   World        *world;
   GtkTreeView  *view;
@@ -500,9 +474,8 @@ we_lplugin_load_cb(GtkButton *button, gpointer data)
 }
 
 
-static
 void
-we_lplugin_help_cb(GtkButton *button, gpointer data)
+we_lplugin_help_cb(gpointer data, GtkButton *button)
 {
   World            *world;
   GtkTreeView      *view;
@@ -534,7 +507,8 @@ we_lplugin_help_cb(GtkButton *button, gpointer data)
                                       _("The help is displayed in main window."));
       gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Information"));
       chk = gtk_check_button_new_with_mnemonic(_("_Do not display this message again"));
-      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(msgdlg)->vbox), chk,
+      gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(msgdlg))),
+                         chk,
                          FALSE, FALSE, 4);
 
       gtk_widget_show_all(msgdlg);
