@@ -1,7 +1,7 @@
-/* $Id: we_scripting.c 1014 2009-02-01 12:46:52Z ekalin $ */
+/* $Id: we_scripting.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <string.h>
 #include <libintl.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -42,54 +41,44 @@
  *************************/
 
 
-GtkWidget *
-get_we_scripting_tab(GtkWidget *wndParent, GladeXML *gladexml)
+GObject *
+get_we_scripting_tab(GtkBuilder *ui_builder)
 {
-  GtkWidget *panel;
-  GtkWidget *txtScriptFile;
-  GtkWidget *btnSelectNone;
+  GObject *panel;
 
-  panel = glade_xml_get_widget(gladexml, "we_panel_scripting");
-
-  txtScriptFile = glade_xml_get_widget(gladexml, "txtScriptFile");
-  btnSelectNone = glade_xml_get_widget(gladexml, "btnSelectNone");
-
-  g_signal_connect_swapped(G_OBJECT(btnSelectNone), "clicked",
-                           G_CALLBACK(gtk_file_chooser_unselect_all),
-                           txtScriptFile);
+  panel = gtk_builder_get_object(ui_builder, "we_panel_scripting");
 
   return panel;
 }
 
 
 void
-fill_we_scripting_tab(World *world, GladeXML *gladexml)
+fill_we_scripting_tab(World *world, GtkBuilder *ui_builder)
 {
-  GtkWidget *txtScriptFile;
+  GtkFileChooser *txtScriptFile;
 
   if (!world) {
     return;
   }
 
-  txtScriptFile = glade_xml_get_widget(gladexml, "txtScriptFile");
+  txtScriptFile = GTK_FILE_CHOOSER(gtk_builder_get_object(ui_builder,
+                                                          "txtScriptFile"));
   if (world->scriptfile) {
-    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(txtScriptFile),
-                                  world->scriptfile);
+    gtk_file_chooser_set_filename(txtScriptFile, world->scriptfile);
   }
 }
 
 
 
 void
-update_we_scripting_parameters(World *world)
+update_we_scripting_parameters(World *world, GtkBuilder *ui_builder)
 {
-  GladeXML  *gladexml;
-  GtkWidget *txtScriptFile;
-  gchar     *newval;
+  GtkFileChooser *txtScriptFile;
+  gchar          *newval;
 
-  gladexml = glade_get_widget_tree(world->dlgEditWorld);
-  txtScriptFile = glade_xml_get_widget(gladexml, "txtScriptFile");
-  newval = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(txtScriptFile));
+  txtScriptFile = GTK_FILE_CHOOSER(gtk_builder_get_object(ui_builder,
+                                                          "txtScriptFile"));
+  newval = gtk_file_chooser_get_filename(txtScriptFile);
   if (!newval) {
     g_free(world->scriptfile);
     world->scriptfile = NULL;
