@@ -1,7 +1,7 @@
-/* $Id: we_timers.c 1016 2009-02-02 12:10:40Z ekalin $ */
+/* $Id: we_timers.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -37,14 +36,15 @@
  * Function prototypes *
  ***********************/
 static GtkTreeModel *create_we_timers_model(World *world);
-static void          configure_view(GtkTreeView *view, GtkTreeModel *model);
+static void          we_timer_configure_view(GtkTreeView *view,
+                                             GtkTreeModel *model);
 static void          we_timer_add_cb(GtkButton *button, gpointer data);
 static void          we_timer_edit_cb(GtkButton *button, gpointer data);
 static void          we_timer_delete_cb(GtkButton *button, gpointer data);
-static void          view_row_dblclick_cb(GtkTreeView        *view,
-                                          GtkTreePath        *path,
-                                          GtkTreeViewColumn  *col,
-                                          gpointer            data);
+static void          we_timer_view_row_dblclick_cb(GtkTreeView        *view,
+                                                   GtkTreePath        *path,
+                                                   GtkTreeViewColumn  *col,
+                                                   gpointer            data);
 static gboolean      edit_timer(World *world, Timer *timer, gboolean editing);
 static void          we_timer_move_cb(GtkButton *button, gpointer data);
 static gboolean      we_timer_view_keypress_cb(GtkWidget   *widget,
@@ -53,61 +53,71 @@ static gboolean      we_timer_view_keypress_cb(GtkWidget   *widget,
 
 
 
-GtkWidget *
+GObject *
 get_we_timers_tab(World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *panel;
-  GtkWidget        *lblType;
-  GtkWidget        *btnAdd;
-  GtkWidget        *btnEdit;
-  GtkWidget        *btnDelete;
-  GtkWidget        *btnUp;
-  GtkWidget        *btnDown;
-  GtkWidget        *btnImport;
-  GtkWidget        *btnExport;
+  GtkBuilder       *ui_builder;
+  gchar            *objects[] = { "we_panel_generic_graphical_editor", NULL };
+  GError           *error = NULL;
+  GObject          *panel;
+  GtkLabel         *lblType;
+  GObject          *btnAdd;
+  GObject          *btnEdit;
+  GObject          *btnDelete;
+  GObject          *btnUp;
+  GObject          *btnDown;
+  GObject          *btnExport;
   GtkTreeSelection *selection;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "we_panel_generic_graphical_editor", NULL);
-  panel = glade_xml_get_widget(gladexml, "we_panel_generic_graphical_editor");
-  world->viewTimer = GTK_TREE_VIEW(glade_xml_get_widget(gladexml,
+  /* Create the dialog */
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+
+  panel = gtk_builder_get_object(ui_builder,
+                                 "we_panel_generic_graphical_editor");
+  /* So it's not destroyed when the builder is destroyed */
+  g_object_ref(panel);
+
+  world->viewTimer = GTK_TREE_VIEW(gtk_builder_get_object(ui_builder,
                                                         "viewObjects"));
   g_object_set_data(G_OBJECT(world->viewTimer), "forworld", world);
   g_signal_connect(G_OBJECT(world->viewTimer), "row-activated",
-                   G_CALLBACK(view_row_dblclick_cb), world);
+                   G_CALLBACK(we_timer_view_row_dblclick_cb), world);
   g_signal_connect(G_OBJECT(world->viewTimer), "key-press-event",
                    G_CALLBACK(we_timer_view_keypress_cb), world);
 
-  lblType = glade_xml_get_widget(gladexml, "lblGEType");
-  gtk_label_set_markup(GTK_LABEL(lblType),
-                       "<span weight=\"bold\">Timers</span>");
+  lblType = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblGEType"));
+  gtk_label_set_markup(lblType, "<span weight=\"bold\">Timers</span>");
 
-  btnAdd = glade_xml_get_widget(gladexml, "btnGEAdd");
-  g_signal_connect(G_OBJECT(btnAdd), "clicked",
+  btnAdd = gtk_builder_get_object(ui_builder, "btnGEAdd");
+  g_signal_connect(btnAdd, "clicked",
                    G_CALLBACK(we_timer_add_cb), world->viewTimer);
-  btnEdit = glade_xml_get_widget(gladexml, "btnGEEdit");
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
+  btnEdit = gtk_builder_get_object(ui_builder, "btnGEEdit");
+  g_signal_connect(btnEdit, "clicked",
                    G_CALLBACK(we_timer_edit_cb), world->viewTimer);
-  btnDelete = glade_xml_get_widget(gladexml, "btnGEDelete");
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
+  btnDelete = gtk_builder_get_object(ui_builder, "btnGEDelete");
+  g_signal_connect(btnDelete, "clicked",
                    G_CALLBACK(we_timer_delete_cb), world->viewTimer);
 
-  btnUp = glade_xml_get_widget(gladexml, "btnGEUp");
-  g_object_set_data(G_OBJECT(btnUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnUp), "clicked",
+  btnUp = gtk_builder_get_object(ui_builder, "btnGEUp");
+  g_object_set_data(btnUp, "isup", GINT_TO_POINTER(TRUE));
+  g_signal_connect(btnUp, "clicked",
                    G_CALLBACK(we_timer_move_cb), world->viewTimer);
-  btnDown = glade_xml_get_widget(gladexml, "btnGEDown");
-  g_object_set_data(G_OBJECT(btnDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnDown), "clicked",
+  btnDown = gtk_builder_get_object(ui_builder, "btnGEDown");
+  g_object_set_data(btnDown, "isup", GINT_TO_POINTER(FALSE));
+  g_signal_connect(btnDown, "clicked",
                    G_CALLBACK(we_timer_move_cb), world->viewTimer);
-  btnImport = glade_xml_get_widget(gladexml, "btnGEImport");
-  g_signal_connect(G_OBJECT(btnImport), "clicked",
-                   G_CALLBACK(we_guied_import_cb), world);
-  btnExport = glade_xml_get_widget(gladexml, "btnGEExport");
-  g_object_set_data(G_OBJECT(btnExport), "preselect", "Timer");
-  g_signal_connect(G_OBJECT(btnExport), "clicked",
-                   G_CALLBACK(we_guied_export_cb), world);
+
+  btnExport = gtk_builder_get_object(ui_builder, "btnGEExport");
+  g_object_set_data(btnExport, "preselect", "Timer");
 
   world->timer_model = create_we_timers_model(world);
   world->timer_model_filter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(world->timer_model, NULL));
@@ -117,7 +127,8 @@ get_we_timers_tab(World *world)
                                          world,
                                          NULL);
 
-  configure_view(world->viewTimer, GTK_TREE_MODEL(world->timer_model_filter));
+  we_timer_configure_view(world->viewTimer,
+                          GTK_TREE_MODEL(world->timer_model_filter));
   selection = gtk_tree_view_get_selection(world->viewTimer);
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
   g_object_set_data(G_OBJECT(selection), "btnEdit",   btnEdit);
@@ -128,7 +139,8 @@ get_we_timers_tab(World *world)
   g_signal_connect(G_OBJECT(selection), "changed",
                    G_CALLBACK(we_guied_selection_changed_cb), NULL);
 
-  g_object_unref(gladexml);
+  gtk_builder_connect_signals(ui_builder, world);
+  g_object_unref(ui_builder);
   return panel;
 }
 
@@ -162,7 +174,7 @@ create_we_timers_model(World *world)
 
 static
 void
-configure_view(GtkTreeView *view, GtkTreeModel *model)
+we_timer_configure_view(GtkTreeView *view, GtkTreeModel *model)
 {
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
@@ -395,10 +407,10 @@ we_timer_delete_cb(GtkButton *button, gpointer data)
 
 static
 void
-view_row_dblclick_cb(GtkTreeView        *view,
-                     GtkTreePath        *path,
-                     GtkTreeViewColumn  *col,
-                     gpointer            data)
+we_timer_view_row_dblclick_cb(GtkTreeView        *view,
+                              GtkTreePath        *path,
+                              GtkTreeViewColumn  *col,
+                              gpointer            data)
 {
   World        *world;
   GtkTreeModel *model;
@@ -423,7 +435,10 @@ static
 gboolean
 edit_timer(World *world, Timer *timer, gboolean editing)
 {
-  GladeXML        *gladexml;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "dlgEditTimer",
+                                 "adjustment1", "adjustment2", NULL };
+  GError          *error = NULL;
   GtkWidget       *dlgEditTimer;
   GtkEntry        *txtName;
   GtkSpinButton   *spnInterval;
@@ -436,36 +451,43 @@ edit_timer(World *world, Timer *timer, gboolean editing)
   gboolean         newenabled;
   gint             newinterval;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditTimer", NULL);
-  dlgEditTimer    = glade_xml_get_widget(gladexml, "dlgEditTimer");
-  gtk_dialog_set_default_response(GTK_DIALOG(dlgEditTimer),
-                                  GTK_RESPONSE_OK);
-  txtName      = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                   "txtTimerName");
-  txtAction    = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                   "txtTimerAction");
-  spnInterval  = (GtkSpinButton *) glade_xml_get_widget(gladexml,
-                                                        "spnTimerInterval");
-  spnCount     = (GtkSpinButton *) glade_xml_get_widget(gladexml,
-                                                        "spnTimerCount");
-  chkTemporary = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTimerTemporary");
-  chkEnabled   = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                         "chkTimerEnabled");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return FALSE;
+  }
+
+  dlgEditTimer = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgEditTimer"));
+  gtk_dialog_set_default_response(GTK_DIALOG(dlgEditTimer), GTK_RESPONSE_OK);
+  txtName      = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtTimerName"));
+  txtAction    = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtTimerAction"));
+  spnInterval  = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder,
+                                                        "spnTimerInterval"));
+  spnCount     = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder,
+                                                        "spnTimerCount"));
+  chkTemporary = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTimerTemporary"));
+  chkEnabled   = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTimerEnabled"));
 
   /* Only display note when editing a plugin's timer */
   if (!timer->owner_plugin) {
-    GtkWidget *label;
+    GObject *label;
 
-    label = glade_xml_get_widget(gladexml, "lblNoteTimer");
-    gtk_widget_destroy(label);
-    label = glade_xml_get_widget(gladexml, "lblNoteTextTimer");
-    gtk_widget_destroy(label);
+    label = gtk_builder_get_object(ui_builder, "lblNoteTimer");
+    gtk_widget_destroy(GTK_WIDGET(label));
+    label = gtk_builder_get_object(ui_builder, "lblNoteTextTimer");
+    gtk_widget_destroy(GTK_WIDGET(label));
   }
 
   /* We don't need it anymore */
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 
   /* Fill-in values */
   if (timer->name) {
