@@ -1,7 +1,7 @@
-/* $Id: we_triggers.c 1016 2009-02-02 12:10:40Z ekalin $ */
+/* $Id: we_triggers.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -37,86 +36,99 @@
  * Function prototypes *
  ***********************/
 static GtkTreeModel *create_we_triggers_model(World *world);
-static void          configure_view(GtkTreeView *view, GtkTreeModel *model);
+static void          we_trigger_configure_view(GtkTreeView *view,
+                                               GtkTreeModel *model);
 static void          we_trigger_add_cb(GtkButton *button, gpointer data);
 static void          we_trigger_edit_cb(GtkButton *button, gpointer data);
 static void          we_trigger_delete_cb(GtkButton *button, gpointer data);
-static void          view_row_dblclick_cb(GtkTreeView        *view,
-                                          GtkTreePath        *path,
-                                          GtkTreeViewColumn  *col,
-                                          gpointer            data);
+static void          we_trigger_view_row_dblclick_cb(GtkTreeView        *view,
+                                                     GtkTreePath        *path,
+                                                     GtkTreeViewColumn  *col,
+                                                     gpointer            data);
 static gboolean      edit_trigger(World *world, Trigger *trigger);
 static gboolean      we_trigger_view_keypress_cb(GtkWidget   *widget,
                                                  GdkEventKey *evt,
                                                  gpointer     data);
 static void          we_trigger_move_cb(GtkButton *button, gpointer data);
-/* Glade signals */
-G_MODULE_EXPORT void we_trigger_chkhighlight_toggled_cb(GtkWidget *widget,
-                                                        gpointer   data);
+/* XML UI signals */
+G_MODULE_EXPORT void we_trigger_chkhighlight_toggled_cb(gpointer   data,
+                                                        GtkWidget *widget);
 G_MODULE_EXPORT void we_trigger_btnconfigh_cb(GtkWidget *widget,
                                               gpointer   data);
 
 
 
-GtkWidget *
+GObject *
 get_we_triggers_tab(World *world)
 {
-  GladeXML         *gladexml;
-  GtkWidget        *panel;
-  GtkWidget        *vbox;
-  GtkWidget        *lblType;
-  GtkWidget        *btnAdd;
-  GtkWidget        *btnEdit;
-  GtkWidget        *btnDelete;
-  GtkWidget        *btnUp;
-  GtkWidget        *btnDown;
-  GtkWidget        *btnImport;
-  GtkWidget        *btnExport;
+  GtkBuilder       *ui_builder;
+  gchar            *objects[] = { "we_panel_generic_graphical_editor", NULL };
+  GError           *error = NULL;
+  GObject          *panel;
+  GtkLabel         *lblType;
+  GObject          *btnAdd;
+  GObject          *btnEdit;
+  GObject          *btnDelete;
+  GObject          *btnUp;
+  GObject          *btnDown;
+  GObject          *btnExport;
+  GtkBox           *vbox;
   GtkWidget        *btnTest;
   GtkTreeSelection *selection;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "we_panel_generic_graphical_editor", NULL);
-  panel = glade_xml_get_widget(gladexml, "we_panel_generic_graphical_editor");
-  world->viewTrigger = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewObjects"));
+  /* Create the dialog */
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
+
+  panel = gtk_builder_get_object(ui_builder,
+                                 "we_panel_generic_graphical_editor");
+  /* So it's not destroyed when the builder is destroyed */
+  g_object_ref(panel);
+
+  world->viewTrigger = GTK_TREE_VIEW(gtk_builder_get_object(ui_builder,
+                                                            "viewObjects"));
   g_object_set_data(G_OBJECT(world->viewTrigger), "forworld", world);
   g_signal_connect(G_OBJECT(world->viewTrigger), "row-activated",
-                   G_CALLBACK(view_row_dblclick_cb), world);
+                   G_CALLBACK(we_trigger_view_row_dblclick_cb), world);
   g_signal_connect(G_OBJECT(world->viewTrigger), "key-press-event",
                    G_CALLBACK(we_trigger_view_keypress_cb), world);
 
-  lblType = glade_xml_get_widget(gladexml, "lblGEType");
-  gtk_label_set_markup(GTK_LABEL(lblType),
-                       "<span weight=\"bold\">Triggers</span>");
+  lblType = GTK_LABEL(gtk_builder_get_object(ui_builder, "lblGEType"));
+  gtk_label_set_markup(lblType, "<span weight=\"bold\">Triggers</span>");
 
-  btnAdd = glade_xml_get_widget(gladexml, "btnGEAdd");
-  g_signal_connect(G_OBJECT(btnAdd), "clicked",
+  btnAdd = gtk_builder_get_object(ui_builder, "btnGEAdd");
+  g_signal_connect(btnAdd, "clicked",
                    G_CALLBACK(we_trigger_add_cb), world->viewTrigger);
-  btnEdit = glade_xml_get_widget(gladexml, "btnGEEdit");
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
+  btnEdit = gtk_builder_get_object(ui_builder, "btnGEEdit");
+  g_signal_connect(btnEdit, "clicked",
                    G_CALLBACK(we_trigger_edit_cb), world->viewTrigger);
-  btnDelete = glade_xml_get_widget(gladexml, "btnGEDelete");
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
+  btnDelete = gtk_builder_get_object(ui_builder, "btnGEDelete");
+  g_signal_connect(btnDelete, "clicked",
                    G_CALLBACK(we_trigger_delete_cb), world->viewTrigger);
 
-  btnUp = glade_xml_get_widget(gladexml, "btnGEUp");
-  g_object_set_data(G_OBJECT(btnUp), "isup", GINT_TO_POINTER(TRUE));
-  g_signal_connect(G_OBJECT(btnUp), "clicked",
+  btnUp = gtk_builder_get_object(ui_builder, "btnGEUp");
+  g_object_set_data(btnUp, "isup", GINT_TO_POINTER(TRUE));
+  g_signal_connect(btnUp, "clicked",
                    G_CALLBACK(we_trigger_move_cb), world->viewTrigger);
-  btnDown = glade_xml_get_widget(gladexml, "btnGEDown");
-  g_object_set_data(G_OBJECT(btnDown), "isup", GINT_TO_POINTER(FALSE));
-  g_signal_connect(G_OBJECT(btnDown), "clicked",
+  btnDown = gtk_builder_get_object(ui_builder, "btnGEDown");
+  g_object_set_data(btnDown, "isup", GINT_TO_POINTER(FALSE));
+  g_signal_connect(btnDown, "clicked",
                    G_CALLBACK(we_trigger_move_cb), world->viewTrigger);
-  btnImport = glade_xml_get_widget(gladexml, "btnGEImport");
-  g_signal_connect(G_OBJECT(btnImport), "clicked",
-                   G_CALLBACK(we_guied_import_cb), world);
-  btnExport = glade_xml_get_widget(gladexml, "btnGEExport");
-  g_object_set_data(G_OBJECT(btnExport), "preselect", "Trigger");
-  g_signal_connect(G_OBJECT(btnExport), "clicked",
-                   G_CALLBACK(we_guied_export_cb), world);
+
+  btnExport = gtk_builder_get_object(ui_builder, "btnGEExport");
+  g_object_set_data(btnExport, "preselect", "Trigger");
 
   /* Add an extra button for "Test Triggers" */
-  vbox = glade_xml_get_widget(gladexml, "vbox_int_generic_graphical_editor");
+  vbox = GTK_BOX(gtk_builder_get_object(ui_builder,
+                                        "vbox_int_generic_graphical_editor"));
   btnTest = gtk_button_new_with_label(_("Test Triggers"));
   g_signal_connect_swapped(G_OBJECT(btnTest), "clicked",
                            G_CALLBACK(open_test_triggers_dialog), world);
@@ -124,7 +136,7 @@ get_we_triggers_tab(World *world)
   if (!world->connected) {
     gtk_widget_set_sensitive(btnTest, FALSE);
   }
-  gtk_box_pack_start(GTK_BOX(vbox), btnTest, FALSE, FALSE, 0);
+  gtk_box_pack_start(vbox, btnTest, FALSE, FALSE, 0);
 
   world->trigger_model = create_we_triggers_model(world);
   world->trigger_model_filter = GTK_TREE_MODEL_FILTER(gtk_tree_model_filter_new(world->trigger_model, NULL));
@@ -134,8 +146,8 @@ get_we_triggers_tab(World *world)
                                          world,
                                          NULL);
 
-  configure_view(world->viewTrigger,
-                 GTK_TREE_MODEL(world->trigger_model_filter));
+  we_trigger_configure_view(world->viewTrigger,
+                            GTK_TREE_MODEL(world->trigger_model_filter));
   selection = gtk_tree_view_get_selection(world->viewTrigger);
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
   g_object_set_data(G_OBJECT(selection), "btnEdit",   btnEdit);
@@ -146,8 +158,8 @@ get_we_triggers_tab(World *world)
   g_signal_connect(G_OBJECT(selection), "changed",
                    G_CALLBACK(we_guied_selection_changed_cb), NULL);
 
-
-  g_object_unref(gladexml);
+  gtk_builder_connect_signals(ui_builder, world);
+  g_object_unref(ui_builder);
   return panel;
 }
 
@@ -181,7 +193,7 @@ create_we_triggers_model(World *world)
 
 static
 void
-configure_view(GtkTreeView *view, GtkTreeModel *model)
+we_trigger_configure_view(GtkTreeView *view, GtkTreeModel *model)
 {
   GtkCellRenderer   *renderer;
   GtkTreeViewColumn *column;
@@ -483,10 +495,10 @@ we_trigger_delete_cb(GtkButton *button, gpointer data)
 
 static
 void
-view_row_dblclick_cb(GtkTreeView        *view,
-                     GtkTreePath        *path,
-                     GtkTreeViewColumn  *col,
-                     gpointer            data)
+we_trigger_view_row_dblclick_cb(GtkTreeView        *view,
+                                GtkTreePath        *path,
+                                GtkTreeViewColumn  *col,
+                                gpointer            data)
 {
   World        *world;
   GtkTreeModel *model;
@@ -511,7 +523,9 @@ static
 gboolean
 edit_trigger(World *world, Trigger *trigger)
 {
-  GladeXML        *gladexml;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "dlgEditTrigger", NULL };
+  GError          *error = NULL;
   GtkWidget       *dlgEditTrigger;
   GtkEntry        *txtName;
   GtkEntry        *txtPattern;
@@ -522,7 +536,7 @@ edit_trigger(World *world, Trigger *trigger)
   GtkToggleButton *chkKeepExec;
   GtkToggleButton *chkRewriter;
   GtkToggleButton *chkHighlight;
-  GtkWidget       *btnConfigH;
+  GObject         *btnConfigH;
   GtkToggleButton *chkEnabled;
   const gchar     *newname;
   const gchar     *newpattern;
@@ -533,46 +547,59 @@ edit_trigger(World *world, Trigger *trigger)
   gboolean         newicase;
   gboolean         need_recompute = FALSE;
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditTrigger", NULL);
-  glade_xml_signal_autoconnect(gladexml);
-  dlgEditTrigger    = glade_xml_get_widget(gladexml, "dlgEditTrigger");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return FALSE;
+  }
+  gtk_builder_connect_signals(ui_builder, NULL);
+
+  dlgEditTrigger = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                     "dlgEditTrigger"));
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditTrigger),
                                   GTK_RESPONSE_OK);
-  txtName      = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                   "txtTriggerName");
-  txtPattern   = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                   "txtTriggerPattern");
-  txtAction    = (GtkEntry *) glade_xml_get_widget(gladexml,
-                                                   "txtTriggerAction");
-  chkICase     = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerICase");
-  chkGag       = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerGag");
-  chkGagLog    = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerGagLog");
-  chkKeepExec  = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerKeepExec");
-  chkRewriter  = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerRewriter");
-  chkHighlight = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerHighlight");
-  btnConfigH   = glade_xml_get_widget(gladexml, "btnTriggerConfigHighlight");
-  chkEnabled   = (GtkToggleButton *) glade_xml_get_widget(gladexml,
-                                                          "chkTriggerEnabled");
 
-  g_object_set_data(G_OBJECT(btnConfigH), "trigger", trigger);
+  txtName      = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtTriggerName"));
+  txtPattern   = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtTriggerPattern"));
+  txtAction    = GTK_ENTRY(gtk_builder_get_object(ui_builder,
+                                                  "txtTriggerAction"));
+  chkICase     = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerICase"));
+  chkGag       = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerGag"));
+  chkGagLog    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerGagLog"));
+  chkKeepExec  = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerKeepExec"));
+  chkRewriter  = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerRewriter"));
+  chkHighlight = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerHighlight"));
+  btnConfigH   = gtk_builder_get_object(ui_builder, "btnTriggerConfigHighlight");
+  chkEnabled   = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                          "chkTriggerEnabled"));
+
+  g_object_set_data(btnConfigH, "trigger", trigger);
 
   /* Only display note when editing a plugin's trigger */
   if (!trigger->owner_plugin) {
-    GtkWidget *label;
+    GObject *label;
 
-    label = glade_xml_get_widget(gladexml, "lblNotePlugin");
-    gtk_widget_destroy(label);
-    label = glade_xml_get_widget(gladexml, "lblNoteTextPlugin");
-    gtk_widget_destroy(label);
+    label = gtk_builder_get_object(ui_builder, "lblNotePlugin");
+    gtk_widget_destroy(GTK_WIDGET(label));
+    label = gtk_builder_get_object(ui_builder, "lblNoteTextPlugin");
+    gtk_widget_destroy(GTK_WIDGET(label));
   }
 
+  /* We don't need it anymore */
+  g_object_unref(ui_builder);
+
   /* Fill-in values */
   if (trigger->name) {
     gtk_entry_set_text(txtName, trigger->name);
@@ -677,12 +704,10 @@ edit_trigger(World *world, Trigger *trigger)
 
       /* We've finished successfully */
       gtk_widget_destroy(dlgEditTrigger);
-      g_object_unref(gladexml);
       return TRUE;
     } else {
       /* Cancel pressed */
       gtk_widget_destroy(dlgEditTrigger);
-      g_object_unref(gladexml);
       return FALSE;
     }
   }
@@ -690,13 +715,10 @@ edit_trigger(World *world, Trigger *trigger)
 
 
 void
-we_trigger_chkhighlight_toggled_cb(GtkWidget *widget, gpointer data)
+we_trigger_chkhighlight_toggled_cb(gpointer data, GtkWidget *widget)
 {
-  GladeXML  *gladexml;
-  GtkWidget *btnConfigH;
+  GtkWidget *btnConfigH = GTK_WIDGET(data);
 
-  gladexml = glade_get_widget_tree(widget);
-  btnConfigH = glade_xml_get_widget(gladexml, "btnTriggerConfigHighlight");
   gtk_widget_set_sensitive(btnConfigH, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
 }
 
@@ -705,9 +727,12 @@ void
 we_trigger_btnconfigh_cb(GtkWidget *widget, gpointer data)
 {
   Trigger         *trigger;
-  GladeXML        *gladexml;
+  GtkBuilder      *ui_builder;
+  gchar           *objects[] = { "dlgEditTriggerHighlight",
+                                 "model1", "model2", "model3", "model4",
+                                 "model5", "adjustment3", NULL };
+  GError          *error = NULL;
   GtkWidget       *dlgEditHighlight;
-  GtkWidget       *table;
   GtkToggleButton *radTargetLine;
   GtkToggleButton *radTargetMatch;
   GtkToggleButton *radTargetSubstring;
@@ -720,36 +745,35 @@ we_trigger_btnconfigh_cb(GtkWidget *widget, gpointer data)
 
   trigger = g_object_get_data(G_OBJECT(widget), "trigger");
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditTriggerHighlight", NULL);
-  dlgEditHighlight = glade_xml_get_widget(gladexml, "dlgEditTriggerHighlight");
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_objects_from_file(ui_builder,
+                                         get_kildclient_installed_file("kildclient.ui"),
+                                         objects,
+                                         &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return;
+  }
+  dlgEditHighlight
+    = GTK_WIDGET(gtk_builder_get_object(ui_builder, "dlgEditTriggerHighlight"));
 
-  table = glade_xml_get_widget(gladexml, "tblTriggerHighlight");
-  cmbFgColor = GTK_COMBO_BOX(create_combo_with_colors());
-  gtk_combo_box_prepend_text(cmbFgColor, _("Do not change"));
-  gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(cmbFgColor),
-                   1, 2, 1, 2,
-                   GTK_FILL, GTK_FILL, 0, 0);
-  cmbBgColor = GTK_COMBO_BOX(create_combo_with_colors());
-  gtk_combo_box_prepend_text(cmbBgColor, _("Do not change"));
-  gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(cmbBgColor),
-                   1, 2, 2, 3,
-                   GTK_FILL, GTK_FILL, 0, 0);
+  cmbFgColor = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbFgColor"));
+  cmbBgColor = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbBgColor"));
 
-  radTargetLine      = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
-                                                  "radTargetLine"));
-  radTargetMatch     = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
-                                                  "radTargetMatch"));
-  radTargetSubstring = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml,
-                                                  "radTargetSubstring"));
-  spnTarget = GTK_SPIN_BUTTON(glade_xml_get_widget(gladexml, "spnTarget"));
-
-  cmbItalic    = GTK_COMBO_BOX(glade_xml_get_widget(gladexml,
-                                                    "cmbHighlightItalic"));
-  cmbStrike    = GTK_COMBO_BOX(glade_xml_get_widget(gladexml,
-                                                    "cmbHighlightStrike"));
-  cmbUnderline = GTK_COMBO_BOX(glade_xml_get_widget(gladexml,
-                                                    "cmbHighlightUnderline"));
+  radTargetLine
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "radTargetLine"));
+  radTargetMatch
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "radTargetMatch"));
+  radTargetSubstring
+    = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder, "radTargetSubstring"));
+  spnTarget
+    = GTK_SPIN_BUTTON(gtk_builder_get_object(ui_builder, "spnTarget"));
+  cmbItalic
+    = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbHighlightItalic"));
+  cmbStrike
+    = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbHighlightStrike"));
+  cmbUnderline
+    = GTK_COMBO_BOX(gtk_builder_get_object(ui_builder, "cmbHighlightUnderline"));
 
   /* Fill in values. */
   if (trigger->high_target == -1) {
@@ -761,10 +785,10 @@ we_trigger_btnconfigh_cb(GtkWidget *widget, gpointer data)
     gtk_spin_button_set_value(spnTarget, trigger->high_target);
   }
   /* +1 is because -1 corresponds to "No change" */
-  gtk_combo_box_set_active(cmbFgColor,   trigger->high_fg_color + 1);
-  gtk_combo_box_set_active(cmbBgColor,   trigger->high_bg_color + 1);
-  gtk_combo_box_set_active(cmbItalic,    trigger->high_italic + 1);
-  gtk_combo_box_set_active(cmbStrike,    trigger->high_strike + 1);
+  gtk_combo_box_set_active(cmbFgColor,   trigger->high_fg_color  + 1);
+  gtk_combo_box_set_active(cmbBgColor,   trigger->high_bg_color  + 1);
+  gtk_combo_box_set_active(cmbItalic,    trigger->high_italic    + 1);
+  gtk_combo_box_set_active(cmbStrike,    trigger->high_strike    + 1);
   gtk_combo_box_set_active(cmbUnderline, trigger->high_underline + 1);
 
   /* Run the dialog */
@@ -787,7 +811,7 @@ we_trigger_btnconfigh_cb(GtkWidget *widget, gpointer data)
   }
 
   gtk_widget_destroy(dlgEditHighlight);
-  g_object_unref(gladexml);
+  g_object_unref(ui_builder);
 }
 
 
