@@ -1,7 +1,7 @@
-/* $Id: worldeditor.c 1028 2009-02-14 20:59:30Z ekalin $ */
+/* $Id: worldeditor.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 #include <sys/stat.h>
 #include <libintl.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -47,19 +46,19 @@ enum {
 /***********************
  * Function prototypes *
  ***********************/
-static void          world_editor_response_cb(GtkWidget *dialog,
-                                              gint       response,
-                                              gpointer   data);
 static GtkWidget    *create_edit_world_dialog(World *world,
                                               GtkWindow  *parent);
 static GtkTreeStore *create_model(GtkNotebook *notebook,
                                   World       *world,
-                                  GtkWidget   *parent,
-                                  GladeXML    *gladexml);
+                                  GtkBuilder  *ui_builder);
 static void          list_selection_changed_cb(GtkTreeSelection *selection,
                                                gpointer data);
 static GtkWidget    *new_descriptive_panel(const char *title,
                                            const char *message);
+/* XML UI signals */
+G_MODULE_EXPORT void world_editor_response_cb(GtkWidget *dialog,
+                                              gint       response,
+                                              gpointer   data);
 
 
 /*************************
@@ -90,13 +89,14 @@ edit_world(World     **world,
 
   gtk_window_present(GTK_WINDOW((*world)->dlgEditWorld));
 
-  gtk_signal_connect(GTK_OBJECT((*world)->dlgEditWorld), "response",
-                     G_CALLBACK(world_editor_response_cb), *world);
-
   if (modal) {
     response = gtk_dialog_run(GTK_DIALOG((*world)->dlgEditWorld));
+
     gtk_widget_destroy((*world)->dlgEditWorld);
     (*world)->dlgEditWorld = NULL;
+    g_object_unref((*world)->ui_builder);
+    (*world)->ui_builder = NULL;
+
     return (response == GTK_RESPONSE_OK);
   }
 
@@ -104,27 +104,28 @@ edit_world(World     **world,
 }
 
 
-static
 void
 world_editor_response_cb(GtkWidget *dialog, gint response, gpointer data)
 {
-  World    *world = (World *) data;
-  gboolean *newworld;
+  World      *world = (World *) data;
+  GtkBuilder *ui_builder;
+  gboolean   *newworld;
+
+  ui_builder = world->ui_builder;
 
   if (response == GTK_RESPONSE_OK) {
     if (world != default_world) {
-      update_we_general_parameters(world);
+      update_we_general_parameters(world, ui_builder);
     }
-    /* Fonts and colors are updated automatically */
-    update_we_input_parameters(world);
-    update_we_logging_parameters(world);
+    update_we_input_parameters(world, ui_builder);
+    update_we_logging_parameters(world, ui_builder);
     if (world != default_world) {
-      update_we_scripting_parameters(world);
+      update_we_scripting_parameters(world, ui_builder);
     }
-    update_we_protocols_parameters(world);
-    update_we_misc_parameters(world);
+    update_we_protocols_parameters(world, ui_builder);
+    update_we_misc_parameters(world, ui_builder);
     newworld = g_object_get_data(G_OBJECT(dialog), "newworldptr");
-    update_we_advanced_parameters(world, newworld);
+    update_we_advanced_parameters(world, newworld, ui_builder);
 
     /* In case the name or display name was changed, we must update the tab */
     if (world->gui) {
@@ -137,6 +138,19 @@ world_editor_response_cb(GtkWidget *dialog, gint response, gpointer data)
       /* The window title gets update automatically because when the world
          editor is closed, the main window gets focus. */
     }
+  } else {
+    /* Revert parameters that might have been changed */
+    if (world != default_world) {
+      fill_we_general_tab(world, ui_builder);
+    }
+    fill_we_input_tab(world, ui_builder);
+    fill_we_logging_tab(world, ui_builder);
+    if (world != default_world) {
+      fill_we_scripting_tab(world, ui_builder);
+    }
+    fill_we_protocols_tab(world, ui_builder);
+    fill_we_misc_tab(world, ui_builder);
+    fill_we_advanced_tab(world, ui_builder);
   }
 
   gtk_widget_hide(dialog);
@@ -147,10 +161,10 @@ static
 GtkWidget *
 create_edit_world_dialog(World *world, GtkWindow *parent)
 {
-  GladeXML  *gladexml;
-  GtkWidget *dlgEditWorld;
-  GtkWidget *treeView;
-  GtkWidget *notebook;
+  GError      *error = NULL;
+  GtkWidget   *dlgEditWorld;
+  GtkTreeView *treeView;
+  GtkNotebook *notebook;
 
   GtkTreeStore      *model;
   GtkCellRenderer   *renderer;
@@ -158,25 +172,33 @@ create_edit_world_dialog(World *world, GtkWindow *parent)
   GtkTreeSelection  *selection;
   GtkTreeIter        iter;
 
+  /* Create the dialog */
+  if (!world->ui_builder) {
+    world->ui_builder = gtk_builder_new();
+  }
+  if (!gtk_builder_add_from_file(world->ui_builder,
+                                 get_kildclient_installed_file("dlgEditWorld.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    return NULL; /* This will cause warnings in the calling function, but
+                    there is need to abort the program because of this. */
+  }
 
-  gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                           "dlgEditWorld", NULL);
-  glade_xml_signal_autoconnect(gladexml);
+  dlgEditWorld = GTK_WIDGET(gtk_builder_get_object(world->ui_builder,
+                                                   "dlgEditWorld"));
 
-  dlgEditWorld = glade_xml_get_widget(gladexml, "dlgEditWorld");
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditWorld), GTK_RESPONSE_OK);
-  g_object_set_data(G_OBJECT(gladexml), "dlgEditWorld", dlgEditWorld);
 
-  notebook = glade_xml_get_widget(gladexml, "ntbkWEPanel");
-  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
+  notebook = GTK_NOTEBOOK(gtk_builder_get_object(world->ui_builder,
+                                                 "ntbkWEPanel"));
+  gtk_notebook_set_show_tabs(notebook, FALSE);
 
-  model = create_model(GTK_NOTEBOOK(notebook),
-                       world,
-                       dlgEditWorld,
-                       gladexml);
+  model = create_model(notebook, world, world->ui_builder);
   g_object_set_data(G_OBJECT(model), "notebookptr", notebook);
-  treeView = glade_xml_get_widget(gladexml, "treeViewWE");
-  gtk_tree_view_set_model(GTK_TREE_VIEW(treeView), GTK_TREE_MODEL(model));
+  treeView = GTK_TREE_VIEW(gtk_builder_get_object(world->ui_builder,
+                                                  "treeViewWE"));
+  gtk_tree_view_set_model(treeView, GTK_TREE_MODEL(model));
   g_object_unref(G_OBJECT(model));
   renderer = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes(NULL,
@@ -184,16 +206,18 @@ create_edit_world_dialog(World *world, GtkWindow *parent)
                                                     "text",
                                                     COLUMN_NAME,
                                                     NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(treeView), column);
-  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeView), FALSE);
+  gtk_tree_view_append_column(treeView, column);
+  gtk_tree_view_set_headers_visible(treeView, FALSE);
 
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeView));
+  selection = gtk_tree_view_get_selection(treeView);
   g_signal_connect(G_OBJECT(selection), "changed",
                    G_CALLBACK(list_selection_changed_cb), treeView);
 
   gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
   gtk_tree_selection_select_iter(selection, &iter);
 
+  gtk_builder_connect_signals(world->ui_builder, world);
+
   return dlgEditWorld;
 }
 
@@ -202,11 +226,11 @@ static
 GtkTreeStore *
 create_model(GtkNotebook *notebook,
              World       *world,
-             GtkWidget   *parent,
-             GladeXML    *gladexml)
+             GtkBuilder  *ui_builder)
 {
   GtkTreeStore *model;
-  GtkWidget    *panel;
+  GObject      *panel;
+  GtkWidget    *display_panel;
   GtkTreeIter   iter;
   GtkTreeIter   iterDisplay;
   GtkTreeIter   iterAutomation;
@@ -215,37 +239,37 @@ create_model(GtkNotebook *notebook,
                              G_TYPE_STRING,
                              G_TYPE_POINTER);
 
-  panel = get_we_general_tab(gladexml);
+  panel = get_we_general_tab(world, ui_builder);
   if (world != default_world) {
     gtk_tree_store_append(model, &iter, NULL);
-    fill_we_general_tab(world, gladexml);
+    fill_we_general_tab(world, ui_builder);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("General"),
                        COLUMN_WIDGET, panel,
                        -1);
   } else {
-    gtk_widget_destroy(panel);
+    gtk_widget_destroy(GTK_WIDGET(panel));
   }
 
   gtk_tree_store_append(model, &iterDisplay, NULL);
-  panel = new_descriptive_panel(_("Display"),
-                                _("Here you can set options to configure the appearance of KildClient."));
-  gtk_notebook_append_page(notebook, panel, NULL);
+  display_panel = new_descriptive_panel(_("Display"),
+                                        _("Here you can set options to configure the appearance of KildClient."));
+  gtk_notebook_append_page(notebook, display_panel, NULL);
   gtk_tree_store_set(model, &iterDisplay,
                      COLUMN_NAME, _("Display"),
-                     COLUMN_WIDGET, panel,
+                     COLUMN_WIDGET, display_panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, &iterDisplay);
-  panel = get_we_mainwindow_tab(gladexml);
-  fill_we_mainwindow_tab(world, gladexml);
+  panel = get_we_mainwindow_tab(ui_builder);
+  fill_we_mainwindow_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Main Window"),
                      COLUMN_WIDGET, panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, &iterDisplay);
-  panel = get_we_colors_tab(world, gladexml);
+  panel = get_we_colors_tab(world, ui_builder);
   /* Colors is filled in automatically */
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Colors"),
@@ -253,24 +277,24 @@ create_model(GtkNotebook *notebook,
                      -1);
 
   gtk_tree_store_append(model, &iter, &iterDisplay);
-  panel = get_we_statusbar_tab(gladexml);
-  fill_we_statusbar_tab(world, gladexml);
+  panel = get_we_statusbar_tab(ui_builder);
+  fill_we_statusbar_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Status Bar"),
                      COLUMN_WIDGET, panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, NULL);
-  panel = get_we_input_tab(gladexml);
-  fill_we_input_tab(world, gladexml);
+  panel = get_we_input_tab(ui_builder);
+  fill_we_input_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Input"),
                      COLUMN_WIDGET, panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, NULL);
-  panel = get_we_logging_tab(gladexml, world);
-  fill_we_logging_tab(world, gladexml);
+  panel = get_we_logging_tab(ui_builder, world);
+  fill_we_logging_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Logging"),
                      COLUMN_WIDGET, panel,
@@ -278,25 +302,25 @@ create_model(GtkNotebook *notebook,
 
   if (world != default_world) {
     gtk_tree_store_append(model, &iterAutomation, NULL);
-    panel = new_descriptive_panel(_("Automation"),
-                                  _("Here you can set options to configure the automation features of KildClient, such as triggers, aliases, plugins, etc."));
-    gtk_notebook_append_page(notebook, panel, NULL);
+    display_panel = new_descriptive_panel(_("Automation"),
+                                          _("Here you can set options to configure the automation features of KildClient, such as triggers, aliases, plugins, etc."));
+    gtk_notebook_append_page(notebook, display_panel, NULL);
     gtk_tree_store_set(model, &iterAutomation,
                        COLUMN_NAME, _("Automation"),
-                       COLUMN_WIDGET, panel,
+                       COLUMN_WIDGET, display_panel,
                        -1);
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
-    panel = get_we_scripting_tab(parent, gladexml);
-    fill_we_scripting_tab(world, gladexml);
+    panel = get_we_scripting_tab(ui_builder);
+    fill_we_scripting_tab(world, ui_builder);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Scripting"),
                        COLUMN_WIDGET, panel,
                        -1);
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
-    panel = get_we_plugins_tab(gladexml, world);
-    fill_we_scripting_tab(world, gladexml);
+    panel = get_we_plugins_tab(ui_builder, world);
+    fill_we_scripting_tab(world, ui_builder);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Plugins"),
                        COLUMN_WIDGET, panel,
@@ -304,7 +328,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_triggers_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Triggers"),
                        COLUMN_WIDGET, panel,
@@ -312,7 +339,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_aliases_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Aliases"),
                        COLUMN_WIDGET, panel,
@@ -320,7 +350,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_macros_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Macros"),
                        COLUMN_WIDGET, panel,
@@ -328,7 +361,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_timers_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Timers"),
                        COLUMN_WIDGET, panel,
@@ -336,7 +372,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_hooks_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Hooks"),
                        COLUMN_WIDGET, panel,
@@ -344,7 +383,10 @@ create_model(GtkNotebook *notebook,
 
     gtk_tree_store_append(model, &iter, &iterAutomation);
     panel = get_we_vars_tab(world);
-    gtk_notebook_append_page(notebook, panel, NULL);
+    gtk_notebook_append_page(notebook, GTK_WIDGET(panel), NULL);
+    /* Let the notebook own the panel, removing the ref added when it
+       was created. */
+    g_object_unref(panel);
     gtk_tree_store_set(model, &iter,
                        COLUMN_NAME, _("Variables"),
                        COLUMN_WIDGET, panel,
@@ -352,24 +394,24 @@ create_model(GtkNotebook *notebook,
   }
 
   gtk_tree_store_append(model, &iter, NULL);
-  panel = get_we_protocols_tab(gladexml);
-  fill_we_protocols_tab(world, gladexml);
+  panel = get_we_protocols_tab(ui_builder);
+  fill_we_protocols_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Protocols"),
                      COLUMN_WIDGET, panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, NULL);
-  panel = get_we_misc_tab(gladexml);
-  fill_we_misc_tab(world, gladexml);
+  panel = get_we_misc_tab(ui_builder);
+  fill_we_misc_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Miscellaneous"),
                      COLUMN_WIDGET, panel,
                      -1);
 
   gtk_tree_store_append(model, &iter, NULL);
-  panel = get_we_advanced_tab(world, gladexml);
-  fill_we_advanced_tab(world, gladexml);
+  panel = get_we_advanced_tab(world, ui_builder);
+  fill_we_advanced_tab(world, ui_builder);
   gtk_tree_store_set(model, &iter,
                      COLUMN_NAME, _("Advanced"),
                      COLUMN_WIDGET, panel,
