@@ -1,7 +1,7 @@
-/* $Id: worldgui.c 1025 2009-02-11 16:56:47Z ekalin $ */
+/* $Id: worldgui.c 1176 2011-05-28 14:43:45Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@
 #include <libintl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 #ifdef __WIN32__
 #  include <windows.h>
 #  include <shellapi.h>
@@ -49,7 +48,6 @@
 static gboolean   hovering_over_link = FALSE;
 static GdkCursor *hand_cursor = NULL;
 static GdkCursor *regular_cursor = NULL;
-#define TOOLTIP_DELAY 300
 
 
 /***********************
@@ -83,24 +81,26 @@ static gboolean  textview_button_press_cb(GtkWidget      *text_view,
 static gboolean  act_if_link(WorldGUI       *gui,
                              GtkTextIter    *iter,
                              GdkEventButton *event);
-static void      menu_popup_url_copy(GtkWidget *widget, gchar *url);
-static gboolean  textview_motion_notify_event(GtkWidget      *text_view,
+static gboolean  textview_motion_notify_event(GtkWidget      *widget,
                                               GdkEventMotion *event,
                                               gpointer        data);
-static gboolean  textview_visibility_notify_event(GtkWidget         *text_view,
+static gboolean  textview_visibility_notify_event(GtkWidget          *widget,
                                                   GdkEventVisibility *event,
                                                   gpointer            data);
-static gboolean  textview_leave_notify_event(GtkWidget        *text_view,
-                                             GdkEventCrossing *event,
-                                             gpointer          data);
-static void      set_cursor_if_appropriate(WorldGUI *gui,
-                                           gint      x,
-                                           gint      y);
-static gboolean  worldgui_tooltip_timeout(gpointer data);
-static gboolean  worldgui_paint_tooltip_cb(GtkWidget      *widget,
-                                          GdkEventExpose *event,
-                                          gpointer        data);
-static gchar    *worldgui_get_tooltip_text(WorldGUI *gui);
+static void      set_cursor_if_appropriate(GtkTextView *text_view,
+                                           WorldGUI    *gui,
+                                           gint         x,
+                                           gint         y);
+static gboolean  worldgui_query_tooltip_cb(GtkWidget  *widget,
+                                           gint        x,
+                                           gint        y,
+                                           gboolean    keyboard_mode,
+                                           GtkTooltip *tooltip,
+                                           gpointer    data);
+static gchar    *worldgui_get_tooltip_text(WorldGUI    *gui,
+                                           GtkTextView *text_view,
+                                           gint         win_x,
+                                           gint         win_y);
 static void      textview_size_allocate_cb(GtkWidget     *widget,
                                            GtkAllocation *allocation,
                                            gpointer       data);
@@ -110,6 +110,14 @@ static void      close_search_box_cb(GtkButton *widget, gpointer data);
 static gboolean  search_box_keypress_cb(GtkWidget   *widget,
                                         GdkEventKey *evt,
                                         gpointer     data);
+static void      scrolled_win_size_allocate_cb(GtkWidget     *widget,
+                                               GtkAllocation *allocation,
+                                               gpointer       data);
+static void      scrolled_win_value_changed_cb(GtkAdjustment *adjustment,
+                                               gpointer       data);
+static gboolean  set_split_to_zero(gpointer data);
+/* XML UI Signals */
+G_MODULE_EXPORT void menu_popup_url_copy(GtkAction *action, gchar *url);
 
 
 
@@ -121,16 +129,14 @@ world_gui_new(gboolean is_for_mud)
      will be included in the notebook.
      If FALSE, it is a KCWin gui, and it has less features.
   */
-  WorldGUI    *gui;
-  GtkWidget   *iconClearWidget;
-  GtkWidget   *frame;
-  GtkTooltips *tooltips;
+  WorldGUI      *gui;
+  GtkWidget     *iconClearWidget;
+  GtkWidget     *frame;
+  GtkAdjustment *verticalAdjust;
 
   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
 
-  GList *combochildren;
-
   PangoFontDescription *fontDesc;
   GtkTextIter           end_iter;
   GdkColor              black = { 0, 0, 0, 0 };
@@ -151,6 +157,7 @@ world_gui_new(gboolean is_for_mud)
   gtk_scrolled_window_set_policy(gui->scrolled_win,
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_ALWAYS);
+
   gui->txtView = GTK_TEXT_VIEW(gtk_text_view_new());
   gtk_text_view_set_editable(gui->txtView, FALSE);
   gui->txtBuffer = gtk_text_view_get_buffer(gui->txtView);
@@ -160,14 +167,46 @@ world_gui_new(gboolean is_for_mud)
                                                  "end",
                                                  &end_iter,
                                                  FALSE);
+  gtk_container_add(GTK_CONTAINER(gui->scrolled_win), GTK_WIDGET(gui->txtView));
 
-  gtk_container_add(GTK_CONTAINER(gui->scrolled_win),
-                    GTK_WIDGET(gui->txtView));
-  gtk_box_pack_start(GTK_BOX(gui->vbox), GTK_WIDGET(gui->scrolled_win),
-                     TRUE, TRUE, 0);
-
-  /* Search box */
   if (is_for_mud) {
+    g_signal_connect(G_OBJECT(gui->scrolled_win), "size-allocate",
+                     G_CALLBACK(scrolled_win_size_allocate_cb), gui);
+    verticalAdjust = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
+    g_signal_connect(G_OBJECT(verticalAdjust), "value-changed",
+                     G_CALLBACK(scrolled_win_value_changed_cb), gui);
+
+    /* Second TextView for split screen */
+    gui->scrwinScroll = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
+    gtk_scrolled_window_set_policy(gui->scrwinScroll,
+                                   GTK_POLICY_AUTOMATIC,
+                                   GTK_POLICY_ALWAYS);
+    gui->txtViewScroll
+      = GTK_TEXT_VIEW(gtk_text_view_new_with_buffer(gui->txtBuffer));
+    gtk_text_view_set_editable(gui->txtViewScroll, FALSE);
+    gtk_container_add(GTK_CONTAINER(gui->scrwinScroll),
+                      GTK_WIDGET(gui->txtViewScroll));
+
+    /* Create split pane */
+    gui->split_pane = GTK_PANED(gtk_vpaned_new());
+    gtk_paned_pack1(gui->split_pane, GTK_WIDGET(gui->scrwinScroll), FALSE, TRUE);
+    gtk_paned_pack2(gui->split_pane, GTK_WIDGET(gui->scrolled_win), TRUE, TRUE);
+    /* For some reason, we cannot set to 0 directly. */
+    g_idle_add(set_split_to_zero, gui);
+
+    gtk_box_pack_start(GTK_BOX(gui->vbox), GTK_WIDGET(gui->split_pane),
+                       TRUE, TRUE, 0);
+  } else {
+    /* No split screen. However, we create the text view so that the
+       functions that configure it do not fail. */
+    gui->txtViewScroll
+      = GTK_TEXT_VIEW(gtk_text_view_new_with_buffer(gui->txtBuffer));
+    gtk_box_pack_start(GTK_BOX(gui->vbox), GTK_WIDGET(gui->scrolled_win),
+                       TRUE, TRUE, 0);
+  }
+
+  if (is_for_mud) {
+    /* Search box */
     GtkWidget *iconCloseWidget;
     GtkWidget *btnClose;
     GtkWidget *label;
@@ -230,13 +269,8 @@ world_gui_new(gboolean is_for_mud)
   gtk_box_pack_start(GTK_BOX(gui->commandArea), gui->btnClear,
                      FALSE, FALSE, 0);
 
-  tooltips = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltips, gui->btnClear,
-                       _("Click to clear the command input area."),
-                       _("Click this button to erase the text that is in "
-                         "the command input area (at the right of this "
-                         "button) so that you can start typing again."));
-
+  gtk_widget_set_tooltip_text(gui->btnClear,
+                              _("Click to clear the command input area."));
 
   if (is_for_mud) {
     /* The full-featured SimoComboBox */
@@ -249,7 +283,6 @@ world_gui_new(gboolean is_for_mud)
                      G_CALLBACK(combo_poped_cb), gui);
     g_signal_connect(G_OBJECT(gui->cmbEntry), "popup-closed",
                      G_CALLBACK(combo_closed_cb), gui);
-    combochildren = gtk_container_get_children(GTK_CONTAINER(gui->cmbEntry));
     g_signal_connect(G_OBJECT(gui->cmbEntry), "activate",
                      G_CALLBACK(activate_cb), gui);
     g_signal_connect(G_OBJECT(gui->cmbEntry->txtview), "key-press-event",
@@ -283,7 +316,10 @@ world_gui_new(gboolean is_for_mud)
   /* Keep the focus always in the entry box */
   g_signal_connect_swapped(G_OBJECT(gui->txtView), "focus-in-event",
                            G_CALLBACK(gtk_widget_grab_focus), gui->cmbEntry);
-
+  if (is_for_mud) {
+    g_signal_connect_swapped(G_OBJECT(gui->txtViewScroll), "focus-in-event",
+                             G_CALLBACK(gtk_widget_grab_focus), gui->cmbEntry);
+  }
 
   /* Set initial colors and state */
   gui->ta.state.fg_color = gui->ta.saved_state.fg_color
@@ -346,7 +382,7 @@ world_gui_new(gboolean is_for_mud)
     rcstyle  = gtk_rc_style_new();
     rcstyle->xthickness = rcstyle->ythickness = 0;
     gtk_widget_modify_style(btnClose, rcstyle);
-    gtk_rc_style_unref(rcstyle);
+    g_object_unref(rcstyle);
 
     iconCloseWidget = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
                                                GTK_ICON_SIZE_MENU);
@@ -362,12 +398,14 @@ world_gui_new(gboolean is_for_mud)
        default font so that the size can be calculated correctly. */
     fontDesc = pango_font_description_from_string(DEFAULT_TERMINAL_FONT);
     gtk_widget_modify_font(GTK_WIDGET(gui->txtView), fontDesc);
+    gtk_widget_modify_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
     pango_font_description_free(fontDesc);
     gtk_widget_modify_base(GTK_WIDGET(gui->txtView),
                            GTK_STATE_NORMAL,
                            &black);
-
-    gtk_widget_add_events(GTK_WIDGET(gui->txtView), GDK_LEAVE_NOTIFY_MASK);
+    gtk_widget_modify_base(GTK_WIDGET(gui->txtViewScroll),
+                           GTK_STATE_NORMAL,
+                           &black);
 
     /* Detect resizes */
     g_signal_connect(G_OBJECT(gui->txtView), "size-allocate",
@@ -387,16 +425,22 @@ world_gui_new(gboolean is_for_mud)
     /* URLs support */
     g_signal_connect(gui->txtView, "button-press-event",
                      G_CALLBACK(textview_button_press_cb), gui);
+    g_signal_connect(gui->txtViewScroll, "button-press-event",
+                     G_CALLBACK(textview_button_press_cb), gui);
     g_signal_connect(gui->txtView, "motion-notify-event",
                      G_CALLBACK(textview_motion_notify_event), gui);
+    g_signal_connect(gui->txtViewScroll, "motion-notify-event",
+                     G_CALLBACK(textview_motion_notify_event), gui);
     g_signal_connect(gui->txtView, "visibility-notify-event",
                      G_CALLBACK(textview_visibility_notify_event), gui);
+    g_signal_connect(gui->txtViewScroll, "visibility-notify-event",
+                     G_CALLBACK(textview_visibility_notify_event), gui);
 
-    /* Tool tips */
-    /* We do not connect motion-notify because we already have a callback
-       used in the URL support stuff. */
-    g_signal_connect(gui->txtView, "leave-notify-event",
-                     G_CALLBACK(textview_leave_notify_event), gui);
+    /* Tooltips */
+    g_signal_connect(G_OBJECT(gui->txtView), "query-tooltip",
+                     G_CALLBACK(worldgui_query_tooltip_cb), gui);
+    g_signal_connect(G_OBJECT(gui->txtViewScroll), "query-tooltip",
+                     G_CALLBACK(worldgui_query_tooltip_cb), gui);
 
     /* Status bar */
     gui->statusbar_box = GTK_BOX(gtk_hbox_new(FALSE, 4));
@@ -510,10 +554,6 @@ free_world_gui(WorldGUI *gui)
   if (gui->line_times) {
     k_circular_queue_free(gui->line_times);
   }
-  worldgui_destroy_tooltip_window(gui);
-  if (gui->tooltip_timeout_id) {
-    g_source_remove(gui->tooltip_timeout_id);
-  }
 
   g_free(gui);
 }
@@ -615,13 +655,6 @@ keypress_cb(GtkWidget   *widget,
   int            direction = 1;
   gdouble        newval;
 
-  /* By personal experience, tooltips are only useful when idle. When the
-     user is typing, they probably do not want them. */
-  worldgui_destroy_tooltip_window(gui);
-  if (gui->tooltip_timeout_id) {
-    g_source_remove(gui->tooltip_timeout_id);
-  }
-
   if (world->print_next_keycode) {
     if (gtk_accelerator_valid(evt->keyval, evt->state)) {
       world->keycode_to_print = gtk_accelerator_name(evt->keyval, evt->state);
@@ -669,7 +702,7 @@ keypress_cb(GtkWidget   *widget,
   case '1': case '2': case '3': case '4': case '5':
   case '6': case '7': case '8': case '9':
     if (evt->state & GDK_MOD1_MASK) { /* Alt+digit */
-      int newpage = evt->keyval - '1';
+      guint newpage = evt->keyval - '1';
 
       if (newpage < g_list_length(open_worlds)) {
         set_focused_world(newpage);
@@ -681,9 +714,16 @@ keypress_cb(GtkWidget   *widget,
   case GDK_Up:
   case GDK_KP_Up:
     direction = 0;
+    /* Fall through */
 
   case GDK_Down:
   case GDK_KP_Down:
+    /* Do nothing if option to ignore these keys is active. */
+    if (world->ignore_up_down_keys) {
+      return FALSE;
+    }
+    /* Do nothing if arrows have been used to move around the multi-line
+       input bar */
     if (world->input_n_lines > 1 && world->cmd_position_changed) {
       return FALSE;
     }
@@ -697,11 +737,17 @@ keypress_cb(GtkWidget   *widget,
 
   case GDK_Page_Down:
   case GDK_KP_Page_Down:
-    adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
+    /* If the scrollback window is visible, move that */
+    if (gtk_paned_get_position(gui->split_pane) > 0) {
+      adjustment = gtk_scrolled_window_get_vadjustment(gui->scrwinScroll);
+    } else {
+      adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
+    }
     newval = gtk_adjustment_get_value(adjustment)
-      + direction*adjustment->page_increment/2;
-    if (newval > (adjustment->upper - adjustment->page_size)) {
-      newval = (adjustment->upper - adjustment->page_size);
+      + direction*gtk_adjustment_get_page_increment(adjustment)/2;
+    if (newval
+        > (gtk_adjustment_get_upper(adjustment) - gtk_adjustment_get_page_size(adjustment))) {
+      newval = gtk_adjustment_get_upper(adjustment) - gtk_adjustment_get_page_size(adjustment);
     }
     gtk_adjustment_set_value(adjustment, newval);
     gtk_adjustment_changed(adjustment);
@@ -712,7 +758,7 @@ keypress_cb(GtkWidget   *widget,
     if (evt->state & GDK_CONTROL_MASK) {
       adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
       gtk_adjustment_set_value(adjustment,
-                               adjustment->upper - adjustment->page_size);
+                               gtk_adjustment_get_upper(adjustment) - gtk_adjustment_get_page_size(adjustment));
       gtk_adjustment_changed(adjustment);
       return TRUE;
     }
@@ -902,16 +948,19 @@ clear_button_cb(GtkButton *button, gpointer data)
   WorldGUI *gui = (WorldGUI *) data;
   World    *world = gui->world;
 
-  simo_combo_box_clear_text(gui->cmbEntry);
-  gtk_widget_grab_focus(GTK_WIDGET(gui->cmbEntry));
-
   if (world) {
+    simo_combo_box_clear_text(gui->cmbEntry);
+
     /* So that up key recalls the previous command */
     last_path = g_strdup_printf("%d", world->cmd_list_size - 1);
     gtk_tree_model_get_iter_from_string(world->cmd_list,
                                         &world->current_cmd,
                                         last_path);
+  } else {
+    gtk_entry_set_text(gui->txtEntry, "");
   }
+
+  gtk_widget_grab_focus(GTK_WIDGET(gui->cmbEntry));
 }
 
 
@@ -924,6 +973,7 @@ configure_gui(WorldGUI *gui, World *world)
 
   fontDesc = pango_font_description_from_string(world->terminalfont);
   gtk_widget_modify_font(GTK_WIDGET(gui->txtView), fontDesc);
+  gtk_widget_modify_font(GTK_WIDGET(gui->txtViewScroll), fontDesc);
   pango_font_description_free(fontDesc);
   fontDesc = pango_font_description_from_string(world->entryfont);
   if (gui->world) {
@@ -942,7 +992,12 @@ configure_gui(WorldGUI *gui, World *world)
                               world->wrap
                                 ? GTK_WRAP_WORD_CHAR
                                 : GTK_WRAP_CHAR);
-
+  gtk_text_view_set_indent(gui->txtView, -world->wrap_indent);
+  gtk_text_view_set_wrap_mode(gui->txtViewScroll,
+                              world->wrap
+                                ? GTK_WRAP_WORD_CHAR
+                                : GTK_WRAP_CHAR);
+  gtk_text_view_set_indent(gui->txtViewScroll, -world->wrap_indent);
 
   /* If this GUI is for a MUD, and not for a KCWin */
   if (gui->world) {
@@ -980,6 +1035,12 @@ configure_gui(WorldGUI *gui, World *world)
 
     /* Spell checking. */
     worldgui_configure_spell(gui);
+
+    /* Tooltips */
+    gtk_widget_set_has_tooltip(GTK_WIDGET(gui->txtView),
+                               gui->world->use_tooltips);
+    gtk_widget_set_has_tooltip(GTK_WIDGET(gui->txtViewScroll),
+                               gui->world->use_tooltips);
   }
 }
 
@@ -1036,10 +1097,6 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
   GtkTextIter  start;
   GtkTextIter  end;
   gchar       *matched_url;
-  GladeXML    *gladexml;
-  GtkWidget   *mnuPopupURL = NULL;
-  GtkWidget   *mnuURLOpen;
-  GtkWidget   *mnuURLCopy;
 
   if (!gtk_text_iter_has_tag(iter, gui->txttag_url)) {
     return FALSE;
@@ -1059,20 +1116,25 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
   if (event->button == 1) {
     menu_url_open(NULL, matched_url);
   } else {
-    gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
-                             "mnuPopupURL", NULL);
-    mnuPopupURL = glade_xml_get_widget(gladexml, "mnuPopupURL");
-    mnuURLOpen  = glade_xml_get_widget(gladexml, "mnuURLOpen");
-    mnuURLCopy  = glade_xml_get_widget(gladexml, "mnuURLCopy");
+    GtkBuilder *ui_builder;
+    GError     *error = NULL;
+    GtkMenu    *mnuPopupURL;
 
-    g_signal_connect(G_OBJECT(mnuURLOpen), "activate",
-                     G_CALLBACK(menu_url_open), matched_url);
-    g_signal_connect(G_OBJECT(mnuURLCopy), "activate",
-                     G_CALLBACK(menu_popup_url_copy), matched_url);
+    ui_builder = gtk_builder_new();
+    if (!gtk_builder_add_from_file(ui_builder,
+                                   get_kildclient_installed_file("mnuPopupURL.ui"),
+                                   &error)) {
+      g_warning(_("Error loading UI from XML file: %s"), error->message);
+      g_error_free(error);
+      return TRUE;
+    }
 
-    g_object_unref(gladexml);
-    gtk_menu_popup(GTK_MENU(mnuPopupURL), NULL, NULL, NULL, NULL,
+    mnuPopupURL = GTK_MENU(gtk_builder_get_object(ui_builder, "mnuPopupURL"));
+    gtk_builder_connect_signals(ui_builder, matched_url);
+
+    gtk_menu_popup(mnuPopupURL, NULL, NULL, NULL, NULL,
                    event->button, event->time);
+    g_object_unref(ui_builder);
   }
 
   return TRUE;
@@ -1081,7 +1143,7 @@ act_if_link(WorldGUI *gui, GtkTextIter *iter, GdkEventButton *event)
 
 #ifndef __WIN32__
 void
-menu_url_open(GtkWidget *widget, char *url)
+menu_url_open(GtkAction *action, char *url)
 {
   char *to_run;
 
@@ -1094,7 +1156,7 @@ menu_url_open(GtkWidget *widget, char *url)
 }
 #else /* defined __WIN32__ */
 void
-menu_url_open(GtkWidget *widget, char *url)
+menu_url_open(GtkAction *action, char *url)
 {
   ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOWNORMAL);
 
@@ -1103,13 +1165,14 @@ menu_url_open(GtkWidget *widget, char *url)
 #endif
 
 
-static
 void
-menu_popup_url_copy(GtkWidget *widget, char *url)
+menu_popup_url_copy(GtkAction *action, char *url)
 {
+  GtkWidget    *widget;
   GdkDisplay   *display;
   GtkClipboard *clipboard;
 
+  widget = GTK_WIDGET(gtk_action_get_proxies(action)->data);
   display = gtk_widget_get_display(GTK_WIDGET(widget));
 
   clipboard = gtk_clipboard_get_for_display(display, GDK_SELECTION_PRIMARY);
@@ -1124,52 +1187,26 @@ menu_popup_url_copy(GtkWidget *widget, char *url)
 
 static
 gboolean
-textview_motion_notify_event(GtkWidget      *text_view,
+textview_motion_notify_event(GtkWidget      *widget,
                              GdkEventMotion *event,
                              gpointer        data)
 {
   WorldGUI    *gui = (WorldGUI *) data;
+  GtkTextView *text_view = GTK_TEXT_VIEW(widget);
   gint         x, y;
-  GtkTextIter  iter;
 
   /*
    * Update the cursor image if the pointer moved.
    */
-  gtk_text_view_window_to_buffer_coords(gui->txtView,
+  gtk_text_view_window_to_buffer_coords(text_view,
                                         GTK_TEXT_WINDOW_WIDGET,
                                         event->x, event->y, &x, &y);
 
-  set_cursor_if_appropriate(gui, x, y);
+  set_cursor_if_appropriate(text_view, gui, x, y);
 
   /* I don't know why this call is here, but I copied this from the
      gtk-demo, and it is necessary. */
-  gdk_window_get_pointer(text_view->window, NULL, NULL, NULL);
-
-
-  /*
-   * Tooltip
-   */
-  gui->tooltip_x = x;
-  gui->tooltip_y = y;
-  if (gui->tooltip_timeout_id) {
-    if ((y > gui->tooltip_line_y)
-        && ((y - gui->tooltip_line_height) < gui->tooltip_line_y)) {
-      return FALSE;
-    }
-    /* We have left the line. Remove timeout, a new one will be created
-       later. */
-    worldgui_destroy_tooltip_window(gui);
-    g_source_remove(gui->tooltip_timeout_id);
-  }
-
-  gtk_text_view_get_line_at_y(gui->txtView, &iter, y, &gui->tooltip_line_y);
-  gtk_text_view_get_line_yrange(gui->txtView, &iter,
-                                &gui->tooltip_line_y,
-                                &gui->tooltip_line_height);
-  gui->tooltip_line_number = gtk_text_iter_get_line(&iter);
-
-  gui->tooltip_timeout_id = g_timeout_add(TOOLTIP_DELAY,
-                                          worldgui_tooltip_timeout, gui);
+  gdk_window_get_pointer(gtk_widget_get_window(widget), NULL, NULL, NULL);
 
   return FALSE;
 }
@@ -1180,22 +1217,23 @@ textview_motion_notify_event(GtkWidget      *text_view,
  */
 static
 gboolean
-textview_visibility_notify_event(GtkWidget          *text_view,
+textview_visibility_notify_event(GtkWidget          *widget,
                                  GdkEventVisibility *event,
                                  gpointer            data)
 {
   /* Also update the cursor image if the window becomes visible
    * (e.g. when a window covering it got iconified). */
-  WorldGUI *gui = (WorldGUI *) data;
+  WorldGUI    *gui       = (WorldGUI *) data;
+  GtkTextView *text_view = GTK_TEXT_VIEW(widget);
   gint      wx, wy, bx, by;
 
-  gdk_window_get_pointer(GTK_WIDGET(gui->txtView)->window, &wx, &wy, NULL);
+  gdk_window_get_pointer(gtk_widget_get_window(widget), &wx, &wy, NULL);
 
-  gtk_text_view_window_to_buffer_coords(gui->txtView,
+  gtk_text_view_window_to_buffer_coords(text_view,
                                         GTK_TEXT_WINDOW_WIDGET,
                                         wx, wy, &bx, &by);
 
-  set_cursor_if_appropriate(gui, bx, by);
+  set_cursor_if_appropriate(text_view, gui, bx, by);
 
   return FALSE;
 }
@@ -1203,25 +1241,26 @@ textview_visibility_notify_event(GtkWidget          *text_view,
 
 static
 void
-set_cursor_if_appropriate(WorldGUI *gui,
-                          gint      x,
-                          gint      y)
+set_cursor_if_appropriate(GtkTextView *text_view,
+                          WorldGUI    *gui,
+                          gint         x,
+                          gint         y)
 {
   GtkTextIter    iter;
   gboolean       hovering = FALSE;
 
-  gtk_text_view_get_iter_at_location(gui->txtView, &iter, x, y);
+  gtk_text_view_get_iter_at_location(text_view, &iter, x, y);
   hovering = gtk_text_iter_has_tag(&iter, gui->txttag_url);
 
   if (hovering != hovering_over_link) {
     hovering_over_link = hovering;
 
     if (hovering_over_link) {
-      gdk_window_set_cursor(gtk_text_view_get_window(gui->txtView,
+      gdk_window_set_cursor(gtk_text_view_get_window(text_view,
                                                      GTK_TEXT_WINDOW_TEXT),
                             hand_cursor);
     } else {
-      gdk_window_set_cursor(gtk_text_view_get_window(gui->txtView,
+      gdk_window_set_cursor(gtk_text_view_get_window(text_view,
                                                      GTK_TEXT_WINDOW_TEXT),
                             regular_cursor);
     }
@@ -1231,142 +1270,57 @@ set_cursor_if_appropriate(WorldGUI *gui,
 
 static
 gboolean
-textview_leave_notify_event(GtkWidget        *text_view,
-                            GdkEventCrossing *event,
-                            gpointer          data)
+worldgui_query_tooltip_cb(GtkWidget  *widget,
+                          gint        win_x,
+                          gint        win_y,
+                          gboolean    keyboard_mode,
+                          GtkTooltip *tooltip,
+                          gpointer    data)
 {
-  WorldGUI *gui = (WorldGUI *) data;
+  WorldGUI      *gui       = (WorldGUI *) data;
+  GtkTextView   *text_view = GTK_TEXT_VIEW(widget);
+  GtkTextIter    iter;
+  gint           tooltip_line_top_y;
+  gint           x, y;
+  GtkAllocation  alloc;
+  GdkRectangle   rect;
+  gchar         *text;
 
-  if (gui->tooltip_timeout_id) {
-    g_source_remove(gui->tooltip_timeout_id);
-    gui->tooltip_timeout_id = 0;
+  /* No world yet? */
+  if (!gui->world) {
+    return FALSE;
   }
 
-  worldgui_destroy_tooltip_window(gui);
-
-  return FALSE;
-}
-
-
-static
-gboolean
-worldgui_tooltip_timeout(gpointer data)
-{
-  WorldGUI    *gui = (WorldGUI *) data;
-  gchar       *linetime;
-  PangoLayout *layout;
-  gint         scr_w, scr_h, w, h, x, y;
-  int          mon_num;
-  GdkScreen   *screen = NULL;
-  GdkRectangle mon_size;
-  gboolean     tooltip_top = FALSE;
-
-  linetime = worldgui_get_tooltip_text(gui);
-
-  if (gui->tooltip_window) {
-    gtk_widget_destroy(gui->tooltip_window);
+  if (keyboard_mode) {
+    return FALSE;
   }
 
-  gui->tooltip_window = gtk_window_new(GTK_WINDOW_POPUP);
-  gtk_widget_set_app_paintable(gui->tooltip_window, TRUE);
-  gtk_window_set_resizable(GTK_WINDOW(gui->tooltip_window), FALSE);
-  gtk_widget_set_name(gui->tooltip_window, "gtk-tooltips");
-  g_signal_connect(G_OBJECT(gui->tooltip_window), "expose_event",
-                   G_CALLBACK(worldgui_paint_tooltip_cb), gui);
-  gtk_widget_ensure_style(gui->tooltip_window);
+  /* Determine are for which the tooltip is valid - this corresponds
+     to the area of the buffer line. */
+  gtk_text_view_window_to_buffer_coords(text_view,
+                                        GTK_TEXT_WINDOW_WIDGET,
+                                        win_x, win_y, &x, &y);
+  gtk_text_view_get_line_at_y(text_view, &iter, y, NULL);
+  gtk_text_view_get_line_yrange(text_view, &iter,
+                                &tooltip_line_top_y,
+                                &rect.height);
+  gtk_text_view_buffer_to_window_coords(text_view,
+                                        GTK_TEXT_WINDOW_WIDGET,
+                                        0, tooltip_line_top_y,
+                                        &rect.x, &rect.y);
+#if GTK_MINOR_VERSION < 18
+  alloc = widget->allocation;
+#else
+  gtk_widget_get_allocation(widget, &alloc);
+#endif
+  rect.width = alloc.width;
 
-  layout = gtk_widget_create_pango_layout(gui->tooltip_window, NULL);
-  pango_layout_set_wrap(layout, PANGO_WRAP_WORD);
-  pango_layout_set_width(layout, 300000);
-  pango_layout_set_text(layout, linetime, -1);
-  pango_layout_get_size(layout, &w, &h);
+  gtk_tooltip_set_tip_area(tooltip, &rect);
 
-  /* Code to determine size and copied from Gaim (just like the rest
-     of this tooltip code, by the way, but this is more mystical and
-     wasn't touched). */
-  gdk_display_get_pointer(gdk_display_get_default(), &screen, &x, &y, NULL);
-  mon_num = gdk_screen_get_monitor_at_point(screen, x, y);
-  gdk_screen_get_monitor_geometry(screen, mon_num, &mon_size);
-
-  scr_w = mon_size.width + mon_size.x;
-  scr_h = mon_size.height + mon_size.y;
-
-  w = PANGO_PIXELS(w) + 8;
-  h = PANGO_PIXELS(h) + 8;
-
-  if (w > mon_size.width)
-    w = mon_size.width - 10;
-
-  if (h > mon_size.height)
-    h = mon_size.height - 10;
-
-  //if (GTK_WIDGET_NO_WINDOW(gtkblist->window))
-  //  y += gtkblist->window->allocation.y;
-
-  x -= ((w >> 1) + 4);
-
-  if ((y + h + 4) > scr_h || tooltip_top)
-    y = y - h - 5;
-  else
-    y = y + 6;
-
-  if (y < mon_size.y)
-    y = mon_size.y;
-
-  if (y != mon_size.y) {
-    if ((x + w) > scr_w)
-      x -= (x + w + 5) - scr_w;
-    else if (x < mon_size.x)
-      x = mon_size.x;
-  } else {
-    x -= (w / 2 + 10);
-    if (x < mon_size.x)
-      x = mon_size.x;
-  }
-
-  g_free(linetime);
-  g_object_unref(layout);
-  gtk_widget_set_size_request(gui->tooltip_window, w, h);
-  gtk_window_move(GTK_WINDOW(gui->tooltip_window), x, y);
-  gtk_widget_show(gui->tooltip_window);
-
-  return FALSE;
-}
-
-
-static
-gboolean
-worldgui_paint_tooltip_cb(GtkWidget      *widget,
-                          GdkEventExpose *event,
-                          gpointer        data)
-{
-  WorldGUI    *gui = (WorldGUI *) data;
-  gchar       *linetime;
-  PangoLayout *layout;
-  GtkStyle    *style;
-
-  linetime = worldgui_get_tooltip_text(gui);
-
-  layout = gtk_widget_create_pango_layout(gui->tooltip_window, NULL);
-  pango_layout_set_wrap(layout, PANGO_WRAP_WORD);
-  pango_layout_set_width(layout, 300000);
-  pango_layout_set_text(layout, linetime, -1);
-  style = gui->tooltip_window->style;
-
-  gtk_paint_flat_box(style,
-                     gui->tooltip_window->window,
-                     GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-                     NULL, gui->tooltip_window, "tooltip",
-                     0, 0, -1, -1);
-
-  gtk_paint_layout(style,
-                   gui->tooltip_window->window,
-                   GTK_STATE_NORMAL, FALSE,
-                   NULL, gui->tooltip_window, "tooltip",
-                   4, 4, layout);
-
-  g_free(linetime);
-  g_object_unref(layout);
+  /* Get the tooltip text */
+  text = worldgui_get_tooltip_text(gui, text_view, x, y);
+  gtk_tooltip_set_text(tooltip, text);
+  g_free(text);
 
   return TRUE;
 }
@@ -1374,18 +1328,20 @@ worldgui_paint_tooltip_cb(GtkWidget      *widget,
 
 static
 gchar *
-worldgui_get_tooltip_text(WorldGUI *gui)
+worldgui_get_tooltip_text(WorldGUI    *gui,
+                          GtkTextView *text_view,
+                          gint         x,
+                          gint         y)
 {
   GtkTextIter  iter;
   gchar       *tip;
 
-  gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(gui->txtView), &iter,
-                                     gui->tooltip_x, gui->tooltip_y);
-
+  gtk_text_view_get_iter_at_location(text_view, &iter, x, y);
   if (gtk_text_iter_has_tag(&iter, gui->txttag_url)) {
     tip = g_strdup(_("Click to open link; right-click for more options."));
   } else {
-    size_t    len;
+    gint   tooltip_line_number;
+    size_t len;
 #ifdef HAVE_LOCALTIME_R
     struct tm time_tm;
 #endif
@@ -1393,21 +1349,23 @@ worldgui_get_tooltip_text(WorldGUI *gui)
 
     tip = g_malloc(MAX_BUFFER * sizeof(gchar));
 
+    tooltip_line_number = gtk_text_iter_get_line(&iter);
+
     g_snprintf(tip, MAX_BUFFER,
                "Line %ld, ",
-               gui->tooltip_line_number + gui->world->deleted_lines + 1);
+               tooltip_line_number + gui->world->deleted_lines + 1);
     len = strlen(tip);
 
 #ifdef HAVE_LOCALTIME_R
     localtime_r(&k_circular_queue_nth(gui->line_times,
                                       time_t,
-                                      gui->tooltip_line_number),
+                                      tooltip_line_number),
                 &time_tm);
     time_tmp = &time_tm;
 #else
     time_tmp = localtime(&k_circular_queue_nth(gui->line_times,
                                                time_t,
-                                               gui->tooltip_line_number));
+                                               tooltip_line_number));
 #endif
 
     strftime(tip + len, MAX_BUFFER - len, "%c", time_tmp);
@@ -1417,41 +1375,53 @@ worldgui_get_tooltip_text(WorldGUI *gui)
 }
 
 
-void
-worldgui_destroy_tooltip_window(WorldGUI *gui)
-{
-  if (gui->tooltip_window == NULL) {
-    return;
-  }
-
-  gtk_widget_destroy(gui->tooltip_window);
-  gui->tooltip_window = NULL;
-}
-
-
 static
 void
 textview_size_allocate_cb(GtkWidget     *widget,
                           GtkAllocation *allocation,
                           gpointer       data)
 {
-  WorldGUI *gui   = (WorldGUI *) data;
-  World    *world = gui->world;
+  WorldGUI     *gui   = (WorldGUI *) data;
+  World        *world = gui->world;
+  GdkRectangle  rect;
+
+  gtk_text_view_get_visible_rect(gui->txtView, &rect);
 
   /* Some checks to prevent send_naws_size() from being called when
      world is invalid. */
   if (world && world->gui && world->gui == gui
       && world->connected && world->use_naws == TRUE) {
-    GdkRectangle rect;
-
     /* See if the size has changed. */
-    gtk_text_view_get_visible_rect(gui->txtView, &rect);
     if (rect.height != world->last_naws_size.height
         || rect.width != world->last_naws_size.width) {
       send_naws_size(world);
       world->last_naws_size = rect;
     }
   }
+
+  /* Dealing with the wrap column:
+     If the user shrinks the window so that the view is too narrow to
+     show wrap_column columns, then we wrap normally. If it's wider,
+     we want to keep the wrap at wrap_column, and show blank white
+     space for the rest. To do thath, we pad the view's right-margin
+     with whatever space is left over. */
+  if (world) {
+    if (!world->use_wrap_column || world->wrap_column < 40) {
+      /* No wrap column set */
+      gtk_text_view_set_right_margin(gui->txtView, 0);
+      gtk_text_view_set_right_margin(gui->txtViewScroll, 0);
+    } else {
+      gint char_width, char_height;
+      gint wrap_width;
+      gint margin;
+
+      ansitextview_get_char_size(gui, &char_height, &char_width);
+      wrap_width = char_width * (world->wrap_column + 1);
+      margin = rect.width > wrap_width ? (rect.width - wrap_width) : 0;
+      gtk_text_view_set_right_margin(gui->txtView, margin);
+      gtk_text_view_set_right_margin(gui->txtViewScroll, margin);
+    }
+  }
 }
 
 
@@ -1515,8 +1485,14 @@ do_find(WorldGUI *gui)
                                    &match_end,
                                    NULL)) {
     gtk_text_buffer_select_range(gui->txtBuffer, &match_start, &match_end);
-    gtk_text_view_scroll_to_iter(gui->txtView, &match_start,
-                                 0, FALSE, 0, 0);
+    /* If split pane is active, scroll that plane */
+    if (gtk_paned_get_position(gui->split_pane) > 0) {
+      gtk_text_view_scroll_to_iter(gui->txtViewScroll, &match_start,
+                                   0, FALSE, 0, 0);
+    } else {
+      gtk_text_view_scroll_to_iter(gui->txtView, &match_start,
+                                   0, FALSE, 0, 0);
+    }
 
     /* Mark point at which Find Next will start */
     gtk_text_iter_forward_char(&match_start);
@@ -1554,3 +1530,79 @@ search_box_keypress_cb(GtkWidget   *widget,
 
   return FALSE;
 }
+
+
+static
+void
+scrolled_win_size_allocate_cb(GtkWidget     *widget,
+                              GtkAllocation *allocation,
+                              gpointer       data)
+{
+  /* When the bottom TextView (the main one) is resized, we want its
+     top part to shrink/grow and the bottom part to remain displaying
+     what it was displaying before.
+
+     To do that, the value of (upper - (value + page_size)) of the
+     ScrolledWindow's Adjustment must remain the same. This value
+     corresponds to the size of the part of the TextView that is not
+     displayed and is below the displayed part.
+
+     upper never changes while scrolling, so we only need to make sure
+     that (value + page_size) remains constant. To do that, so we
+     store this value at the end of the function so that in the next
+     call the previous value is know so that the new position of value
+     is calculated.
+
+     Whenever value changes, this stored value is also updated.
+  */
+  WorldGUI      *gui = (WorldGUI *) data;
+  GtkAdjustment *verticalAdjust;
+  double         page_size, upper;
+  double         newValue;
+
+  verticalAdjust
+    = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(widget));
+  upper = gtk_adjustment_get_upper(verticalAdjust);
+  page_size = gtk_adjustment_get_page_size(verticalAdjust);
+
+  /* First call */
+  //if (gui->last_scroll_info == -1) {
+  //  gui->last_scroll_info = page_size;
+  //  return;
+  //}
+
+  newValue = gui->last_scroll_info - page_size;
+  if (newValue < 0) {
+    newValue = 0;
+  }
+  if (newValue > (upper - page_size)) {
+    newValue = upper - page_size;
+  }
+
+  gtk_adjustment_set_value(verticalAdjust, newValue);
+
+  gui->last_scroll_info = newValue + page_size;
+}
+
+
+static
+void
+scrolled_win_value_changed_cb(GtkAdjustment *adjustment,
+                              gpointer       data)
+{
+  WorldGUI *gui = (WorldGUI *) data;
+
+  gui->last_scroll_info = gtk_adjustment_get_value(adjustment)
+                          + gtk_adjustment_get_page_size(adjustment);
+}
+
+
+static
+gboolean
+set_split_to_zero(gpointer data)
+{
+  WorldGUI *gui = (WorldGUI *) data;
+
+  gtk_paned_set_position(gui->split_pane, 0);
+  return FALSE;
+}
