@@ -1,7 +1,7 @@
-/* $Id: worldselector.c 1019 2009-02-03 19:31:11Z ekalin $ */
+/* $Id: worldselector.c 1143 2011-02-13 16:13:58Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
+ * Copyright (C) 2004-2011 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #endif
 #include <libintl.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
 #include "kildclient.h"
 #include "perlscript.h"
@@ -52,12 +51,11 @@ enum {
 /*************************
  * File global variables *
  *************************/
-GtkWidget *dlgSelectWorld;
-GtkWidget *radFromList;
-GtkWidget *radManually;
-GtkWidget *txtHost;
-GtkWidget *txtPort;
-GtkWidget *lstWorlds;
+GtkWidget       *dlgSelectWorld;
+GtkToggleButton *radManually;
+GtkEntry        *txtHost;
+GtkEntry        *txtPort;
+GtkTreeView     *lstWorlds;
 
 
 /***********************
@@ -81,16 +79,17 @@ static gint          sort_by_name(GtkTreeModel *model,
 static char         *try_find_world_from_name(const gchar *name);
 static char         *compare_world_from_name(const gchar *file,
                                              const gchar *name);
-static void          set_manually_cb(GtkEditable *widget,
-                                     gpointer user);
 static void          list_selection_changed_cb(GtkTreeSelection *selection,
                                                gpointer data);
-static void          list_row_activated_cb(GtkTreeView       *treeview,
-                                           GtkTreePath       *treepath,
-                                           GtkTreeViewColumn *column,
-                                           gpointer           data);
 static void          add_edit_buttons_cb(GtkButton *btn, gpointer data);
-static void          delete_button_cb(GtkButton *btn, gpointer data);
+/* XML UI signals */
+G_MODULE_EXPORT void set_manually_cb(GtkEditable *widget,
+                                     gpointer user);
+G_MODULE_EXPORT void list_row_activated_cb(gpointer           data,
+                                           GtkTreeView       *treeview,
+                                           GtkTreePath       *treepath,
+                                           GtkTreeViewColumn *column);
+G_MODULE_EXPORT void delete_button_cb(GtkButton *btn, gpointer data);
 
 
 
@@ -98,41 +97,31 @@ World *
 get_world_to_connect()
 {
   GtkWidget *dlgWorldSelector;
-  World *wrld = NULL;
-  gboolean success;
+  World     *world = NULL;
+  gboolean   success;
 
   dlgWorldSelector = create_world_selector_dlg(GTK_WINDOW(wndMain));
   gtk_widget_show_all(dlgWorldSelector);
-  gtk_entry_set_text(GTK_ENTRY(txtHost), "");
-  gtk_entry_set_text(GTK_ENTRY(txtPort), "");
 
   success = FALSE;
   while (!success) {
     if (gtk_dialog_run(GTK_DIALOG(dlgWorldSelector)) == GTK_RESPONSE_OK) {
-      const gchar *portstr;
+      world = create_new_world(TRUE);
+      world->connected = FALSE;
 
-      wrld = create_new_world(TRUE);
-      wrld->connected = FALSE;
-
-      if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radManually))) {
-        wrld->name = g_strdup(_("Unnamed world"));
-        /* Since there is never a character in this case, it makes no
-           sense to use the default world settings for display name. */
-        wrld->display_name = g_strdup(wrld->name);
-        wrld->new_text_name =
-          g_strdup_printf("<span foreground=\"red\">%s</span>",
-                          wrld->display_name);
-        wrld->host = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtHost)));
-        portstr = gtk_entry_get_text(GTK_ENTRY(txtPort));
-        wrld->port = g_strdup(portstr);
-        wrld->character_used = NULL;    /* No auto-logon */
+      if (gtk_toggle_button_get_active(radManually)) {
+        create_world_from_parameters(world,
+                                     g_strdup(gtk_entry_get_text(txtHost)),
+                                     g_strdup(gtk_entry_get_text(txtPort)),
+                                     NULL,
+                                     NULL);
         success = TRUE;
       } else {
         GtkTreeSelection *selection;
         GtkTreeModel     *model;
         GtkTreeIter       iter;
 
-        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lstWorlds));
+        selection = gtk_tree_view_get_selection(lstWorlds);
         if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
           char   *file;
           gint    charidx;
@@ -142,7 +131,7 @@ get_world_to_connect()
                              COLUMN_FILE, &file,
                              COLUMN_INDEX, &charidx,
                              -1);
-          success = load_world_from_file(file, wrld, FALSE, &error);
+          success = load_world_from_file(file, world, FALSE, &error);
           if (!success) {
             GtkWidget *msgdlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
                                                        GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -154,19 +143,19 @@ get_world_to_connect()
             gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Error"));
             gtk_dialog_run(GTK_DIALOG(msgdlg));
             gtk_widget_destroy(msgdlg);
-            free_world(wrld);
-            wrld = NULL;
+            free_world(world);
+            world = NULL;
           } else {
             if (charidx == -1) {
-              wrld->character_used = NULL;
+              world->character_used = NULL;
             } else {
               GtkTreePath         *path;
               GtkTreeRowReference *ref;
 
               path = gtk_tree_path_new_from_indices(charidx, -1);
-              ref  = gtk_tree_row_reference_new(wrld->logon_characters, path);
+              ref  = gtk_tree_row_reference_new(world->logon_characters, path);
 
-              wrld->character_used = ref;
+              world->character_used = ref;
               gtk_tree_path_free(path);
             }
           }
@@ -179,7 +168,7 @@ get_world_to_connect()
   }
   gtk_widget_destroy(dlgWorldSelector);
 
-  return wrld;
+  return world;
 }
 
 
@@ -187,14 +176,11 @@ static
 GtkWidget*
 create_world_selector_dlg(GtkWindow* parent)
 {
-  GtkWidget *scrolledWin;
-  GtkWidget *btnbox;
-  GtkWidget *btnNew;
-  GtkWidget *btnEdit;
-  GtkWidget *btnDelete;
-  GtkWidget *table;
-  GtkWidget *lblHost;
-  GtkWidget *lblPort;
+  GtkBuilder *ui_builder;
+  GError     *error = NULL;
+  GObject    *radFromList;
+  GObject    *btnNew;
+  GObject    *btnEdit;
 
   GtkTreeStore      *model;
   GtkCellRenderer   *renderer;
@@ -202,31 +188,32 @@ create_world_selector_dlg(GtkWindow* parent)
   GtkTreeSelection  *selection;
   GtkTreeIter       *last_open_iter = NULL;
 
-  dlgSelectWorld = gtk_dialog_new_with_buttons(_("Connect to"),
-                                               parent,
-                                               GTK_DIALOG_MODAL
-                                               | GTK_DIALOG_DESTROY_WITH_PARENT
-                                               | GTK_DIALOG_NO_SEPARATOR,
-                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CLOSE,
-                                               _("C_onnect"), GTK_RESPONSE_OK,
-                                               NULL);
+  ui_builder = gtk_builder_new();
+  if (!gtk_builder_add_from_file(ui_builder,
+                                 get_kildclient_installed_file("dlgSelectWorld.ui"),
+                                 &error)) {
+    g_warning(_("Error loading UI from XML file: %s"), error->message);
+    g_error_free(error);
+    exit(1); /* No point in continuing */
+  }
+
+  dlgSelectWorld = GTK_WIDGET(gtk_builder_get_object(ui_builder,
+                                                     "dlgSelectWorld"));
+  gtk_window_set_transient_for(GTK_WINDOW(dlgSelectWorld), parent);
   gtk_dialog_set_default_response(GTK_DIALOG(dlgSelectWorld), GTK_RESPONSE_OK);
-  gtk_container_set_border_width(GTK_CONTAINER(dlgSelectWorld), 8);
 
-  radFromList = gtk_radio_button_new_with_mnemonic(NULL,
-                                                   _("_Select a saved world:"));
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox),
-                     radFromList,
-                     FALSE, FALSE, 4);
-
-  scrolledWin = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledWin),
-                                      GTK_SHADOW_IN);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledWin),
-                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  /* Set global variables pointing to widgets */
+  radFromList = gtk_builder_get_object(ui_builder, "radFromList");
+  radManually = GTK_TOGGLE_BUTTON(gtk_builder_get_object(ui_builder,
+                                                         "radManually"));
+  txtHost = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtHost"));
+  txtPort = GTK_ENTRY(gtk_builder_get_object(ui_builder, "txtPort"));
 
+  /* Prepare TreeView with the Worlds */
   model = create_saved_worlds_model(&last_open_iter);
-  lstWorlds = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
+  lstWorlds = GTK_TREE_VIEW(gtk_builder_get_object(ui_builder,
+                                                   "lstWorlds"));
+  gtk_tree_view_set_model(lstWorlds, GTK_TREE_MODEL(model));
   g_object_unref(G_OBJECT(model));
   renderer = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes(NULL,
@@ -234,102 +221,34 @@ create_world_selector_dlg(GtkWindow* parent)
                                                     "text",
                                                     COLUMN_NAME,
                                                     NULL);
-  gtk_tree_view_append_column(GTK_TREE_VIEW(lstWorlds), column);
-  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(lstWorlds), FALSE);
-  gtk_widget_set_size_request(lstWorlds, 200, 175);
+  gtk_tree_view_append_column(lstWorlds, column);
 
-  gtk_container_add(GTK_CONTAINER(scrolledWin), lstWorlds);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox), scrolledWin,
-                     TRUE, TRUE, 4);
-
-  g_signal_connect(G_OBJECT(lstWorlds), "row-activated",
-                   G_CALLBACK(list_row_activated_cb), NULL);
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lstWorlds));
+  /* Pre select last used world */
+  selection = gtk_tree_view_get_selection(lstWorlds);
   if (last_open_iter) {
     GtkTreePath *path;
 
     gtk_tree_selection_select_iter(selection, last_open_iter);
     path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), last_open_iter);
-    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(lstWorlds),
+    gtk_tree_view_scroll_to_cell(lstWorlds,
                                  path, NULL,
                                  FALSE, 0, 0);
     gtk_tree_path_free(path);
   }
+
+  /* Connect signals that can't be specified in the XML UI file */
   g_signal_connect(G_OBJECT(selection), "changed",
-                   G_CALLBACK(list_selection_changed_cb), NULL);
-
-  btnbox = gtk_hbox_new(TRUE, 8);
-
-  btnNew = gtk_button_new_from_stock(GTK_STOCK_NEW);
-  g_signal_connect(G_OBJECT(btnNew), "clicked",
+                   G_CALLBACK(list_selection_changed_cb), radFromList);
+  btnNew = gtk_builder_get_object(ui_builder, "btnNew");
+  g_signal_connect(btnNew, "clicked",
                    G_CALLBACK(add_edit_buttons_cb), GINT_TO_POINTER(0));
-  gtk_box_pack_start(GTK_BOX(btnbox), btnNew, TRUE, TRUE, 0);
-
-  btnEdit = gtk_button_new_with_mnemonic(_("_Edit"));
-  g_signal_connect(G_OBJECT(btnEdit), "clicked",
+  btnEdit = gtk_builder_get_object(ui_builder, "btnEdit");
+  g_signal_connect(btnEdit, "clicked",
                    G_CALLBACK(add_edit_buttons_cb), GINT_TO_POINTER(1));
-  gtk_box_pack_start(GTK_BOX(btnbox), btnEdit, TRUE, TRUE, 0);
 
-  btnDelete = gtk_button_new_from_stock(GTK_STOCK_DELETE);
-  g_signal_connect(G_OBJECT(btnDelete), "clicked",
-                   G_CALLBACK(delete_button_cb), NULL);
-  gtk_box_pack_start(GTK_BOX(btnbox), btnDelete, TRUE, TRUE, 0);
+  gtk_builder_connect_signals(ui_builder, NULL);
+  g_object_unref(ui_builder);
 
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox),
-                     btnbox, TRUE, TRUE, 8);
-
-
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox),
-                     gtk_hseparator_new(), TRUE, TRUE, 8);
-  radManually =
-    gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radFromList),
-                                                   _("Or connect d_irectly:"));
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox),
-                     radManually,
-                     FALSE, FALSE, 4);
-
-  table = gtk_table_new(2, 2, FALSE);
-
-  lblHost = gtk_label_new_with_mnemonic(_("_Host:"));
-  gtk_table_attach(GTK_TABLE(table), lblHost,
-                   0, 1, 0, 1,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   4, 4);
-
-  txtHost = gtk_entry_new();
-  gtk_label_set_mnemonic_widget(GTK_LABEL(lblHost), txtHost);
-  gtk_entry_set_activates_default(GTK_ENTRY(txtHost), TRUE);
-  g_signal_connect(G_OBJECT(txtHost), "changed",
-                   G_CALLBACK(set_manually_cb), NULL);
-  gtk_table_attach(GTK_TABLE(table), txtHost,
-                   1, 2, 0, 1,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   4, 4);
-
-  lblPort = gtk_label_new_with_mnemonic(_("_Port:"));
-  gtk_table_attach(GTK_TABLE(table), lblPort,
-                   0, 1, 1, 2,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   4, 4);
-
-  txtPort = gtk_entry_new();
-  gtk_label_set_mnemonic_widget(GTK_LABEL(lblPort), txtPort);
-  gtk_entry_set_activates_default(GTK_ENTRY(txtPort), TRUE);
-  g_signal_connect(G_OBJECT(txtPort), "changed",
-                   G_CALLBACK(set_manually_cb), NULL);
-  gtk_table_attach(GTK_TABLE(table), txtPort,
-                   1, 2, 1, 2,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-                   4, 4);
-
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgSelectWorld)->vbox), table,
-                     TRUE, TRUE, 4);
-
-  gtk_widget_grab_focus(lstWorlds);
   return dlgSelectWorld;
 }
 
@@ -482,7 +401,7 @@ add_saved_worlds_model_entries(GtkTreeStore *model,
                      -1);
 
   /* Add sub-entries for each of the characters */
-  if (n_chars > 1) {
+  if (n_chars > 0) {
     GtkTreeIter chariter;
     GtkTreeIter subworlditer;
     gint        idx;
@@ -555,12 +474,13 @@ sort_by_name(GtkTreeModel *model,
 
 
 World *
-get_world_from_name(const gchar *name)
+get_world_from_cmdline(const gchar *cmdline)
 {
-  gchar *world_file;
-  World *world;
+  gchar  *world_file;
+  World  *world = NULL;
+  static GRegex *parameters_regex = NULL;
 
-  world_file = try_find_world_from_name(name);
+  world_file = try_find_world_from_name(cmdline);
 
   if (world_file) {
     GError *error = NULL;
@@ -571,19 +491,55 @@ get_world_from_name(const gchar *name)
                              FALSE,
                              &error)) {
       g_free(world_file);
-      return world;
     } else {
       g_free(world_file);
       fprintf(stderr, _("Could not open world '%s': %s\n"),
-              name, error->message);
+              cmdline, error->message);
       free_world(world);
-      return NULL;
+    }
+  } else {
+    GMatchInfo *matchinfo;
+    gchar      *host;
+    gchar      *port;
+    gchar      *user;
+    gchar      *password;
+
+    GError *error;
+    if (parameters_regex == NULL) {
+      parameters_regex = g_regex_new(
+        "^(telnet://)?"
+          "((?<user>[a-zA-Z0-9_.+$!*'()\"-]+)"
+           ":(?<password>[a-zA-Z0-9_.+$!*'()\"-]+)"
+           "@)?"
+          "(?<host>[a-zA-Z0-9.-]+)"
+          "(:(?<port>\\d+))?"
+          "/?$",
+        G_REGEX_NO_AUTO_CAPTURE,
+        0,
+        &error);
+      if (error)
+        fprintf(stderr, error->message);
+    }
+
+    if (g_regex_match(parameters_regex,
+                      cmdline,
+                      0,
+                      &matchinfo)) {
+      host     = g_match_info_fetch_named(matchinfo, "host");
+      port     = g_match_info_fetch_named(matchinfo, "port");
+      user     = g_match_info_fetch_named(matchinfo, "user");
+      password = g_match_info_fetch_named(matchinfo, "password");
+
+      g_match_info_free(matchinfo);
+      world = create_new_world(TRUE);
+      create_world_from_parameters(world, host, port, user, password);
+    } else {
+      g_match_info_free(matchinfo);
+      fprintf(stderr, _("Invalid argument '%s'\n"), cmdline);
     }
   }
 
-  fprintf(stderr, _("Could not find a world named '%s'\n"),
-          name);
-  return NULL;
+  return world;
 }
 
 
@@ -673,7 +629,6 @@ compare_world_from_name(const gchar *file, const gchar *name)
 }
 
 
-static
 void
 set_manually_cb(GtkEditable *widget,
                 gpointer user)
@@ -688,20 +643,17 @@ void
 list_selection_changed_cb(GtkTreeSelection *selection,
                           gpointer data)
 {
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radFromList),
-                               TRUE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), TRUE);
 }
 
 
-static
 void
-list_row_activated_cb(GtkTreeView       *treeview,
+list_row_activated_cb(gpointer           data,
+                      GtkTreeView       *treeview,
                       GtkTreePath       *treepath,
-                      GtkTreeViewColumn *column,
-                      gpointer           data)
+                      GtkTreeViewColumn *column)
 {
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radFromList),
-                               TRUE);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data), TRUE);
   gtk_dialog_response(GTK_DIALOG(dlgSelectWorld), GTK_RESPONSE_OK);
 }
 
@@ -720,7 +672,7 @@ add_edit_buttons_cb(GtkButton *btn, gpointer data)
     char             *file;
     GError           *error = NULL;
 
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lstWorlds));
+    selection = gtk_tree_view_get_selection(lstWorlds);
     if (!gtk_tree_selection_get_selected(selection, &model, &iter))
       return;
 
@@ -747,7 +699,7 @@ add_edit_buttons_cb(GtkButton *btn, gpointer data)
 
     make_startup_plugin_list_for_editing(world);
   } else {
-    model = gtk_tree_view_get_model(GTK_TREE_VIEW(lstWorlds));
+    model = gtk_tree_view_get_model(lstWorlds);
   }
 
   if (edit_world(&world, GTK_WINDOW(dlgSelectWorld), &newworld, TRUE)) {
@@ -781,7 +733,6 @@ add_edit_buttons_cb(GtkButton *btn, gpointer data)
 }
 
 
-static
 void
 delete_button_cb(GtkButton *btn, gpointer data)
 {
@@ -794,7 +745,7 @@ delete_button_cb(GtkButton *btn, gpointer data)
   GtkWidget        *msgdlg;
   gchar            *msg;
 
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lstWorlds));
+  selection = gtk_tree_view_get_selection(lstWorlds);
   if (!gtk_tree_selection_get_selected(selection, &model, &iter))
     return;
 
